      Program calruc
C ---------------------------------------------------------------------
C --- CALRUC   Version: 2.21 Level: 100525                       Main
c
c     Copyright (c) 2010 by Exponent, Inc. 
C ---------------------------------------------------------------------
C --- CALRUC -- Create 3D.dat files from RUC model output in grib format
c
c --- UPDATES:
c
c ---   Version 2.21 Level 100716 from Version 2.1 Level 100525
c       (Note: no version 2.2 exist for calruc.f)
c       1. Add year 2010 
c         
c       Zhong Wu
C       7/16/2010
c
c ---   Version 2.1 Level 100525 from Version 2.01 Level 100111
c       1. Set outliers to their maximum to resolve some very 
C          large mixing ratios in 2009 RUC grib files.  These values
C          are only for packing purpose, and should not be used as actual
C          values. 
C
C       Zhong Wu
C       5/25/2010

c ---   Version 2.01 Level 091014 from Version 2.0 Level 090519
c       1. Relax the Year checks to allow 2009 data
c
c       Zhong Wu
c       10/14/2009
c       
c ---   Version 2.0 Level 090519 from Version 1.971 Level 090511
c       1. Keep negative terrain and GH to account for negative terrains 
C          for RUC data June 11 18:00 and after.
C       2. Separate 2008 into two periods.
C       3.  Change 3D.DAT data format from: 
C          Version 2.1 to Version 2.13
C          (Note: Data Version 2.11 was used for Tiled RUC 3D.DAT files 
C                with optional blank space for missing precip, radiation,
C                and SST. Version 2.12 was used in CALMM5 Version 2.9
C                Level 080102 to eleminate inconsistancy between 
C                the interpolated dot terrain) 
c
c ---   Version 1.971 Level 090511 from Version 1.97 Level 080602
c       1. Update CALUTILS from V2.56 Level 080407 to V2.571 Level 090511
c          (No changes are made to subroutines used in CALRUC)
c        - Add routine to reformat a date string

c          New     : FMT_DATE
c        - Increase control file line length to 200 characters
c          Modified: PARAMS.CAL, READIN
c        - Activate CPU clock using F95 system routine
c          Modified: DATETM
c
c ---   Version 1.97 Level 080602 from Version 1.961 Level 080429
c       1. Add subroutine begend to determine limit of time period
c          due to terrain changes in RUC, and corresponding terrain file
c       Zhong-Xiang Wu
c       6/2/2008

c ---   Version 1.961 Level 080429 from Version 1.96 Level 061130
c       1. Update CALUTILS from V2.54 Level 061020 to V2.56 Level 080407
c        - String entries in exponential notation were not correct
c          if decimal point was missing (2e-02 was read as 0.2e-02).
c          Modified: ALTONU (not used in CALRUC)
c        - Format for output time zone string in BASRUTC wrote zone zero
c          as 'UTC+0  0' instead of 'UTC+0000'; add RETURN statement
c          Modified: BASRUTC, UTCBASR (not used in CALRUC)
C       DGS, 4/29/2008

C ---   Changes to Version 1.96 Level 061130 from Version 1.95, Level 061030
C       1. Use gethours for total hours to account for leap years
C       2. Take off "iswou,ilwou" on Recorder #5 for consistency for RUCDECODE
C       Zhong Wu
C       11/30/2006
C
C ---   Changes to Version 1.95 Level 061030 from Version 1.94, Level 060626
C       1. Update CALUTILS from V2.2 Level 030508 to V2.54 Level 061020
C       2. QA checks but continue to procss the file without stop 
C          according JSS and Al (used in V1.931, Level 060824).
C       Zhong Wu
C       10/30/2006

C ---   Changes to Version 1.94 Level 060626 from Version 1.93, Level 060316 
C       1. Fix bugs for creating 3D.DAT from RUC isobaric level grib files
C       2. Add flag for RUC downloaded files (need now calruc.inp file)
C       3. Coded for NWS and NCEP GRIB files
C 
c ---   Zhong Wu
C       10/30/2006

C ---   Changes to Version 1.93 Level 060316 from Version 1.92, Level 050707 
C       1. Add option to check vertical profiles for catching 
C          unreasonable values (Pr,GH,VPT,Humid, U, V,WP only)
C       2. Stop processing for crached grib files
C
c ---   Zhong Wu
C       3/16/2006
C
C ---   Version 1.92, Level 050707 
c ---   Mahesh Phadnis
c ---   Made changes to the 40km loop. Everything was mostly the same
c ---   except, the grid size and the fixed field reads. For the 40 km
c ---   data, vegetation type is unavailable. Those values were
c ---   initialized to zero.
c ---   Changed the RUC20 file from ruc20new.dat to ruc20.dat

C ---   Version 1.91, Level 050511 from Version 1.9, Level 050217 (Minor)
C       1. Change CVER='1.8' and LEVEL=050204' in Version 1.9 to 
C          CVER='1.91' and  CLEVEL='050511'.
C       Modified by Zhong-Xiang Wu
C       5/11/2005
C---
C---    Version 1.9 Level 050217
C---    Increased geopotential heights by 5 m
C---    Version 1.8 Level 050204
C---    Fixed a bug in the extraction - flat and flon for 10 m winds
C---    Version 1.7 Level 050201
C---    Update first level data at 5 meters
C---    Fixed a bug in the extraction - level 10 was replaced by 
C       the 10m data
C---    Changed the specific humidity to one from the input data itself
C---    version 1.6. Fixed just to read the hybrid data
C ---   Version 1.6 Level 041214
C ---	Update grid information to include vegetation type
C ---	Update data header records according to Zhong's write up
C ---   Version 1.5 Level 041118
C ---	Use 10 m wind data for level 1 in 3D.DAT file
C 	Version 1.4 Level 040916
C ---	Added Cloud, Rain, Ice, Snow and Graupel Mixing Ratios to 3D.dat file
C ---   Format changed from F5.2 to F6.3
C ---	Added Compression in the File with a compression flag in the Input File
C
C 	Version 1.3 Level 040902
C ---	Added Cloud, Rain, Ice, Snow and Graupel Mixing Ratios to 3D.dat file
C
C       Version 1.2 Level 040302
C ---   Fixed RH and Precip problems in the main code
C       Version 1.1 Level 040116
C ---   Created an option for the user to choose between output in old MM5.DAT
C ---   format or the new 3D.DAT format
      
C       Version 1.1 Level 031201      

C 	Create 3D.dat files from RUC model output in grib format
C 	Revised from caleta.f
C 	Mahesh Phadnis 
C 	7/7/2003
C 	Zhong Wu
C 	7/15/2002
C 	Purpose:
C   	1. Decode RUC model output in grib format
C   	2. Select a lat/lon range for some variables
C   	3. Code the selected data in CALMM5/3D.DAT and 2D.DAT format
C
C
C ---------------------------------------------------------------------
C Subroutines of decoding GRIB data are from NOAA/NCEP Office Note 388
C ---------------------------------------------------------------------

      include 'calruc.par'
      include 'calruc.cm1'

c --- set version and level number of program
      cver='2.1'
      clevel='100525'

c --- set dataset type and version
      dataset = '3D.DAT'
      dataver = '2.13'
      dataset2d = '2D.DAT'
      dataver2d = '2.13'
      datamod = '3D.DAT Created from RUC Data'

c --- Open user input control file
      open(ict, file='calruc.inp',status='old',action='read')

c --- Read control file for title,input,output,log file names
      read(ict,101) title
      read(ict,*)ibdate
      read(ict,*)iedate
      read(ict,*)ngap       
      read(ict,101) inbase
      read(ict,101) outbase
      read(ict,101) logfile 
 101  format (a)

      call getflname(inbase,nti)    
      call getflname(outbase,nto)    
      call getflname(logfile,ntl)

c --- Open log file and output grib file
      open (ilg,file=logfile,status='unknown')

c --- Echo control inputs to log file
      write(ilg,1019)cver,clevel
1019  format(1x,'CALRUC - Version: ',a8,3x,'Level: ',a8//)

      write(ilg,101)title(1:nti)
      write(ilg,1022)outbase(1:nto)
      write(ilg,1023)logfile(1:ntl)
      write(ilg,*)
 1022 format('Output CALRUC/3D.DAT file: ',a)
 1023 format('calruc log file:     ',a)

c     Selected domain range for CALRUC data: 
C     (Lat1/Lat2, Lon1/Lon2 in decimal, < 0 for SH or WH)
C     (Veritical levels: 1-50, 50 is ~50 hPa)
      read(ict,*)ires
      read(ict,*)itype
      read(ict,*)iapply
      read(ict,*)isite
      read(ict,*)iform
      read(ict,*)icompress
      read(ict,*)iselect
      read(ict,*)rlatmin,rlatmax        
      read(ict,*)rlonmin,rlonmax
      read(ict,*)nz1,nz2
      read(ict,*)ihgtshift
      read(ict,*)ichk_prof

      write(ilg,372)ires
      write(ilg,373)itype
      write(ilg,374)iapply
      write(ilg,3741)isite
      write(ilg,375)iform      
      write(ilg,376)iselect
      if (iselect.eq.0) then
        write(ilg,380)int(rlatmin),int(rlatmax)
        write(ilg,381)int(rlonmin),int(rlonmax)
      else
        write(ilg,377)rlatmin,rlatmax
        write(ilg,378)rlonmin,rlonmax
      endif

      if(itype.eq.0) then
         if(nz2.gt.mxnz) then
            write(ilg,*)' nz2 > mxnz ',nz2,mxnz, ' set nz2=mxnz'
            nz2=mxnz
         endif
      else
         if(nz2.gt.mxnzp) then
            write(ilg,*)' nz2 > mxnzp ',nz2,mxnzp, ' set nz2=mxnzp'
            nz2=mxnzp
         endif
      endif
      write(ilg,379)nz1,nz2

      if(ihgtshift.ge.0) then
        write(ilg,382)ihgtshift
      else
        write(ilg,*)' No vertical shift in Geopotential Height Desired'
      endif
 372  format('Resolution of Input Data (0 - 20 km, 1 - 40 km) :',
     &t55,(x,i3))
 373  format('Type of Input File (0 - hybrid, 1 - pressure) :',
     &t55,(x,i3))
 374  format('Application Type (0 - analysis, 1 - forecast):',
     &t55,(x,i3))
 3741 format('FTP web site  (1 - NCEP, 2 - NWS, 3 - ARM):',
     &t55,(x,i3))
 375  format('Format of Output (0 - MM5.DAT, 1 - 3D.DAT):',t55,(x,i3))
 376  format('Selection type (0 - i,j , 1 - Lat/Lon) :',t55,(x,i3))
 377  format('latitude  range:',t55,2(4x,f7.2))
 378  format('longitude range:',t55,2(2x,f9.2))
 379  format('Vertical Levels:',t55,2(2x,i3))
 380  format('J range:',t55,2(x,i3))
 381  format('I range:',t55,2(x,i3))
 382  format('Used desired shift in Geopotential Height = ',i3,' m')

      write(ilg,*)' ichk_prof:',ichk_prof

      write(ilg,*)

      if (iselect.eq.1) then
      if (rlatmin.ge.rlatmax) then
         write(ilg,*)' Error: latitude min > latitude max'
         write(ilg,*)' stop'
         stop
      endif
      if (rlonmin.ge.rlonmax) then
         write(ilg,*)' Error: longitude min > longitude max'
         write(ilg,*)' stop'
         stop
      endif
      endif
      if (iselect.eq.0) then
        ilatmin = int(rlatmin)
        ilatmax = int(rlatmax)
        ilonmin = int(rlonmin)
        ilonmax = int(rlonmax)      
      if (ilatmin.gt.ilatmax) then
         write(ilg,*)' Error: i min > i max'
         write(ilg,*)' stop'
         stop
      endif
      if (ilonmin.gt.ilonmax) then
         write(ilg,*)' Error: j min > j max'
         write(ilg,*)' stop'
         stop
      endif
      endif
      
      if (nz1.ge.nz2) then
         write(ilg,*)' Error: NZ min > NZ max'
         write(ilg,*)' stop'
         stop
      endif

C     Read in grid configuration for RUC Grid 
C     from a look-up table
      if(ires.eq.0) then

c ---    Determine which header file is needed for RUC 20 data
c ---    and the limit of beg/end dates for this period
         call begend(ibdate,idate1,idate2,infile,ilg)

         if(iedate.gt.idate2) then
            write(ilg,331)iedate,idate2
            print 331,iedate,idate2
 331        format(/,' **** Error: Required ending date passes',
     &          ' the ending date for this period ******',/,
     &          ' Required ending date:',i12,/,
     &          ' Ending date on tile: ',i12,/,
     &          ' Reset ending date and Run again.')
            stop
         endif
      else
         infile='ruc40.dat'
      endif

      open(1,file=infile,status='old',action='read')

      read(1,*)nx212,ny212
      read(1,*)flats,flonv,re,conf
      read(1,*)xbm,xem,dxm
      read(1,*)ybm,yem,dym
      read(1,*)xbc,ybc,dxym
      
      if(ires.eq.0) then  ! mxruc, myruc depends on 20 or 40 km file
         mxruc = mxnx     ! for 20-km
	 myruc = mxny
         do j=1,myruc
           do i=1,mxruc
            read(1,*)ii,jj,(xy212(i1,i,j),i1=1,7),termp,iveg(i,j) 
                        !order:x,y,flon,flat,xc,yc,fm
            if(ii.ne.i .or. jj.ne.j) then
               print *,'Error in I/J: ',ii,jj,i,j
               stop
            endif
            ter(i,j)=termp  !read terrain data from a predefine input file
           enddo
        enddo
      else
         mxruc = 151   ! for 40-km
	 myruc = 113
         do j=1,myruc
           do i=1,mxruc
            read(1,*)ii,jj,(xy212(i1,i,j),i1=1,7),termp 
                        !order:x,y,flon,flat,xc,yc,fm
            if(ii.ne.i .or. jj.ne.j) then
               print *,'Error in I/J: ',ii,jj,i,j
               stop
            endif
            iveg(i,j)=0
            ter(i,j)=termp  !read terrain data from a predefine input file
           enddo
        enddo

      endif 

      close(1)

C     Select I/J ranges based on required lat/lon

      call getrange
      
C     Open output file name based on required date
      write(outfile,501)outbase(1:nto)
 501  format(a)
      write(outfile2,504)outbase(1:nto)
 504  format(a)

      open (im3d,file=outfile,status='unknown')
      if(iosrf.eq.1) 
     &   open (im2d,file=outfile2,status='unknown')

c --- convert ibdate and iedate to year, month, day and hour
c     (note: ibeg/iend in in YYMMDDHH form)
      call getdate(ibdate,iyearbeg,imonthbeg,idaybeg,ihourbeg)
      call julday(ilg,iyearbeg,imonthbeg,idaybeg,julbeg)

      call getdate(iedate,iyearend,imonthend,idayend,ihourend)
      call julday(ilg,iyearend,imonthend,idayend,julend)

c --- number of hours of extract in the output (asked by the user)
      call gethours(ibdate,iedate,nhours)

      write(ilg,*)' '
      write(ilg,*)'Number of Grib files:',nhours
      print *,'Number of Grib files:',nhours

      IYY = iyearbeg
      IMM = imonthbeg
      IJUL = julbeg
      IHH = ihourbeg
      IHHINI=IHH
C --- Define the output format
      ioutform=81+ioutw*10+ioutq+ioutc+iouti+ioutg

C --- Loop over RUC grib files ------
      do 5000 ihour=1,nhours,ngap

      call grday(ilg,IYY,IJUL,IMM,IDD)
      call getfilename(infile,inbase,ires,itype,iapply
     &                 ,isite,IYY,IMM,IDD,IHH,IHHINI,ihour)

      nt=index(infile, ' ')-1
      write(*,1021)ihour,infile(1:nt)
      write(ilg,1021)ihour,infile(1:nt)
 1021 format(' Input RUC file # ',i3,' ',a)

      open(in,err=5500,file=infile,form='binary',access='sequential'
     &    ,status='old',action='read')

C     Initialize check arrays
      if (itype.eq.0) then
      
        do i=1,n3d
          do k=1,mxnz
            ichk3d(k,i)=0
          enddo
        enddo
        do i=1,n2d
          ichk2d(i)=0
        enddo
	
      else
      
        do i=1,n3dp
          do k=1,mxnzp
            ichk3dp(k,i)=0
          enddo
        enddo
        do i=1,n2dp
          ichk2dp(i)=0
        enddo
	
      endif
        
      ihr_fst=(ihour-1)*1

c --- Decode RUC grib file

      call degrib    ! read/decode one grib file (for one model hour)

      goto 5600

 5500 print *,'Error to open file: '
      print *,infile
      print *,'Processing stopped'
      print *,'Get this file and restart the program'

      write(ilg,*)'Error to open file: '
      write(ilg,*)infile
      write(ilg,*)'Processing stopped'
      write(ilg,*)'Get this file and restart the program'
      stop

 5600 close(in)
 
      if(ihour.eq.1) call wrthd

C --- Check whether all variables needed are there

      if (itype.eq.0) then
      
        do i=1,n3d
          do k=1,mxnz
            id=ichk3d(k,i)
            if(id.eq.0) then
               if(i.ne.3 .and. i.lt.8) then
                  write(ilg,*)'Warning: Zero 3D Var -',i,k
               elseif(k.le.23 .and. k.ne.1) then
                  write(ilg,*)'Warning: Zero 3D Var -',i,k
               endif
            endif
          enddo
        enddo
        do i=1,n2d
          id=ichk2d(i)
          if(id.eq.0) then
            write(ilg,*)'Warning: Zero 2D Var -',i
          endif
        enddo
      
      else

        do i=1,n3dp
          do k=1,mxnzp
            id=ichk3dp(k,i)
            if(id.eq.0) then
               if(i.ne.3) then
                  write(ilg,*)'Warning: Zero 3D Var -',i,k
               elseif(k.le.23 .and. k.ne.1) then
                  write(ilg,*)'Warning: Zero 3D Var -',i,k
               endif
            endif
          enddo
        enddo
        do i=1,n2dp
          id=ichk2dp(i)
          if(id.eq.0) then
            write(ilg,*)'Warning: Zero 2D Var -',i
          endif
        enddo

      endif       

C --- Check outliers for 3D and 2D (Only for Pr,GH,VPT,Humid, U, V,WP)
      if(ichk_prof.ne.1) goto 1500 
      ifail=0
      do ivar=1,7 
      alow=alim3d(1,ivar)
      ahig=alim3d(2,ivar)

      do j=ny1,ny2
      do i=nx1,nx2
         do k=1,mxnz

C ---       Range check first
            aa=x3d(i,j,k,ivar)
            if(aa.lt.alow .or. aa.gt.ahig) then
               write(ilg,102)cnames(ivar),i,j,k,aa,alow,ahig
               print 102,cnames(ivar),i,j,k,aa,alow,ahig
 102           format(' Warning: Out Range-Reset: ',a8,3i5,3f12.4)
               if(aa.lt.alow) then
                  aa=alow
               else
                  aa=ahig
               endif
               x3d(i,j,k,ivar)=aa
            endif
            xtmp(k)=x3d(i,j,k,ivar)
         enddo
         if(ivar.le.1) then
            iorder=-1
         elseif (ivar.le.2) then
            iorder=1
         else
            iorder=0
         endif

         alow=alim3d(1,ivar)
         ahig=alim3d(2,ivar)

         iok=1
         h0=ter(i,j)
         call chkprof(xtmp,mxnz,iorder,ivar,ilg,iok,i,j,h0)

         if(iok.ne.1) ifail=ifail+1

      enddo
      enddo
      enddo

      if(ifail.gt.0) then
         write(ilg,104)ifail
         print 104,ifail
         write(ilg,105)trim(infile)
         print 105,trim(infile)
 104     format(' Terminated due to QA Failures:',i6,
     &          /,' Check list file for errors')
 105     format(' Grib File with violations:',/,1x,a,
     &          /,' Processing Stopped')
         stop
      endif

 1500 continue

C     All variables are there
C     Print *,'Find all variable for ',idate

C     Output one-hour data in MM53D format
      print *,'Output to 3D.DAT at ',idate
      write(ilg,*)'Output to 3D.DAT at ',idate
      write(ilg,*)' '
      
      if(itype.eq.0) then   !hybrid level

      do j=ny1,ny2
      do i=nx1,nx2
      
c ---    calculate the temperature from virtual potential temperature

         a1 = (1000/(x3d(i,j,1,1)/100))**(0.286*(1-0.251*x3d(i,j,1,4)))
         tv = x3d(i,j,1,3)/a1
	 tmp = tv/(1+0.608*x3d(i,j,1,4))
	 
	 hgt = x3d(i,j,1,2)
	  
         expnt = alog(x3d(i,j,1,1)) + 9.81*hgt/(287.04*tmp) 	 
	 slp = exp(expnt)/100.	        !calculation of sea-level pressure
         pre=x2d(i,j,1) + x2d(i,j,2)    !total precipitation
         if(pre.ne.0) pre=pre/10.0/1.0  ! change unit to cm/hr
                                        ! Precip is 1-hour accumulation
         snow=x2d(i,j,3)
         if(snow.gt.0) then
          isnow=1
         else
          isnow=0
         endif

C---     Defining Latitude and Longitude         
         flon=xy212(3,i,j)
         flat=xy212(4,i,j)

C---	 Use 10 m winds instead of the surface winds        
         u10m = x2d(i,j,6)
         v10m = x2d(i,j,7)
         call uv2ws(u10m,v10m,ws10,wd10)
         call rotate(ws10,wd10,flat,flon)
         iwd10=nint(wd10)
         if(iwd.eq.0) iwd10=360

         temp2m = x2d(i,j,5)
         q2m = x2d(i,j,8)*1000
         rswd = 0.
         rlwd = 0.
         sst = 0.

         write(im3d,98)idate,i,j,slp,pre,isnow,rswd,rlwd,
     &                 temp2m,q2m,wd10,ws10,sst
 98      format(i10,2i3,f7.1,f5.2,i2,3f8.1,f8.2,3f8.1)

C---     Loop over vertical levels (from surface to top)
         do k=nz1,nz2
	 	 
	  a1 = 
     &(1000/(x3d(i,j,k,1)/100))**(0.286*(1-0.251*x3d(i,j,k,4)))
         tv = x3d(i,j,k,3)/a1
	  atk = tv/(1+0.608*x3d(i,j,k,4))
	  atc = atk - 273.16
	    
	  pvs = 6.112*exp(17.67*atc/(atc+243.5))
	  humids = 0.622*pvs*100/x3d(i,j,k,1)

          rh=x3d(i,j,k,4)*100/humids

          if(k.eq.1 .and. rh.eq.0) rh=x3d(i,j,k+1,4)*100/humids
            
          irh=nint(rh)
          irh=min(100,irh)

          pp=x3d(i,j,k,1)/100.
          ips=nint(pp)			!pressure
	    
	  hgt = x3d(i,j,k,2)
          izh=nint(hgt)

          uu=x3d(i,j,k,5)
          vv=x3d(i,j,k,6)
          call uv2ws(uu,vv,ws,wd)
          call rotate(ws,wd,flat,flon)	!rotate winds according to true north
          iwd=nint(wd)
          if(iwd.eq.0) iwd=360	
    
          wp=x3d(i,j,k,7)		!vertical winds
          call getw(wp,pp,atk,rh,ww,aq)
          aq=x3d(i,j,k,4)*1000.          !specific humidity in g/kg

         cloud = x3d(i,j,k,8)*1000. 	! units in RUC data are in kg/kg
         rain  = x3d(i,j,k,9)*1000.
         rice  = x3d(i,j,k,10)*1000. 	
         snow  = x3d(i,j,k,11)*1000.
         graup = x3d(i,j,k,12)*1000. 

         if((cloud.gt.99.995).or.(rain.gt.99.995).or.(rice.gt.99.995)
     &       .or.(snow.gt.99.995).or.(graup.gt.99.995)) then
           print *, 'Mixing ratios greater than 10 '
           write(ilg,*)'Mixing ratios greater than 10 '
           print *, 'Cloud, Rain, Ice, Snow, Graupel '
           write(ilg,*)'Cloud, Rain, Ice, Snow, Graupel '
           print *, cloud,rain,rice,snow,graup
           write(ilg,*)cloud,rain,rice,snow,graup
           stop
         endif

         ncmp=0
         do ii=1,mxcmp
           xcmp(ii)=0
         enddo

C---     Compress for output formats of 93-95 
         if(ioutform.ge.93) then
           aqc=cloud
           aqr=rain
           ncmp=ncmp+1
           xcmp(ncmp)=aqc
           ncmp=ncmp+1
           xcmp(ncmp)=aqr
         endif
         if(ioutform.ge.94) then
           aqi=rice
           aqs=snow
           ncmp=ncmp+1
           xcmp(ncmp)=aqi
           ncmp=ncmp+1
           xcmp(ncmp)=aqs
         endif
         if(ioutform.ge.95) then
           aqg=graup
           ncmp=ncmp+1
           xcmp(ncmp)=aqg
         endif
 
C---	 Shift the vertical level by the user defined specification
         if(ihgtshift.ge.0) then
           izh = izh + ihgtshift
         endif

C---	 Compress Data
         if (icompress.eq.1) then
           call wrtcmp(ips,izh,atk,iwd,ws,ww,irh,aq,xcmp,ncmp)
	 else	    
           write(im3d,92)ips,izh,atk,iwd,ws,ww,irh,aq,cloud,rain,
     &                 rice,snow,graup
         endif
 92      format(i4,i6,f6.1,i4,f5.1,f6.2,i3,1f5.2,5f6.3)
         enddo

      enddo
      enddo
      
      else    !Pressure levels
      
      do j=ny1,ny2
      do i=nx1,nx2
      
         pre=x2d(i,j,1) + x2d(i,j,2)    !total precipitation
         if(pre.ne.0) pre=pre/10.0/1.0 ! change unit to cm/hr
                           ! Precip is 1-hour accumulation
         snow=x2d(i,j,3)
         if(snow.gt.0) then
            isnow=1
         else
            isnow=0
         endif

         temp2m = x2d(i,j,4)
         q2m = x2d(i,j,5)*1000

C---	 Use 10 m winds instead of the surface winds        
C---     Defining Latitude and Longitude         
         flon=xy212(3,i,j)
         flat=xy212(4,i,j)

         u10m = x2d(i,j,6)
         v10m = x2d(i,j,7)
         call uv2ws(u10m,v10m,ws10,wd10)
         call rotate(ws10,wd10,flat,flon)
         iwd10=nint(wd10)
         if(iwd.eq.0) iwd10=360

c ---    Use MSLP in RUC files, instead of calculating
         slp =x2d(i,j,8)/100.

         rswd = 0.
         rlwd = 0.
         sst = 0.

         write(im3d,98)idate,i,j,slp,pre,isnow,rswd,rlwd,
     &                 temp2m,q2m,wd10,ws10,sst

c        Loop over vertical levels (from surface to top)
         do k=nz1,nz2
            rh=x3d(i,j,k,3)

            if(k.eq.1 .and. rh.eq.0) rh=x3d(i,j,k+1,3)
            
            irh=nint(rh)
            irh=min(100,irh)

            pp=pres(k)
            ips=nint(pp)
            izh=nint(x3d(i,j,k,1))
            atk=x3d(i,j,k,2)

            uu=x3d(i,j,k,4)
            vv=x3d(i,j,k,5)

            call uv2ws(uu,vv,ws,wd)
            call rotate(ws,wd,flat,flon)

            iwd=nint(wd)
            if(iwd.eq.0) iwd=360

            wp=x3d(i,j,k,6)
            call getw(wp,pp,atk,rh,ww,aq)

            write(im3d,93)ips,izh,atk,iwd,ws,ww,irh,aq
 93         format(i4,i6,f6.1,i4,f5.1,f6.2,i3,f5.2)
         enddo

      enddo
      enddo
      
      endif

      call incr(ilg,IYY,IJUL,IHH,NGAP)

 5000 continue

      close(ict)
      close(ilg)
      close(im3d)
      if(iosrf.eq.1) close(im2d)

      print *,'Processing succeeded'
      print *,'--------------------'

      stop
      end

      include 'calruc.blk'
c----------------------------------------------------------------------
c --- BRING IN CALPUFF SYSTEM UTILITY SUBROUTINES
      include 'calutils.for'
c      include 'coordlib.for'

c ***********************************************************************

      subroutine deGRIB

c Obtained from NOAA/NCEP Office Note 338 - March 10, 1998 version
c Modified by Zhong-Xiang Wu to fit the gribtyc.f (8/5/99)
c
c New main program to solve 'GRIB' in BDS problem   19971105
c
c This GRIB decoder has been tested on SGI/iris4, HP/ux, SUN/sunOS4.1.3 and
c     DEC/Alpha.
c
c maximum byte array expected: 60000*8
c
      include 'calruc.par'
      include 'calruc.cm1'

      parameter (nnx=720,nny=361,nmax=nnx*nny)  ! global, 0.5 degree
      parameter (mxrd=1024)           ! number of bytes in each read, was 1024
      parameter (nbit=32)             ! number of bits in a word,
      parameter (nword=nbit/8)        ! number of bytes in a word,
c      parameter (mxcray=60000)        ! max number of words for a GRIB on Cray
      parameter (mxcray=300000)        ! max number of words for a GRIB on Cray
      parameter (mxlen=mxcray*8/nword) ! max number of words for a GRIB on Sun
      parameter (mxone=mxrd/nword)     ! max length of ione
      dimension ingrib(mxlen), ione(mxone)
c     character one*(mxrd),inf*72, two*(mxcray)
      character one*1024,inf*80, two*300000
      character*132 asum
      character*1 str1
      integer*4 laspos
      integer*4 ibloc(4),lengrib,gribedi
      logical   newgrib
      equivalence (one,ione),(two,ingrib)
      save
      
c ---------------------------------------------------------------
c      write(*,*)'enter 1)summarize  2)debug  3)debug & summarize?'
c      read(*,*) isel

      isel=1
      isum=1
      iprnt=0  ! Print (1) or not (0) detail info
      mgrib=0

      istart=1
c
      ngrib=0
      laspos=0
      one=' '
      newgrib=.false.
      do i=1,3
c        ist=fgetc(in,one(i:i))
c        if(ist.ne.0) go to 9998
         read(in,err=9998)str1
         one(i:i)=str1
         laspos=laspos+1
      enddo
      two(1:3)=one(1:3)

   10 continue
      one(1:3)=two(1:3)
      indxt=0
c      ist=fgetc(in,one(4:4))
c      if(ist.ne.0) go to 9998
      read(in,err=9998,end=9999)str1
      one(4:4)=str1
      laspos=laspos+1
     
      if(one(1:4).eq.'GRIB') then
        ibeg=laspos-4
        if(isel.ne.1) write(*,*) '....found one GRIB, starts ',ibeg
        do ka=5,8
c          ist=fgetc(in,one(ka:ka))
c          if(ist.ne.0) go to 9998
           read(in,err=9998)str1
           one(ka:ka)=str1
           laspos=laspos+1
        enddo
        two(1:8)=one(1:8)

        indxt=8
c
c GRIB x x x  1 -------> GRIB Edition 1
c GRIB 0 0 24 0 -------> ECMWF GRIB Edition 0
c GRIB 98       -------> ECMWF GRIB Edition X, no lenGRIB
c
c swap one after 'GRIB' since GRIB is in ascii       ! DEC_ALPHA
        call swap4(ione(2),ione(2),4)                ! DEC_ALPHA
        call gbytes(one,ibloc,32,8,0,4)
        if(iprnt.ne.0) write(*,*) ' ibloc 4: ',(ibloc(jk),jk=1,4)
        if(ibloc(1).ne.98 .and. ibloc(4).eq.1) then
          call gbyte(one,lengrib,32,24)
          if(iprnt.ne.0) write(*,*) ' GRIB Edition 1 ',lengrib
          gribedi=1
          newgrib=.true.
        elseif(ibloc(1).eq.0 .and. ibloc(2).eq.0 .and. 
     &    ibloc(3).eq.24 .and. ibloc(4).eq.0 ) then
          call gbyte(one,lenpds,32,24)
          if(iprnt.ne.0) write(*,*) ' ECMWF GRIB Edition 0 ',lenpds
          gribedi=0
          newgrib=.true.
        elseif(ibloc(1).eq.98) then
          if(iprnt.ne.0) write(*,*) ' ECMWF GRIB Edition X '
          gribedi=-1
          lenpds=20
          newgrib=.true.
        else              !wrong GRIB at wrong place, probably data
          newgrib=.false.
        endif

        if(gribedi.eq.1 .and. indxt.lt.lengrib) then
          do i=indxt+1,lengrib
c            ist=fgetc(in,two(i:i))
c            if(ist.ne.0) go to 9998
             read(in,err=9998)str1
             two(i:i)=str1
             laspos=laspos+1
          enddo

          ngrib=ngrib+1
          if(ngrib.lt.mgrib) go to 10

          indxt=(lengrib+3)/4

          asum=' '
          if(isum.ne.0) write(asum(1:6),'(i6)') ngrib
c
          call swap4(ingrib(2),ingrib(2),lengrib-4)     ! DEC_ALPHA 1998MAY06
c
          call genGRIB1_decode(iprnt,isum,mxlen,nbit,nword,
c          call genGRIB1_decode(iprnt,isum,
     &                         indxt,asum,ingrib)
          if(isum.ne.0) write(asum(117:131),'(i15)') ibeg
c          if(isum.ne.0) write(*,'(a)') asum(1:132)
          if(isel.ne.1) then
c            write(*,*) 'enter 1 to continue?'
c            read(*,*) imore
            imore=1     !ZWU
            if(imore.ne.1) call exit(0)
          endif
          two(1:3)=two(lengrib-2:lengrib)
        else
          two(1:3)=one(2:4)
        endif
      else
        two(1:3)=one(2:4)
      endif          

c      if(ngrib.lt.87) go to 10
c      if(isetup.eq.1) then
c         goto 9998
c      else
         go to 10
c      endif

 9998 continue
      print *,'Read file Error'
      goto 9997

 9999 continue
c      print *,'End of gib file'

 9997 close(in)
 

      return
      end

c --------------------------------------------------------------------
      subroutine genGRIB1_decode(iprnt,isum,mxlen,nbit,nword,
     &                           lnin,asum,ingrib)
c
c This has been changed on 19971002
c iprnt: 0, no debug
c        1, debug mode, print extra information
c isum:  0, no summary
c        1, prepare one line summary in asum
c
c maximum byte array expected: 60000*8
c
c basic logic:
c  (1) read one complete GRIB record       (machine dependent)
c  (2) decode PDS
c  (3) decode GDS
c  (4) decode BMS if there
c  (5) decode BDS
c  if more processing, repeat steps (1) to (5)

      include 'calruc.par'
      include 'calruc.cm1'

      parameter (nnx=720,nny=361,nmax=nnx*nny) ! global, 0.5 degree
c      parameter (nbit=32)                       ! number of bits in a word,
c      parameter (nword=nbit/8)                  ! number of bytes in a word,
c      parameter (mxcray=60000)        ! max number of words for a GRIB on Cray
c      parameter (mxlen=mxcray*8/nword)! max number of words for a GRIB on Sun

      dimension ingrib(mxlen)
      integer ione
      integer*4 idum,ioff,iskp,ibloc(4)
      character*4 one, short*5
      character*132 asum
      equivalence (ione,one)
c PDS 
      integer*4 lengrib,gribedi,pds(22),kgds,kbms,lvl2
c GDS used
      integer*4 lengds
c BMS used
      integer*4 lenbms, mapuse
      logical*1 mapb(nmax)
c BDS used
      integer*4 lenbds,npoints,ix(nmax)
      dimension datan(nmax)
c     dimension grd1(nmax), grd2(144,73)        ! for select a region use

      save

c --------------------------------------------------------------------
      do i=1,nmax
         ix(i)=0
         datan(i)=0.
      enddo
c
c read in one GRIB record until the next 'GRIB'
c
c To handle blank records of "NCAR NMC 2.5 degree global analysis grids",
c  use direct access and check for 'GRIB'. If no 'GRIB', ierr=-9.
c
      ioff=0
      do i=1,lnin
        ione=ingrib(i)
        if(one.eq.'GRIB') go to 15
      enddo
      write(*,*) '****WARNING, no GRIB found in this grib record.'
      return
c
   15 continue
      ioff=i-1
c ioff is the number of bytes before 'GRIB'
      if(iprnt.ne.0) write(*,*) ' One GRIB starts at: ',ioff
c
c check to see if NMC or ECMWF
c
      iskp=(ioff+4)*8
      call gbytes(ingrib,ibloc,iskp,8,0,4)  
      if(ibloc(4).eq.1 .and. ibloc(1).ne.98) then
        call gbyte(ingrib,lengrib,iskp,24)
        call gbyte(ingrib,gribedi,iskp+24,8)
        if(iprnt.ne.0) write(*,*) '.. GRIB Edition 1 '
        ioff=ioff+8
        iskp=ioff*8
        call gbyte(ingrib,lenpds,iskp,24)
      elseif(ibloc(1).eq.0 .and. ibloc(2).eq.0 .and.
     &       ibloc(3).eq.24 .and. ibloc(4).eq.0 ) then
        call gbyte(ingrib,lengrib,iskp,24)
        call gbyte(ingrib,gribedi,iskp+24,8)
        if(iprnt.ne.0) write(*,*) '.. ECMWF GRIB Edition 0 '
        lenpds=lengrib
        ioff=ioff+4
        iskp=ioff*8
      elseif(ibloc(1).eq.98) then
        if(iprnt.ne.0) write(*,*) '.. ECMWF GRIB Edition X '
        gribedi=-1
        lenpds=20
        ioff=4
        iskp=ioff*8
      else
        write(*,*) '..unknown GRIB record, exit ',lengrib,idum
        call exit(0)
      endif
      if(iprnt.ne.0) write(*,*) ' GRIB, length,edi= ',lengrib,gribedi
      idum=lenpds+ioff
c
c at this time, ioff is the number of bytes before PDS
      if(iprnt.ne.0) write(*,*) 'lengrib,lenpds,ioff: ',lengrib,lenpds,ioff
c
c begin to process GRIB
c to check BLOK, loop through the whole ingrib array
c
c
c ioff is the number of bytes before PDS
c iskp is the number of bits before PDS
c
c process PDS
c
      call pds_GRIB1(iprnt,gribedi,iskp,idum,
     &   kgds,kbms,dscalf,ierr,lvl2,pds,ingrib)

c To handle old GRIB Edition from ECMWF
c     if(gribedi.eq.-1) then
c       call gbyte(ingrib,kgds,7*8+7,1)
c       call gbyte(ingrib,kbms,7*8+6,1)
c       write(*,*) '....... revised kgds,kbms .....'
c       write(*,*) '.... kgds = ',kgds
c       write(*,*) '.... kbms = ',kbms
c       call gbytes(ingrib,ibloc,10*8,8,0,2)
c       if(pds(8).eq.100 .or. pds(8).eq.103 .or.
c    &     pds(8).eq.105 .or. pds(8).eq.107 .or.
c    &     pds(8).eq.109 .or. pds(8).eq.111 .or.
c    &     pds(8).eq.113 .or. pds(8).eq.125 .or.
c    &     pds(8).eq.160 .or. pds(8).eq.200 .or.
c    &     pds(8).eq.201 ) then
c         pds(9)=ibloc(1)*32+ibloc(2)
c         write(*,*) '....... revised level.....'
c         write(*,*) '....level = ',pds(9)
c       endif
c     endif
c

c     Determine time flag
      ip7=pds(7)                ! var ID
      ip8=pds(8)                ! level/layer type
      ip9=pds(9)                ! pressur/height

      icen=pds(21)
      iyr=pds(10)
      imon=pds(11)
      iday=pds(12)
      ihr=pds(13)
      iunit=pds(15)
      iforecast1=pds(16)
      iforecast2=pds(17)
      itrange=pds(18)
      ips19=pds(19)
      ips20=pds(20)

c      do i=1,20
c         print *,'pds:',i,pds(i)
c      enddo

      if(iforecast2.ne.-9999) then
         nhrinc=iunit*(iforecast1+iforecast2)
      else
         nhrinc=iunit*(iforecast1)
      endif

      call julday(ilg,iyr,imon,iday,ijuldy)
      call incr(ilg,iyr,ijuldy,ihr,nhrinc)
      call grday(ilg,iyr,ijuldy,imon,iday)
      call timestamp(iyr,imon,iday,ihr,idateout)

      if(istart.eq.1) then
         idate=idateout
         istart=0
      endif

c process GDS
c
      if(kgds.eq.0) then
        lengds=0
        go to 40
      endif
      iskp=(pds(1)+ioff)*8
      call gbyte(ingrib,lengds,iskp,24)
      if(iprnt.ne.0) write(*,*) ' GDS length: ',lengds

      call gds_GRIB1(iprnt,gribedi,lnin,iskp,
     &    kgds,ierr,nx,ny,la1,lo1,pds,ingrib,lov,idx,idy,igds6)

c      print *,'Valid time: ',idateout,pds(7),pds(9)

c BMS section
c
 40   continue
      mapuse=0
      if(kbms.eq.0) then
        lenbms=0
c       go to 50
      else
        iskp=(lengds+pds(1)+ioff)*8
        call gbyte(ingrib,lenbms,iskp,24)
      endif
      call bms_GRIB1(iprnt,lnin,nmax,iskp,kbms,nx,
     &  ny,la1,lo1,ierr,numgd,mapuse,pds,ingrib,mapb)

      if(iprnt.ne.0) write(*,*) ' BMS length: ',lenbms
c
c BDS section
c
   50 continue
      iskp=(lenbms+lengds+pds(1)+ioff)*8
      call gbyte(ingrib,lenbds,iskp,24)
      if(iprnt.ne.0) write(*,*) '     bdslen: ',lenbds
c
c check the total length with GRIB header
c
      if(pds(3).eq.98 .and. gribedi.ne.1) then      ! ECMWF edition 0 or X '
        lenall=(4+lenpds+lengds+lenbms+lenbds+4)/1  ! 'GRIB+...+7777'
      else
        lenall=(8+pds(1)+lengds+lenbms+lenbds+4)/1
      endif
      if(iprnt.ne.0) write(*,*) 'lenall,pds(1),gds,bms,bds: ',
     &     ioff,pds(1),lengds,lenbms,lenbds,lenall
      short=' '
      if((lnin*nword).lt.lenall) then
c        write(*,*) ' *** SHORT GRIB record ',lnin*nword,lenall
c        if((lenall-(lnin*nword)).gt.4) then
c          write(*,*) ' *** no bds_GRIB1, return '
c          return
c        endif

        write(*,*) ' Following GRIB file Corrupted'
        write(*,'(1x,a)')trim(infile)
        write(*,*) ' Processing Terminated'
        write(ilg,*) ' Following GRIB File Corrupted'
        write(ilg,'(1x,a)')trim(infile)
        write(ilg,*) ' Processing Terminated'

        stop

      endif
      if(iprnt.ne.0) then
c        write(*,*) 'enter 1 to continue ...'
c        read(*,*) imore
        imore=1                 !ZWU
        if(imore.ne.1) go to 9998
      endif
      if(iprnt.ne.0) write(*,*) '..reading BDS, iskp = ',iskp
c
      call bds_GRIB1(iprnt,gribedi,lnin,iskp,dscalf,ierr,nmax,
     &  npoints,rc00,rmax,rmin,numgd,mapuse,mapb,pds,ingrib,
     &  ix,datan)

      if(ierr.eq.-3) then
        write(*,*) '****Too many grids than specified..'
        ierr=0
        return
      endif
C ****** ------ ******
C Specifically for the hybrid level case, the data is on a constant
C virtual potential temperature surface. Hence the degrib program
C sees just one value and bombs

      if(ierr.eq.-8.and.ip7.eq.189) then
        npoints = 67725
        datan(1:npoints) = datan(1)/100.
	goto 1010
      endif
      
C ****** ------ ******     
      
      if(ierr.eq.-8) then
cmjp        write(*,*) '****Constant field, all grids = ',datan(1)
cmjp        write(*,*) '    No number of grids can be computed from bds'
cmjp        write(*,*) '    Use number of grids from gds section'
        ierr=0
        return
      endif
      if(ierr.eq.-11) then
        write(*,*) '****UNKNOWN predefined grid type, '
        write(*,*) '    No grids decoded'
        ierr=0
        return
      endif
      if(ierr.eq.-12) then
        write(*,*) '****COMPLEX PACKING of spherical harmonic '
        write(*,*) '    coefficients, to be developed!'
        write(*,*) '    No grids decoded'
        ierr=0
        return
      endif
      if(iprnt.ne.0) write(*,83) (datan(k),k=1,15)
 83   format(4(E15.8,1X),E15.8)

C     Convert 1-D datan to a 2-D array and save it for selected variables. 
C     There are some variables occurring more than once. Using those only
C     consistent with forecast-hour in file name
1010  if(iapply.eq.1) then   !!! ONLY FOR FORECAST APPLICATIONS
        if(ip7.ge.61 .and. ip7.le.65) then
          if(iforecast2.ne.ihr_fst) then
            print 9,ihr_fst,iforecast2,ip7,ip8,ip9
            goto 9998
          endif
        elseif(iforecast1.ne.ihr_fst) then
          print 9,ihr_fst,iforecast1,ip7,ip8,ip9
          goto 9998
        endif
      endif
 9    format('Variable Not Needed:',6i8)

      if(itype.eq.0) then   ! LOOP OVER HYBRID LEVEL DATA
	      
C       3-D variable
        do ivar=1,n3d
          id=id3d(ivar)
          if(ip7.ne.id) goto 2000 
          if(ip8.ne.109) goto 2000	 	 	 
          if(mapuse.ne.0 .and. mapuse.ne.(nx*ny)) then
            write(*,*) 'ERROR: bitmap is used:',mapuse,nx*ny,nx,ny
            call exit(0)
          endif	 	 
	   if (ip9.le.0) goto 2000	 
	   iz = ip9
          call pass2d(datan,nmax,npoints,iz,ivar)
          ichk3d(iz,ivar)=1
cmjp          write(*,201)ip7,ip8,ip9,idateout
          write(ilg,201)ip7,ip8,ip9,idateout
 201      format('3D Variable selected: ',3i6,i12)
 2000     continue
        enddo
      
C       2-D variable
        do ivar=1,n2d
          id=id2d(ivar)	 
          if(ip7.ne.id) goto 3000
          if(ivar.le.4) then
             if(ip8.ne.1 .or. ip9.ne.0) goto 3000
          elseif(ivar.le.5) then
             if(ip8.ne.105 .or. ip9.ne.2) goto 3000
          elseif(ivar.le.7) then
             if(ip8.ne.105 .or. ip9.ne.10) goto 3000
          elseif(ivar.le.8) then
             if(ip8.ne.105 .or. ip9.ne.2) goto 3000
          elseif(ivar.gt.8) then
             print *,'Illegal Id for 2-D variable:',ivar
             stop
          endif 

          if(mapuse.ne.0 .and. mapuse.ne.(nx*ny)) then
            write(*,*) 'ERROR: bitmap is used:',mapuse,nx*ny,nx,ny
            call exit(0)
          endif
          call pass2d_2(datan,nmax,npoints,ivar)
          ichk2d(ivar)=1
cmjp          write(*,202)ip7,ip8,ip9
          write(ilg,202)ip7,ip8,ip9
 202      format('2D Variable selected: ',5i6)
 3000    continue
        enddo
      
      else  ! LOOP OVER PRESSURE LEVEL DATA
	
C       3-D variable
        do ivar=1,n3dp
          id=id3dp(ivar)
          if(ip7.ne.id) goto 2001 	 	 	 
          if(ip8.ne.100) goto 2001 	 	 	 
          if(mapuse.ne.0 .and. mapuse.ne.(nx*ny)) then
            write(*,*) 'ERROR: bitmap is used:',mapuse,nx*ny,nx,ny
            call exit(0)
          endif 	 
          call getz(ip9,iz,ifound)
          if(ifound.eq.0) goto 2001
          call pass2d(datan,nmax,npoints,iz,ivar)
          ichk3dp(iz,ivar)=1
cmjp          write(*,203)ip7,ip8,ip9,idateout
          write(ilg,203)ip7,ip8,ip9,idateout
 203      format('3D Variable selected: ',3i6,i12)
 2001     continue
        enddo
      
C       2-D variable
        do ivar=1,n2dp
          id=id2dp(ivar)	 
          if(ip7.ne.id) goto 3001
          if(ivar.le.3) then
             if(ip8.ne.1 .or. ip9.ne.0) goto 3001
          elseif(ivar.le.5) then
             if(ip8.ne.105 .or. ip9.ne.2) goto 3001
          elseif(ivar.le.7) then
             if(ip8.ne.105 .or. ip9.ne.10) goto 3001
          elseif(ivar.le.8) then
             if(ip8.ne.102 .or. ip9.ne.0) goto 3001
          elseif(ivar.gt.8) then
             print *,'Illegal Id for 2-D variable:',ivar
             stop
          endif 

          if(mapuse.ne.0 .and. mapuse.ne.(nx*ny)) then
            write(*,*) 'ERROR: bitmap is used:',mapuse,nx*ny,nx,ny
            call exit(0)
         endif
         call pass2d_2(datan,nmax,npoints,ivar)
         ichk2dp(ivar)=1
cmjp         write(*,204)ip7,ip8,ip9
         write(ilg,204)ip7,ip8,ip9
 204     format('2D Variable selected: ',5i6)
 3001    continue
        enddo
      
C ---   In the hybrid coordinate system the program does not
C --    access this loop
        if(ip7.eq.7 .and. ip8.eq.1 .and. ip9.eq.0) then   ! GEOPH at surface
          call pass2d_ter(datan,nmax,npoints)
        endif
	 
	endif !!! HYBRID OR PRESSURE LOOP TERMINATION


C ***************************************************************

 9998 continue
      if(iprnt.ne.0)
     &  write(*,*) ' finish reading input grib record....'
c
c make one line summary
c
      if(isum.ne.0)
     &  write(asum(7:132),3213) (pds(i),i=1,5),(pds(j),j=7,9),
     &    lvl2,(pds(i),i=10,13),(pds(k),k=15,18),pds(21),
     &    pds(22),lenbds,npoints,rmax,rmin,short
 3213 format(i3,i4,2i3,3i4,2i5,i3,3i2.2,2i4,i5,i4,2i5,i8,i6,
     &       2f9.1,1x,a5,18x)
c3213 format(6x,i3,i4,2i3,3i4,2i5,i3,3i2.2,4i4,2i5,i8,i6,2f9.1,1x,a5)
c    nn 32   2  7 80  2   7 100 1000    0 91070600   1   0   0  10  20   0   13152 10512+999999.9+999999.9 short
c123456iiijjjj1231231234iiiijjjj1234512345kkkiijjii12341234mmmmnnnnkkkkjjjj12345678123456fffffffffgggggggggxaaaaa
c12345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890
c0        1         2         3         4         5         6         7         8         9
c
c------------------------------------- select a region, say 40N-20S, 130W-70W
c 1. Check mapuse to detemine if a bitmap is used.
c    If a bitmap is used, npoints will be less than (nx*ny), so fill to grd1(nx*ny).
c 2. Remap grd1(1..npoints) to grd2(nx,ny).  If grd1 are gaussian grids, convert 
c    them to 2.5x2.5 lat,long grids.
c 3. Print the location of grd1(1), i.e. la1,lo1
c 4. Determine the starting,ending (i,j) on 2.5x2.5
c 5. Save the regional grids ((grd2(i,j),i=ibeg,iend),j=jbeg,jend)
c
c step 1:
c     If(mapuse.gt.0) then
c       if(mapuse.ne.(nx*ny)) then
c         write(*,*) '**** ERROR in select a region w/ bitmap ',mapuse,nx*ny,nx,ny
c         call exit(0)
c       endif
c       kcnt=0
c       do i=1,mapuse
c         if(mapb(i).eq.1) then
c            kcnt=kcnt+1
c            grd1(i)=datan(kcnt)
c         else
c            grd1(i)=-9999.
c         endif
c       enddo
c     else
c       if(npoints.ne.(nx*ny)) then
c         write(*,*) '**** ERROR in select a region, ',npoints,nx*ny,nx,ny
c         call exit(0)
c       endif
c       do i=1,npoints
c         grd1(i)=datan(i)
c       enddo
c     endif
c
c step 2:
c     if((nx*ny).ne.(144*73)) then
c       if((nx.ne.192) .or. (ny.ne.94)) then
c         write(*,*) '****ERROR, select a region only works for '
c         write(*,*) '           144x73 (2.5x2.5 degree) grids, or'
c         write(*,*) '           192x94 (gaussian) grids now......'
c         call exit(0)
c       endif
c       call interpolation(nx,ny,grd2,grd1)
c     else
c       do j=1,ny
c         do i=1,nx
c           grd2(i,j)=grd1((j-1)*nx+i)
c         enddo
c       enddo
c     endif
c
c step 3:
c     xlat1=float(la1)/1000.
c     xlon1=float(lo1)/1000.
c     write(*,*) '....grd2(1,1) is located at lat,long ',xlat1,xlon1
c
c step 4: 
c     if((la1 .ne. 90000) .or. (lo1 .ne. 0)) then
c       write(*,*) '****WARNING, adjust la1,lo1 to 90N,0E '
c       xlat1=90.
c       xlon1=0.
c     endif
c     slat=40.
c     elat=-20.
c     slong=360.-130.
c     elong=360.-70.
c     ibeg=(slong-xlon1)/2.5+1
c     iend=(elong-xlon1)/2.5+1
c     jbeg=(xlat1-slat)/2.5+1
c     jend=(xlat1-elat)/2.5+1
c
c step 5: save data
c     write(*,*) ((grd2(i,j),i=ibeg,iend),j=jbeg,jend)
c
c------------------------------------- select a region

      return
      end

c
c
c-- lines below are from ncep_int.f which is available in our anonymous ftp 
c   area on ncardata.ucar.edu in 'ftp/datasets/ds090.0/ncep_int.f' file.
c
      subroutine interpolation (nnx,nny,grdo2,grdi)
c
c This program demonstrates how to use the interpolation routines 
c (same as NCEP's) to get 2.5x2.5 grids from Gaussian (192x94) grids.
c
      parameter (mxx=192, mxy=94, imo=144,jmo=73)
      dimension grdi(mxx*mxy), grdo(imo*jmo),grdo2(imo,jmo)
      dimension gglat(mxy/2), bmapi(mxx*mxy),bmapo(mxx*mxy)
      real PK(mxy/2),PKM1(mxy/2)
      real SJN(mxy/2),WJN(mxy/2),CJN1(mxy/2)
      real AJN(mxy+1),AJO(jmo+1),XIO(imo+1),XJO(jmo+1)
      real WIO1(imo),WIO2(imo)
      integer JPO(jmo)
      real FPX(imo),FP0X(imo),WPX(imo),WTX(imo)
c     character*72 inf, outf
c
c The file 'T62.gauss_lat' is available from anonymous ftp on ncardata.ucar.edu
c under 'ftp/datasets/ds090.0/' directory.  It can be generated here as well.
c
c     open(unit=21,file='T62.gauss_lat')
c     do i=1,14
c       read(21,*)
c     enddo
c     do i=1,47
c       read(21,*) idum,gglat(i)
c     enddo
c     close(unit=21)
c
      gglat(1)=0.
c
c     kin=1
c     write(*,*) 'enter input data file?'
c     read(*,'(a)') inf
c     open(kin,file=inf)
c     write(*,*) 'enter input data dimension? <192,94>'
c     read(*,*) nnx,nny
      if(nnx.gt.mxx .or. (nny.gt.mxy)) then
        write(*,*) '****input dimension too large, exit'
        write(*,*) nnx,nny,'   ',mxx,mxy
        call exit(0)
      endif
      rmax=-999999.0
      rmin= 999999.0
      k=0
      do j=1,nny
        do i=1,nnx
          k=k+1
c         read(kin,*) idum,grdi(k)
          if(grdi(k).ge.rmax) then
            rmax=grdi(k)
c           kmax=idum
          endif
          if(grdi(k).le.rmin) then
            rmin=grdi(k)
c           kmin=idum
          endif
        enddo
      enddo
c     close(kin)
c     write(*,*) 'last idum = ',idum
c     write(*,*) 'max: ',kmax,rmax,'  min: ',kmin,rmin
c
      mp=0 !0:scalar, 1:vector, 2:flag, 3:budget
      mb=0 !0:no bitmap
      imn=192
      ixn=1
      jmn=94
      jxn=192
      ixo=1
      jxo=144
      cio=0.    ! starting longitude in degrees East
      cjo=90.   ! starting latitude in degrees North
      dio=2.5   ! longitude increment in degrees East
      djo=-2.5  ! latitude increment in degrees North
      call gg2ll(mp,mb,nnx,ixn,nny,jxn,gglat,bmapi,grdi,
     &           imo,ixo,jmo,jxo,cio,cjo,dio,djo,bmapo,grdo,
     &           sjn,wjn,cjn1,ajn,ajo,xio,xjo,wio1,wio2,jpo,
     &           fpx,wpx,wtx,fp0x,PK,PKM1)
c
c     do j=1,jmn/2
c       write(*,*) ' Gaussian lat ',j,gglat(j)
c     enddo
      k=0
      do j=1,jmo
        do i=1,imo
          k=k+1
          grdo2(i,j)=grdo(k)
        enddo
      enddo
c
c save interpolated grids into a GrADS (or IEEE) data file
c
c     write(*,*) 'enter output data file?'
c     read(*,'(a)') outf
c     open(unit=23,file=outf,form='unformatted',access='direct',
c    &     recl=imo*jmo*4)
c     write(23,rec=1) ((grdo2(i,j),i=1,imo),j=jmo,1,-1)
c     close(unit=23)
c     stop
c
      return
      end
C-----------------------------------------------------------------------
CFPP$ NOCONCUR R
      SUBROUTINE GG2LL(MP,MB,IMN,IXN,JMN,JXN,CJN,LN,FN,
     &                 IMO,IXO,JMO,JXO,CIO,CJO,DIO,DJO,LO,FO,
     &                 sjn,wjn,cjn1,ajn,ajo,xio,xjo,wio1,wio2,jpo,
     &                 fpx,wpx,wtx,fp0x,PK,PKM1)
c
C$$$  SUBPROGRAM DOCUMENTATION BLOCK
C
C SUBPROGRAM:    GG2LL       INTERPOLATE GAUSSIAN TO LAT-LON GRID.
C   PRGMMR: IREDELL          ORG: W/NMC23    DATE: 92-10-31
C
C ABSTRACT: A HORIZONTAL FIELD IS BILINEARLY INTERPOLATED
C   FROM A GLOBAL GAUSSIAN GRID TO A LATITUDE-LONGITUDE GRID.
C   THE GAUSSIAN LATITUDES MAY BE PASSED OR CALCULATED.
C   THE INPUT GLOBAL FIELD IS ASSUMED TO RUN EASTWARD AND SOUTHWARD,
C   STARTING AT GREENWICH AND THE NORTHERNMOST LATITUDE,
C   BUT THE OUTPUT FIELD ORIENTATION IS DEFINED BY PASSED ARGUMENTS.
C   THE FIELD MAY BE TAGGED A SCALAR, A VECTOR, A FLAG OR A BUDGET.
C   A FLAG FIELD SUCH AS THE LAND-SEA MASK IS NOT INTERPOLATED
C   BUT TAKEN FROM THE VALUES AT THE CLOSEST INPUT GRIDPOINTS.
C   A BUDGET FIELD SUCH AS PRECIPITATION IS INTERPOLATED
C   WHILE PRESERVING THE AREA INTEGRALS OF THE ORIGINAL FIELD.
C   POLAR SCALARS ARE THE AVERAGE OF THE CLOSEST LATITUDE CIRCLE VALUES.
C   POLAR VECTOR COMPONENTS ARE TAKEN FROM THE WAVENUMBER 1 COMPONENT
C   EXTRACTED FROM THE VALUES ON THE CLOSEST LATITUDE CIRCLE.
C   A LOGICAL BITMAP MAY BE PASSED TO MASK OUT PART OF THE INPUT FIELD.
C   IN THIS CASE, AN APPROPRIATE OUTPUT BITMAP IS CONSTRUCTED AND
C   THE OUTPUT FIELD IS COMPUTED ONLY WHERE THE OUTPUT BITMAP IS TRUE
C   AND ONLY USING DATA WHERE THE INPUT BITMAP IS TRUE.
C   WHERE THE OUTPUT BITMAP IS FALSE, THE OUTPUT IS SET TO ZERO.
C   THE BITMAP MAY ALTERNATIVELY BE CONSIDERED AS A TWO-WAY MASK,
C   IN WHICH CASE OUTPUT FIELD IS ALSO COMPUTED WHERE THE OUTPUT BITMAP
C   IS FALSE ONLY USING DATA WHERE THE INPUT BITMAP IS FALSE.
C   THE TWO-WAY MASK OPTION SHOULD NOT BE USED WITH VECTOR COMPONENTS.
C
C PROGRAM HISTORY LOG:
C   92-10-31  IREDELL
C   93-10-21  IREDELL   ALLOW VECTORS, FLAGS, BITMAP, NORTH/SOUTH FLIP
C   94-12-05  IREDELL   ALLOW BUDGETS, OPTIONALLY CALCULATE LATITUDES,
C                       PRECOMPUTE INDICES, GENERALIZE OUTPUT DOMAIN.
C   95-07-11  IREDELL   REWEIGHT VECTOR INTERPOLATION NEAR POLE.
C
C USAGE:    CALL GG2LL(MP,MB,IMN,IXN,JMN,JXN,CJN,LN,FN,
C    &                 IMO,IXO,JMO,JXO,CIO,CJO,DIO,DJO,LO,FO)
C   INPUT ARGUMENT LIST:
C     MP       - INTEGER FIELD PARAMETER IDENTIFIER
C                (0 FOR SCALAR, 1 FOR VECTOR, 2 FOR FLAG, 3 FOR BUDGET)
C     MB       - INTEGER BITMAP IDENTIFIER
C                (0 FOR NO BITMAP, 1 TO INTERPOLATE BITMAP,
C                 2 TO ALSO INTERPOLATE DATA TO BITMAP FALSES)
C     IMN      - INTEGER INPUT LONGITUDE DIMENSION
C     IXN      - INTEGER NUMBER TO SKIP BETWEEN INPUT LONGITUDES
C     JMN      - INTEGER INPUT LATITUDE DIMENSION (EVEN)
C     JXN      - INTEGER NUMBER TO SKIP BETWEEN INPUT LATITUDES
C     CJN      - REAL (JMN/2) GAUSSIAN LATITUDES IN DEGREES NORTH
C                (IF CJN(1)=0., THE LATITUDES ARE CALCULATED.)
C     LN       - LOGICAL ((IMN-1)*IXN+(JMN-1)*JXN+1) BITMAP IF MB=1
C     FN       - REAL ((IMN-1)*IXN+(JMN-1)*JXN+1) FIELD TO INTERPOLATE
C     IMO      - INTEGER OUTPUT LONGITUDE DIMENSION
C     IXO      - INTEGER NUMBER TO SKIP BETWEEN OUTPUT LONGITUDES
C     JMO      - INTEGER OUTPUT LATITUDE DIMENSION
C     JXO      - INTEGER NUMBER TO SKIP BETWEEN OUTPUT LATITUDES
C     CIO      - REAL START LONGITUDE IN DEGREES EAST
C     CJO      - REAL START LATITUDE IN DEGREES NORTH
C     DIO      - REAL LONGITUDE INCREMENT IN DEGREES EAST
C     DJO      - REAL LATITUDE INCREMENT IN DEGREES NORTH
C
C   OUTPUT ARGUMENT LIST:
C     LO       - LOGICAL ((IMO-1)*IXO+(JMO-1)*JXO+1) BITMAP IF MB=1
C     FO       - REAL ((IMO-1)*IXO+(JMO-1)*JXO+1) INTERPOLATED FIELD
C
C SUBPROGRAMS CALLED:
C   GLAT         COMPUTE GAUSSIAN LATITUDES
C
C ATTRIBUTES:
C   LANGUAGE: CRAY FORTRAN 77
C
C$$$
      REAL CJN(JMN/2)
      REAL FN((IMN-1)*IXN+(JMN-1)*JXN+1)
      REAL FO((IMO-1)*IXO+(JMO-1)*JXO+1)
      LOGICAL LN((IMN-1)*IXN+(JMN-1)*JXN+1)
      LOGICAL LO((IMO-1)*IXO+(JMO-1)*JXO+1)
      REAL SJN(JMN/2),WJN(JMN/2),CJN1(JMN/2)
      REAL PK(JMN/2),PKM1(JMN/2)
      REAL AJN(JMN+1),AJO(JMO+1),XIO(IMO+1),XJO(JMO+1)
      REAL WIO1(IMO),WIO2(IMO)
      INTEGER JPO(JMO)
      REAL FPX(IMO),FP0X(IMO),WPX(IMO),WTX(IMO)
      IJN(I,J)=(I-1)*IXN+(J-1)*JXN+1
      IJO(I,J)=(I-1)*IXO+(J-1)*JXO+1
C - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
C  PRECOMPUTE INDICES
      PI=ACOS(-1.)
      IF(CJN(1).EQ.0.) THEN
        write(*,*) '****No Gaussian latitudes, calculate them...'
        CALL GLAT(JMN/2,SJN,WJN,PK,PKM1)
        DO J1=1,JMN/2
          CJN1(J1)=(180/PI)*ASIN(SJN(J1))
          cjn(j1)=cjn1(j1)
        ENDDO
      ELSE
        DO J1=1,JMN/2
          CJN1(J1)=CJN(J1)
        ENDDO
      ENDIF
      DO J=1,JMO
        JPO(J)=0
      ENDDO
      IF(MP.NE.3) THEN
C  SCALAR OR VECTOR OR FLAG
        DO I=1,IMO
          XIO(I)=MOD((CIO+(I-1)*DIO)/360*IMN+2*IMN,FLOAT(IMN))+1
        ENDDO
        IF(MP.EQ.0) THEN
          DO I=1,IMO
            I1=XIO(I)
            WIO1(I)=I1+1-XIO(I)
            WIO2(I)=XIO(I)-I1
          ENDDO
        ELSEIF(MP.EQ.1) THEN
          DL=2*PI/IMN
          DO I=1,IMO
            I1=XIO(I)
            WIO1(I)=SIN((I1+1-XIO(I))*DL)/SIN(DL)
            WIO2(I)=SIN((XIO(I)-I1)*DL)/SIN(DL)
          ENDDO
        ENDIF
CDIR$ IVDEP
        DO J1=1,JMN/2
          AJN(J1)=CJN1(J1)
          AJN(JMN+1-J1)=-CJN1(J1)
        ENDDO
        DO J=1,JMO
          AJO(J)=CJO+(J-1)*DJO
        ENDDO
        IF(DJO.LT.0.) THEN
          JB=1
          JE=JMO
          JI=1
        ELSE
          JB=JMO
          JE=1
          JI=-1
        ENDIF
        J1=1
        DO J=JB,JE,JI
          IF(AJO(J).GE.AJN(1)) THEN
            XJO(J)=1
          ELSEIF(AJO(J).LE.AJN(JMN)) THEN
            XJO(J)=JMN
          ELSE
            DOWHILE(AJO(J).LT.AJN(J1+1))
              J1=J1+1
            ENDDO
            XJO(J)=J1+(AJN(J1)-AJO(J))/(AJN(J1)-AJN(J1+1))
          ENDIF
        ENDDO
        IF(XJO(1).EQ.1.) THEN
          JPO(1)=1
        ELSEIF(XJO(1).EQ.FLOAT(JMN)) THEN
          JPO(1)=JMN
        ENDIF
        IF(XJO(JMO).EQ.1.) THEN
          JPO(JMO)=1
        ELSEIF(XJO(JMO).EQ.FLOAT(JMN)) THEN
          JPO(JMO)=JMN
        ENDIF
      ELSE
C  BUDGET
        DO I=1,IMO+1
          XIO(I)=MOD((CIO+(I-1.5)*DIO)/360*IMN+2*IMN+0.5,FLOAT(IMN))+1
        ENDDO
        IBX=0
        DO I=1,IMO
          XIB=XIO(INT(I+0.5-SIGN(0.5,DIO)))
          XIE=XIO(INT(I+0.5+SIGN(0.5,DIO)))
          IB=XIB
          IE=XIE
          IF(IE.LT.IB) IE=IE+IMN
          IBX=MAX(IBX,IE-IB+1)
        ENDDO
        AJN(1)=1.
        AJN(JMN/2+1)=0.
        AJN(JMN+1)=-1.
CDIR$ IVDEP
        DO J1=2,JMN/2
          AJN(J1)=SIN((PI/180)*0.5*(CJN1(J1-1)+CJN1(J1)))
          AJN(JMN+2-J1)=-AJN(J1)
        ENDDO
        DO J=1,JMO+1
          PJO=CJO+(J-1.5)*DJO
          IF(PJO.GE.90.) THEN
            AJO(J)=1.
          ELSEIF(PJO.LE.-90.) THEN
            AJO(J)=-1.
          ELSE
            AJO(J)=SIN((PI/180)*PJO)
          ENDIF
        ENDDO
        IF(DJO.LT.0.) THEN
          JB=1
          JE=JMO+1
          JI=1
        ELSE
          JB=JMO+1
          JE=1
          JI=-1
        ENDIF
        J1=1
        DO J=JB,JE,JI
          IF(AJO(J).GE.AJN(1)) THEN
            XJO(J)=1
          ELSEIF(AJO(J).LE.AJN(JMN+1)) THEN
            XJO(J)=JMN+1
          ELSE
            DOWHILE(AJO(J).LT.AJN(J1+1))
              J1=J1+1
            ENDDO
            XJO(J)=J1+(AJN(J1)-AJO(J))/(AJN(J1)-AJN(J1+1))
          ENDIF
        ENDDO
        IF(XJO(1).EQ.1.) THEN
          JPO(1)=1
        ELSEIF(XJO(1).EQ.FLOAT(JMN)) THEN
          JPO(1)=JMN
        ENDIF
        IF(XJO(JMO+1).EQ.1.) THEN
          JPO(JMO)=1
        ELSEIF(XJO(JMO+1).EQ.FLOAT(JMN+1)) THEN
          JPO(JMO)=JMN
        ENDIF
      ENDIF
C - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
C  GET POLAR VALUES
      DO J=1,JMO,JMO-1
        IF(JPO(J).GT.0) THEN
          IF(MB.EQ.0) THEN
            IF(MP.EQ.0) THEN
C  FULL SCALAR
              J1=JPO(J)
              FP=0.
              DO I=1,IMN
                FP=FP+FN(IJN(I,J1))
              ENDDO
              FP=FP/IMN
              DO I=1,IMO
                FO(IJO(I,J))=FP
              ENDDO
            ELSEIF(MP.EQ.1) THEN
C  FULL VECTOR
              J1=JPO(J)
              FPC=0.
              FPS=0.
              DO I=1,IMN
                CI=COS(2*PI*(I-1)/IMN)
                SI=SIN(2*PI*(I-1)/IMN)
                FPC=FPC+CI*FN(IJN(I,J1))
                FPS=FPS+SI*FN(IJN(I,J1))
              ENDDO
              FPC=2*FPC/IMN
              FPS=2*FPS/IMN
              DO I=1,IMO
                CI=COS(2*PI*(I-1)/IMO)
                SI=SIN(2*PI*(I-1)/IMO)
                FO(IJO(I,J))=FPC*CI+FPS*SI
              ENDDO
            ELSEIF(MP.EQ.2) THEN
C  FULL FLAG
              J1=JPO(J)
              DO I=1,IMO
                FO(IJO(I,J))=FN(IJN(1,J1))
              ENDDO
            ELSEIF(MP.EQ.3) THEN
C  FULL BUDGET
              XJB=XJO(INT(J+0.5+SIGN(0.5,DJO)))
              XJE=XJO(INT(J+0.5-SIGN(0.5,DJO)))
              JB=XJB
              JE=XJE
              WJB=JB+1-XJB
              WJE=XJE-JE
              FP=0.
              WP=0.
              DO J1=JB,JE
                W=AJN(J1)-AJN(J1+1)
                IF(J1.EQ.JB) W=W*WJB
                IF(J1.EQ.JE) W=W*WJE
                DO I=1,IMN
                  FP=FP+W*FN(IJN(I,J1))
                ENDDO
                WP=WP+W*IMN
              ENDDO
              DO I=1,IMO
                FO(IJO(I,J))=FP/WP
              ENDDO
            ENDIF
          ELSE
            IF(MP.EQ.0) THEN
C  BITMAP OR TWOWAY SCALAR
              J1=JPO(J)
              FP=0.
              WP=0.
              IF(MB.EQ.2) FP0=0.
              DO I=1,IMN
                IF(LN(IJN(I,J1))) THEN
                  FP=FP+FN(IJN(I,J1))
                  WP=WP+1.
                ELSEIF(MB.EQ.2) THEN
                  FP0=FP0+FN(IJN(I,J1))
                ENDIF
              ENDDO
              DO I=1,IMO
                LO(IJO(I,J))=WP.GE.0.5*IMN
              ENDDO
              IF(LO(IJO(1,J))) THEN
                DO I=1,IMO
                  FO(IJO(I,J))=FP/WP
                ENDDO
              ELSEIF(MB.EQ.2.AND.WP.LT.IMN) THEN
                DO I=1,IMO
                  FO(IJO(I,J))=FP0/(IMN-WP)
                ENDDO
              ELSE
                DO I=1,IMO
                  FO(IJO(I,J))=0.
                ENDDO
              ENDIF
            ELSEIF(MP.EQ.1) THEN
C  BITMAP VECTOR
              J1=JPO(J)
              IP=0
              DO I=1,IMN
                IF(LN(IJN(I,J1))) IP=IP+1
              ENDDO
              DO I=1,IMO
                LO(IJO(I,J))=IP.EQ.IMN
              ENDDO
              IF(LO(IJO(1,J))) THEN
                FPC=0.
                FPS=0.
                DO I=1,IMN
                  CI=COS(2*PI*(I-1)/IMN)
                  SI=SIN(2*PI*(I-1)/IMN)
                  FPC=FPC+CI*FN(IJN(I,J1))
                  FPS=FPS+SI*FN(IJN(I,J1))
                ENDDO
                FPC=2*FPC/IMN
                FPS=2*FPS/IMN
                DO I=1,IMO
                  CI=COS(2*PI*(I-1)/IMO)
                  SI=SIN(2*PI*(I-1)/IMO)
                  FO(IJO(I,J))=FPC*CI+FPS*SI
                ENDDO
              ELSE
                DO I=1,IMO
                  FO(IJO(I,J))=0.
                ENDDO
              ENDIF
            ELSEIF(MP.EQ.2) THEN
C  BITMAP OR TWOWAY FLAG
              J1=JPO(J)
              DO I=1,IMO
                LO(IJO(I,J))=LN(IJN(1,J1))
                IF(LN(IJN(1,J1)).OR.MB.EQ.2) THEN
                  FO(IJO(I,J))=FN(IJN(1,J1))
                ELSE
                  FO(IJO(I,J))=0.
                ENDIF
              ENDDO
            ELSEIF(MP.EQ.3) THEN
C  BITMAP OR TWOWAY BUDGET
              XJB=XJO(INT(J+0.5+SIGN(0.5,DJO)))
              XJE=XJO(INT(J+0.5-SIGN(0.5,DJO)))
              JB=XJB
              JE=XJE
              WJB=JB+1-XJB
              WJE=XJE-JE
              FP=0.
              WP=0.
              WT=0.
              IF(MB.EQ.2) THEN
                FP0=0.
              ENDIF
              DO J1=JB,JE
                W=AJN(J1)-AJN(J1+1)
                IF(J1.EQ.JB) W=W*WJB
                IF(J1.EQ.JE) W=W*WJE
                DO I=1,IMN
                  IF(LN(IJN(I,J1))) THEN
                    FP=FP+W*FN(IJN(I,J1))
                    WP=WP+W
                  ELSEIF(MB.EQ.2) THEN
                    FP0=FP0+W*FN(IJN(I,J1))
                  ENDIF
                ENDDO
                WT=WT+W*IMN
              ENDDO
              DO I=1,IMO
                LO(IJO(I,J))=WP.GE.0.5*WT
              ENDDO
              IF(LO(IJO(1,J))) THEN
                DO I=1,IMO
                  FO(IJO(I,J))=FP/WP
                ENDDO
              ELSEIF(MB.EQ.2.AND.WP.LT.WT) THEN
                DO I=1,IMO
                  FO(IJO(I,J))=FP0/(WT-WP)
                ENDDO
              ELSE
                DO I=1,IMO
                  FO(IJO(I,J))=0.
                ENDDO
              ENDIF
            ENDIF
          ENDIF
        ENDIF
      ENDDO
C - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
C  INTERPOLATE THE REST OF THE FIELD
      DO J=1,JMO
        IF(JPO(J).EQ.0) THEN
          IF(MB.EQ.0) THEN
            IF(MP.EQ.0.OR.MP.EQ.1) THEN
C  FULL SCALAR OR VECTOR
              J1=XJO(J)
              J2=MIN(J1+1,JMN)
              WJ2=XJO(J)-J1
              WJ1=1.-WJ2
              DO I=1,IMO
                I1=XIO(I)
                I2=MOD(I1,IMN)+1
                WI1=WIO1(I)
                WI2=WIO2(I)
                W11=WI1*WJ1
                W21=WI2*WJ1
                W12=WI1*WJ2
                W22=WI2*WJ2
                FO(IJO(I,J))=W11*FN(IJN(I1,J1))+W21*FN(IJN(I2,J1))+
     &                       W12*FN(IJN(I1,J2))+W22*FN(IJN(I2,J2))
              ENDDO
            ELSEIF(MP.EQ.2) THEN
C  FULL FLAG
              J1=NINT(XJO(J))
              DO I=1,IMO
                I1=MOD(NINT(XIO(I))-1,IMN)+1
                FO(IJO(I,J))=FN(IJN(I1,J1))
              ENDDO
            ELSEIF(MP.EQ.3) THEN
C  FULL BUDGET
              XJB=XJO(INT(J+0.5+SIGN(0.5,DJO)))
              XJE=XJO(INT(J+0.5-SIGN(0.5,DJO)))
              JB=XJB
              JE=XJE
              WJB=JB+1-XJB
              WJE=XJE-JE
              DO I=1,IMO
                FPX(I)=0.
                WPX(I)=0.
              ENDDO
              DO J1=JB,JE
                WJ=AJN(J1)-AJN(J1+1)
                IF(J1.EQ.JB) WJ=WJ*WJB
                IF(J1.EQ.JE) WJ=WJ*WJE
                DO I1X=1,IBX
                  DO I=1,IMO
                    XIB=XIO(INT(I+0.5-SIGN(0.5,DIO)))
                    XIE=XIO(INT(I+0.5+SIGN(0.5,DIO)))
                    IB=XIB
                    IE=XIE
                    WIB=IB+1-XIB
                    WIE=XIE-IE
                    IF(IE.LT.IB) IE=IE+IMN
                    I1=IB+(I1X-1)
                    IF(I1.LE.IE) THEN
                      W=WJ
                      IF(I1.EQ.IB) W=W*WIB
                      IF(I1.EQ.IE) W=W*WIE
                      I1M=MOD(I1-1,IMN)+1
                      FPX(I)=FPX(I)+W*FN(IJN(I1M,J1))
                      WPX(I)=WPX(I)+W
                    ENDIF
                  ENDDO
                ENDDO
              ENDDO
              DO I=1,IMO
                FO(IJO(I,J))=FPX(I)/WPX(I)
              ENDDO
            ENDIF
          ELSE
            IF(MP.EQ.0.OR.MP.EQ.1) THEN
C  BITMAP OR TWOWAY SCALAR OR VECTOR
              J1=XJO(J)
              J2=MIN(J1+1,JMN)
              WJ2=XJO(J)-J1
              WJ1=1.-WJ2
              DO I=1,IMO
                I1=XIO(I)
                I2=MOD(I1,IMN)+1
                WI1=WIO1(I)
                WI2=WIO2(I)
                W11=WI1*WJ1
                W21=WI2*WJ1
                W12=WI1*WJ2
                W22=WI2*WJ2
                WP=0.
                IJ11=IJN(I1,J1)
                IJ21=IJN(I2,J1)
                IJ12=IJN(I1,J2)
                IJ22=IJN(I2,J2)
                IF(LN(IJ11)) WP=WP+W11
                IF(LN(IJ21)) WP=WP+W21
                IF(LN(IJ12)) WP=WP+W12
                IF(LN(IJ22)) WP=WP+W22
                WT=W11+W21+W12+W22
                IJ=IJO(I,J)
                LO(IJ)=WP.GE.0.5*WT
                IF(LO(IJ)) THEN
                  FP=0.
                  IF(LN(IJ11)) FP=FP+W11*FN(IJ11)
                  IF(LN(IJ21)) FP=FP+W21*FN(IJ21)
                  IF(LN(IJ12)) FP=FP+W12*FN(IJ12)
                  IF(LN(IJ22)) FP=FP+W22*FN(IJ22)
                  FO(IJ)=FP*WT/WP
                ELSEIF(MB.EQ.2.AND.WP.LT.WT) THEN
                  FP=0.
                  IF(.NOT.LN(IJ11)) FP=FP+W11*FN(IJ11)
                  IF(.NOT.LN(IJ21)) FP=FP+W21*FN(IJ21)
                  IF(.NOT.LN(IJ12)) FP=FP+W12*FN(IJ12)
                  IF(.NOT.LN(IJ22)) FP=FP+W22*FN(IJ22)
                  FO(IJ)=FP*WT/(WT-WP)
                ELSE
                  FO(IJ)=0.
                ENDIF
              ENDDO
            ELSEIF(MP.EQ.2) THEN
C  BITMAP OR TWOWAY FLAG
              J1=NINT(XJO(J))
              DO I=1,IMO
                I1=MOD(NINT(XIO(I))-1,IMN)+1
                LO(IJO(I,J))=LN(IJN(I1,J1))
                IF(LO(IJO(I,J)).OR.MB.EQ.2) THEN
                  FO(IJO(I,J))=FN(IJN(I1,J1))
                ELSE
                  FO(IJO(I,J))=0.
                ENDIF
              ENDDO
            ELSEIF(MP.EQ.3) THEN
C  BITMAP OR TWOWAY BUDGET
              XJB=XJO(INT(J+0.5+SIGN(0.5,DJO)))
              XJE=XJO(INT(J+0.5-SIGN(0.5,DJO)))
              JB=XJB
              JE=XJE
              WJB=JB+1-XJB
              WJE=XJE-JE
              DO I=1,IMO
                FPX(I)=0.
                IF(MB.EQ.2) FP0X(I)=0.
                WPX(I)=0.
                WTX(I)=0.
              ENDDO
              DO J1=JB,JE
                WJ=AJN(J1)-AJN(J1+1)
                IF(J1.EQ.JB) WJ=WJ*WJB
                IF(J1.EQ.JE) WJ=WJ*WJE
                DO I1X=1,IBX
                  DO I=1,IMO
                    XIB=XIO(INT(I+0.5-SIGN(0.5,DIO)))
                    XIE=XIO(INT(I+0.5+SIGN(0.5,DIO)))
                    IB=XIB
                    IE=XIE
                    WIB=IB+1-XIB
                    WIE=XIE-IE
                    IF(IE.LT.IB) IE=IE+IMN
                    I1=IB+(I1X-1)
                    IF(I1.LE.IE) THEN
                      W=WJ
                      IF(I1.EQ.IB) W=W*WIB
                      IF(I1.EQ.IE) W=W*WIE
                      I1M=MOD(I1-1,IMN)+1
                      IF(LN(IJN(I1M,J1))) THEN
                        FPX(I)=FPX(I)+W*FN(IJN(I1M,J1))
                        WPX(I)=WPX(I)+W
                      ELSEIF(MB.EQ.2) THEN
                        FP0X(I)=FP0X(I)+W*FN(IJN(I1M,J1))
                      ENDIF
                      WTX(I)=WTX(I)+W
                    ENDIF
                  ENDDO
                ENDDO
              ENDDO
              DO I=1,IMO
                LO(IJO(I,J))=WPX(I).GE.0.5*WTX(I)
                IF(LO(IJO(I,J))) THEN
                  FO(IJO(I,J))=FPX(I)/WPX(I)
                ELSEIF(MB.EQ.2.AND.WPX(I).LT.WTX(I)) THEN
                  FO(IJO(I,J))=FP0X(I)/(WTX(I)-WPX(I))
                ELSE
                  FO(IJO(I,J))=0.
                ENDIF
              ENDDO
            ENDIF
          ENDIF
        ENDIF
      ENDDO
C - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      RETURN
      END

C-----------------------------------------------------------------------
      SUBROUTINE GLAT(JH,SLAT,WLAT,PK,PKM1)
C$$$  SUBPROGRAM DOCUMENTATION BLOCK
C
C SUBPROGRAM:    GLAT        COMPUTE GAUSSIAN LATITUDE FUNCTIONS
C   PRGMMR: IREDELL          ORG: W/NMC23     DATE: 92-10-31
C
C ABSTRACT: COMPUTES SINES OF GAUSSIAN LATITUDE BY ITERATION.
C           THE GAUSSIAN WEIGHTS ARE ALSO COMPUTED.
C
C PROGRAM HISTORY LOG:
C   91-10-31  MARK IREDELL
C
C USAGE:    CALL GLAT(JH,SLAT,WLAT)
C
C   INPUT ARGUMENT LIST:
C     JH       - INTEGER NUMBER OF GAUSSIAN LATITUDES IN A HEMISPHERE
C
C   OUTPUT ARGUMENT LIST:
C     SLAT     - REAL (JH) SINES OF (POSITIVE) GAUSSIAN LATITUDE
C     WLAT     - REAL (JH) GAUSSIAN WEIGHTS FOR THE NH
C
C ATTRIBUTES:
C   LANGUAGE: CRAY FORTRAN
C
C$$$
      DIMENSION SLAT(JH),WLAT(JH)
c     PARAMETER(PI=3.14159265358979,C=(1.-(2./PI)**2)*0.25,EPS=1.E-14)
      PARAMETER(PI=3.14159265358979,C=(1.-(2./PI)**2)*0.25,EPS=1.E-7)
      PARAMETER(JBZ=50)
      DIMENSION PK(JH),PKM1(JH),BZ(JBZ)
      DATA BZ        / 2.4048255577,  5.5200781103,
     $  8.6537279129, 11.7915344391, 14.9309177086, 18.0710639679,
     $ 21.2116366299, 24.3524715308, 27.4934791320, 30.6346064684,
     $ 33.7758202136, 36.9170983537, 40.0584257646, 43.1997917132,
     $ 46.3411883717, 49.4826098974, 52.6240518411, 55.7655107550,
     $ 58.9069839261, 62.0484691902, 65.1899648002, 68.3314693299,
     $ 71.4729816036, 74.6145006437, 77.7560256304, 80.8975558711,
     $ 84.0390907769, 87.1806298436, 90.3221726372, 93.4637187819,
     $ 96.6052679510, 99.7468198587, 102.888374254, 106.029930916,
     $ 109.171489649, 112.313050280, 115.454612653, 118.596176630,
     $ 121.737742088, 124.879308913, 128.020877005, 131.162446275,
     $ 134.304016638, 137.445588020, 140.587160352, 143.728733573,
     $ 146.870307625, 150.011882457, 153.153458019, 156.295034268 /
C - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
C  ESTIMATE LATITUDES USING BESSEL FUNCTION
      R=1./SQRT((2*JH+0.5)**2+C)
      DO J=1,MIN(JH,JBZ)
        SLAT(J)=COS(BZ(J)*R)
      ENDDO
      DO J=JBZ+1,JH
        SLAT(J)=COS((BZ(JBZ)+(J-JBZ)*PI)*R)
      ENDDO
C - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
C  CONVERGE UNTIL ALL SINES OF GAUSSIAN LATITUDE ARE WITHIN EPS
      SPMAX=1.
      DO WHILE(SPMAX.GT.EPS)
        SPMAX=0.
        DO J=1,JH
          PKM1(J)=1.
          PK(J)=SLAT(J)
        ENDDO
        DO N=2,2*JH
          DO J=1,JH
            PKM2=PKM1(J)
            PKM1(J)=PK(J)
            PK(J)=((2*N-1)*SLAT(J)*PKM1(J)-(N-1)*PKM2)/N
          ENDDO
        ENDDO
        DO J=1,JH
          SP=PK(J)*(1.-SLAT(J)**2)/(2*JH*(PKM1(J)-SLAT(J)*PK(J)))
          SLAT(J)=SLAT(J)-SP
          SPMAX=MAX(SPMAX,ABS(SP))
        ENDDO
      ENDDO
C - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
C  COMPUTE COSINES AND GAUSSIAN WEIGHTS
      DO J=1,JH
        WLAT(J)=2.*(1.-SLAT(J)**2)/(2*JH*PKM1(J))**2
      ENDDO
      RETURN
      END

C-----------------------------------------------------------------------
      SUBROUTINE MPFDEF(IPTV,MPF)
C$$$  SUBPROGRAM DOCUMENTATION BLOCK
C
C SUBPROGRAM: MPFDEF         SETS DEFAULT PARAMETER FLAGS
C   PRGMMR: IREDELL          ORG: W/NMC23     DATE: 92-10-31
C
C ABSTRACT: SETS FIELD IDENTIFIER DEFAULTS FOR VARIOUS PARAMETERS.
C   A FLAG OF 0 IS SCALAR, 1 IS VECTOR, 2 IS FLAG AND 3 IS BUDGET.
C   THESE IDENTIFIERS ARE USED IN INTERPOLATION.
C
C PROGRAM HISTORY LOG:
C   93-10-21  IREDELL
C   93-12-05  IREDELL   ADDED BUDGET FLAG
C
C USAGE:    CALL MPFDEF(IPTV,MPF)
C   INPUT ARGUMENTS:
C     IPTV         PARAMTER TABLE VERSION (ONLY 1 OR 2 IS RECOGNIZED)
C   OUTPUT ARGUMENTS:
C     MPF          INTEGER (255) FIELD PARAMETER IDENTIFIERS
C
C ATTRIBUTES:
C   LANGUAGE: CRAY FORTRAN
C
C$$$
      DIMENSION MPF(255)
C - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      do i=1,255
        MPF(i)=0
      enddo
      IF(IPTV.EQ.1.OR.IPTV.EQ.2) THEN
        MPF(033)=1
        MPF(034)=1
        MPF(049)=1
        MPF(050)=1
        MPF(095)=1
        MPF(096)=1
        MPF(124)=1
        MPF(125)=1
        MPF(147)=1
        MPF(148)=1
        MPF(181)=1
        MPF(182)=1
        MPF(183)=1
        MPF(184)=1
        MPF(247)=1
        MPF(248)=1
        MPF(081)=2
        MPF(091)=2
        MPF(140)=2
        MPF(141)=2
        MPF(142)=2
        MPF(143)=2
        MPF(173)=2
        MPF(174)=2
        MPF(175)=2
        MPF(209)=2
        MPF(054)=3      ! PRECIPITABLE WATER (KG/M2)
        MPF(057)=3      ! EVAPORATION (KG/M2)
        MPF(058)=3      ! CLOUD ICE (KG/M2)
        MPF(059)=3      ! PRECIPITATION RATE (KG/M2/S)
        MPF(061)=3      ! TOTAL PRECIPITATION (KG/M2)
        MPF(062)=3      ! LARGE-SCALE PRECIPITATION (KG/M2)
        MPF(063)=3      ! CONVECTIVE PRECIPITATION (KG/M2)
        MPF(064)=3      ! WATER EQUIVALENT SNOWFALL RATE (KG/M2/S)
        MPF(065)=3      ! WATER EQUIVALENT OF SNOW DEPTH (KG/M2)
C       MPF(071)=3      ! TOTAL CLOUD COVER (PERCENT)
C       MPF(072)=3      ! CONVECTIVE CLOUD COVER (PERCENT)
        MPF(073)=3      ! LOW CLOUD COVER (PERCENT)
        MPF(074)=3      ! MIDDLE CLOUD COVER (PERCENT)
        MPF(075)=3      ! HIGH CLOUD COVER (PERCENT)
        MPF(076)=3      ! CLOUD WATER (KG/M2)
        MPF(079)=3      ! LARGE SCALE SNOW (KG/M2)
        MPF(086)=3      ! SOIL WETNESS (KG/M2)
        MPF(090)=3      ! RUNOFF (KG/M2)
        MPF(111)=3      ! NET SOLAR RADIATIVE FLUX AT SURFACE (W/M2)
        MPF(112)=3      ! NET LONGWAVE RADIATIVE FLUX AT SURFACE (W/M2)
        MPF(113)=3      ! NET SOLAR RADIATIVE FLUX AT TOP (W/M2)
        MPF(114)=3      ! NET LONGWAVE RADIATIVE FLUX AT TOP (W/M2)
        MPF(115)=3      ! NET LONGWAVE RADIATIVE FLUX (W/M2)
        MPF(116)=3      ! NET SOLAR RADIATIVE FLUX (W/M2)
        MPF(117)=3      ! TOTAL RADIATIVE FLUX (W/M2)
        MPF(121)=3      ! LATENT HEAT FLUX (W/M2)
        MPF(122)=3      ! SENSIBLE HEAT FLUX (W/M2)
        MPF(123)=3      ! BOUNDARY LAYER DISSIPATION (W/M2)
        MPF(144)=3      ! VOLUMETRIC SOIL MOISTURE CONTENT (FRACTION)
        MPF(145)=3      ! POTENTIAL EVAPORATION RATE (W/M2)
        MPF(146)=3      ! CLOUD WORKFUNCTION (J/KG)
        MPF(155)=3      ! GROUND HEAT FLUX (W/M2)
        MPF(156)=3      ! CONVECTIVE INHIBITION (W/M2)
        MPF(160)=3      ! CLEAR SKY UPWARD SOLAR FLUX (W/M2)
        MPF(161)=3      ! CLEAR SKY DOWNWARD SOLAR FLUX (W/M2)
        MPF(162)=3      ! CLEAR SKY UPWARD LONGWAVE FLUX (W/M2)
        MPF(163)=3      ! CLEAR SKY DOWNWARD LONGWAVE FLUX (W/M2)
        MPF(164)=3      ! CLOUD FORCING NET SOLAR FLUX (W/M2)
        MPF(165)=3      ! CLOUD FORCING NET LONGWAVE FLUX (W/M2)
        MPF(166)=3      ! VISIBLE BEAM DOWNWARD SOLAR FLUX (W/M2)
        MPF(167)=3      ! VISIBLE DIFFUSE DOWNWARD SOLAR FLUX (W/M2)
        MPF(168)=3      ! NEAR IR BEAM DOWNWARD SOLAR FLUX (W/M2)
        MPF(169)=3      ! NEAR IR DIFFUSE DOWNWARD SOLAR FLUX (W/M2)
        MPF(172)=3      ! MOMENTUM FLUX (N/M2)
        MPF(204)=3      ! DOWNWARD SOLAR RADIATIVE FLUX (W/M2)
        MPF(205)=3      ! DOWNWARD LONGWAVE RADIATIVE FLUX (W/M2)
        MPF(211)=3      ! UPWARD SOLAR RADIATIVE FLUX (W/M2)
        MPF(212)=3      ! UPWARD LONGWAVE RADIATIVE FLUX (W/M2)
        MPF(213)=3      ! NON-CONVECTIVE CLOUD COVER (PERCENT)
        MPF(214)=3      ! CONVECTIVE PRECIPITATION RATE (KG/M2/S)
        MPF(223)=3      ! PLANT CANOPY SURFACE WATER (KG/M2)
        MPF(228)=3      ! POTENTIAL EVAPORATION (KG/M2)
        MPF(229)=3      ! SNOW PHASE-CHANGE HEAT FLUX (W/M2)
        MPF(232)=3      ! DOWNWARD TOTAL RADIATION FLUX (W/M2)
        MPF(233)=3      ! UPWARD TOTAL RADIATION FLUX (W/M2)
        MPF(224)=3      ! BASEFLOW-GROUNDWATER RUNOFF (KG/M2)
        MPF(225)=3      ! STORM SURFACE RUNOFF (KG/M2)
        MPF(238)=3      ! SNOW COVER (PERCENT)
      ENDIF
C - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      RETURN
      END
c
c
      SUBROUTINE GBYTE (IN,IOUT,ISKIP,NBYTE)
      CALL GBYTES (IN,IOUT,ISKIP,NBYTE,0,1)
      RETURN
      END

      SUBROUTINE SBYTE (IOUT,IN,ISKIP,NBYTE)
      CALL SBYTES (IOUT,IN,ISKIP,NBYTE,0,1)
      RETURN
      END

      SUBROUTINE GBYTES (IN,IOUT,ISKIP,NBYTE,NSKIP,N)
C          Get bytes - unpack bits:  Extract arbitrary size values from a
C          packed bit string, right justifying each value in the unpacked
C          array.
      DIMENSION IN(*), IOUT(*)
C            IN    = packed array input
C            IO    = unpacked array output
C            ISKIP = initial number of bits to skip
C            NBYTE = number of bits to take
C            NSKIP = additional number of bits to skip on each iteration
C            N     = number of iterations
C************************************** MACHINE SPECIFIC CHANGES START HERE
C          Machine dependent information required:
C            LMWD   = Number of bits in a word on this machine
C            MASKS  = Set of word masks where the first element has only the
C                     right most bit set to 1, the second has the two, ...
C            LEFTSH = Shift left bits in word M to the by N bits
C            RGHTSH = Shift right
C            OR     = Logical OR (add) on this machine.
C            AND    = Logical AND (multiply) on this machine
C          This is for Sun UNIX Fortran, DEC Alpha, and RS6000
      PARAMETER (LMWD=32)
      DIMENSION MASKS(LMWD)
      SAVE    MASKS
      DATA    MASKS /'1'X,'3'X,'7'X,'F'X, '1F'X,'3F'X,'7F'X,'FF'X,
     +'1FF'X,'3FF'X,'7FF'X,'FFF'X, '1FFF'X,'3FFF'X,'7FFF'X,'FFFF'X,
     +'1FFFF'X,       '3FFFF'X,       '7FFFF'X,       'FFFFF'X,
     +'1FFFFF'X,      '3FFFFF'X,      '7FFFFF'X,      'FFFFFF'X,
     +'1FFFFFF'X,     '3FFFFFF'X,     '7FFFFFF'X,     'FFFFFFF'X,
     +'1FFFFFFF'X,    '3FFFFFFF'X,    '7FFFFFFF'X,    'FFFFFFFF'X/
C    +'1FFFFFFFF'X,   '3FFFFFFFF'X,   '7FFFFFFFF'X,   'FFFFFFFFF'X,
C    +'1FFFFFFFFF'X,  '3FFFFFFFFF'X,  '7FFFFFFFFF'X,  'FFFFFFFFFF'X,
C    +'1FFFFFFFFFF'X, '3FFFFFFFFFF'X, '7FFFFFFFFFF'X, 'FFFFFFFFFFF'X,
C    +'1FFFFFFFFFFF'X,'3FFFFFFFFFFF'X,'7FFFFFFFFFFF'X,'FFFFFFFFFFFF'X,
C    +'1FFFFFFFFFFFF'X,   '3FFFFFFFFFFFF'X,   '7FFFFFFFFFFFF'X,
C    +                                        'FFFFFFFFFFFFF'X,
C    +'1FFFFFFFFFFFFF'X,  '3FFFFFFFFFFFFF'X,  '7FFFFFFFFFFFFF'X,
C                                             'FFFFFFFFFFFFFF'X,
C    +'1FFFFFFFFFFFFFF'X, '3FFFFFFFFFFFFFF'X, '7FFFFFFFFFFFFFF'X,
C                                             'FFFFFFFFFFFFFFF'X,
C    +'1FFFFFFFFFFFFFFF'X,'3FFFFFFFFFFFFFFF'X,'7FFFFFFFFFFFFFFF'X,
C                                             'FFFFFFFFFFFFFFFF'X/
C          IBM PC using Microsoft Fortran uses different syntax:
C     DATA MASKS/16#1,16#3,16#7,16#F,16#1F,16#3F,16#7F,16#FF,
C    + 16#1FF,16#3FF,16#7FF,16#FFF,16#1FFF,16#3FFF,16#7FFF,16#FFFF,
C    + 16#1FFFF,16#3FFFF,16#7FFFF,16#FFFFF,16#1FFFFF,16#3FFFFF,
C    + 16#7FFFFF,16#FFFFFF,16#1FFFFFF,16#3FFFFFF,16#7FFFFFF,16#FFFFFFF,
C    + 16#1FFFFFFF,16#3FFFFFFF,16#7FFFFFFF,16#FFFFFFFF/
      INTEGER RGHTSH, OR, AND
      LEFTSH(M,N) = ISHFT(M,N)
      RGHTSH(M,N) = ISHFT(M,-N)
C     OR(M,N)  = M.OR.N
C     AND(M,N) = M.AND.N
C     OR(M,N)  = IOR(M,N)
C     AND(M,N) = IAND(M,N)
C************************************** MACHINE SPECIFIC CHANGES END HERE
C          History:  written by Robert C. Gammill, jul 1972.


C          NBYTE must be less than or equal to LMWD
      ICON = LMWD-NBYTE
      IF (ICON.LT.0) RETURN
      MASK = MASKS (NBYTE)
C          INDEX  = number of words into IN before the next "byte" appears
C          II     = number of bits the "byte" is from the left side of the word
C          ISTEP  = number of bits from the start of one "byte" to the next
C          IWORDS = number of words to skip from one "byte" to the next
C          IBITS  = number of bits to skip after skipping IWORDS
C          MOVER  = number of bits to the right, a byte must be moved to be
C                   right adjusted
      INDEX = ISKIP/LMWD
      II    = MOD (ISKIP,LMWD)
      ISTEP = NBYTE+NSKIP
      IWORDS= ISTEP/LMWD
      IBITS = MOD (ISTEP,LMWD)

      DO 6 I=1,N                                                                
      MOVER = ICON-II
      IF (MOVER) 2,3,4

C          The "byte" is split across a word break.
    2 MOVEL = -MOVER
      MOVER = LMWD-MOVEL
      NP1 = LEFTSH (IN(INDEX+1),MOVEL)
      NP2 = RGHTSH (IN(INDEX+2),MOVER)
      IOUT(I) = AND (OR (NP1,NP2) , MASK)
      GO TO 5                                                                   

C          The "byte" is already right adjusted.
    3 IOUT(I) = AND (IN (INDEX+1) , MASK)
      GO TO 5                                                                   

C          Right adjust the "byte".
    4 IOUT(I) = AND (RGHTSH (IN (INDEX+1),MOVER) , MASK)

    5 II = II+IBITS
      INDEX = INDEX+IWORDS
      IF (II .LT. LMWD) GO TO 6
      II = II-LMWD
      INDEX = INDEX+1
    6 CONTINUE                                                                  

      RETURN                                                                    
      END                                                                       

      SUBROUTINE SBYTES (IOUT,IN,ISKIP,NBYTE,NSKIP,N)
C          Store bytes - pack bits:  Put arbitrary size values into a
C          packed bit string, taking the low order bits from each value
C          in the unpacked array.
      DIMENSION IN(*), IOUT(*)
C            IOUT  = packed array output
C            IN    = unpacked array input
C            ISKIP = initial number of bits to skip
C            NBYTE = number of bits to pack
C            NSKIP = additional number of bits to skip on each iteration
C            N     = number of iterations
C************************************** MACHINE SPECIFIC CHANGES START HERE
C          Machine dependent information required:
C            LMWD   = Number of bits in a word on this machine
C            MASKS  = Set of word masks where the first element has only the
C                     right most bit set to 1, the second has the two, ...
C            LEFTSH = Shift left bits in word M to the by N bits
C            RGHTSH = Shift right
C            OR     = Logical OR (add) on this machine
C            AND    = Logical AND (multiply) on this machine
C            NOT    = Logical NOT (negation) on this machine
C          This is for Sun UNIX Fortran
      PARAMETER (LMWD=32)
      DIMENSION MASKS(LMWD)
      SAVE    MASKS
      DATA    MASKS /'1'X,'3'X,'7'X,'F'X, '1F'X,'3F'X,'7F'X,'FF'X,
     +'1FF'X,'3FF'X,'7FF'X,'FFF'X, '1FFF'X,'3FFF'X,'7FFF'X,'FFFF'X,
     +'1FFFF'X,       '3FFFF'X,       '7FFFF'X,       'FFFFF'X,
     +'1FFFFF'X,      '3FFFFF'X,      '7FFFFF'X,      'FFFFFF'X,
     +'1FFFFFF'X,     '3FFFFFF'X,     '7FFFFFF'X,     'FFFFFFF'X,
     +'1FFFFFFF'X,    '3FFFFFFF'X,    '7FFFFFFF'X,    'FFFFFFFF'X/
      INTEGER RGHTSH, OR, AND
      LEFTSH(M,N) = ISHFT(M,N)
      RGHTSH(M,N) = ISHFT(M,-N)
C     OR(M,N)  = M.OR.N
C     AND(M,N) = M.AND.N
C     OR(M,N)  = IOR(M,N)
C     AND(M,N) = IAND(M,N)
C     NOT(M)   = .NOT.M
C***********************************************************************        

C          NBYTE must be less than or equal to LMWD
      ICON = LMWD-NBYTE
      IF (ICON .LT. 0) RETURN
      MASK = MASKS(NBYTE)
C          INDEX  = number of words into IOUT the next "byte" is to be stored
C          II     = number of bits in from the left side of the word to store it
C          ISTEP  = number of bits from the start of one "byte" to the next
C          IWORDS = number of words to skip from one "byte" to the next
C          IBITS  = number of bits to skip after skipping IWORDS
C          MOVER  = number of bits to the right, a byte must be moved to be
C                   right adjusted
      INDEX = ISKIP/LMWD
      II    = MOD(ISKIP,LMWD)
      ISTEP = NBYTE+NSKIP
      IWORDS = ISTEP/LMWD
      IBITS = MOD(ISTEP,LMWD)

      DO 6 I=1,N                                                                
      J = AND (MASK,IN(I))
      MOVEL = ICON-II
      IF (MOVEL) 2,3,4

C          The "byte" is to be split across a word break
    2 MSK = MASKS (NBYTE+MOVEL)
      IOUT(INDEX+1) = OR (AND(NOT(MSK),IOUT(INDEX+1)),RGHTSH(J,-MOVEL))
      ITEMP = AND (MASKS(LMWD+MOVEL),IOUT(INDEX+2))
      IOUT(INDEX+2) = OR(ITEMP,LEFTSH(J,LMWD+MOVEL))
      GO TO 5                                                                   

C          The "byte" is to be stored right-adjusted
    3 IOUT(INDEX+1) = OR ( AND (NOT(MASK),IOUT(INDEX+1)) , J)
      GO TO 5                                                                   

C          The "byte" is to be stored in middle of word, so shift left.
    4 MSK = LEFTSH(MASK,MOVEL)
      IOUT(INDEX+1) = OR(AND(NOT(MSK),IOUT(INDEX+1)),LEFTSH(J,MOVEL))

    5 II = II+IBITS
      INDEX = INDEX+IWORDS
      IF (II .LT. LMWD) GO TO 6
      II = II-LMWD
      INDEX = INDEX+1
    6 CONTINUE

      RETURN                                                                    
      END                                                                       
c
c ************************************************************************
        subroutine swap4(in,io,nn)
c swaps bytes in groups of 4 to compensate for byte swapping within
c    words which occurs on DEC (VAX) and PC machines.
c
c in - input array to be swapped
c io - ouput array with bytes swapped
c nn - number of bytes to be swapped
c       logical*1 in(1),io(1),ih          ! non_ALPHA
        character*1 in(*),io(*),ih        ! DEC_ALPHA
c        character*1 in(1),io(1),ih        ! DEC_ALPHA
        do 10 i=1,nn,4
        ih=in(i)
        io(i)=in(i+3)
        io(i+3)=ih
        ih=in(i+1)
        io(i+1)=in(i+2)
        io(i+2)=ih
   10   continue
        return
        end

C *************************************************************************
        subroutine swap2(in,io,nn)
c swaps bytes in groups of 2 to compensate for byte swapping within
c    words which occurs on DEC (VAX) and PC machines.
c
c in - input array to be swapped
c io - ouput array with bytes swapped
c nn - number of bytes to be swapped
c       logical*1 in(1),io(1),ih           ! non_ALPHA
        character*1 in(1),io(1),ih         ! DEC_ALPHA
        do 10 i=1,nn,2
        ih=in(i)
        io(i)=in(i+1)
        io(i+1)=ih
   10   continue
        return
        end

c        subroutine filt(m,n,in)
c        logical*1 m(1),n(1),l,u
c        data l/32/,u/127/
c        do 10 i=1,in
c        n(i)=m(i)
c        if(n(i).lt.  l) n(i)=l
c        if(n(i).gt.  u) n(i)=u
c   10   continue
c        return
c        end

c **************************************************************************
c
c This version of anyGRIB1.f has changed the arguments passed to/from
c the subroutine calls.  It works with the GRIB decoders in this directory.
c
c
c Revision History:
c  Jan 03, 1997 -- minor, fix P1,P2 unpacking in pds_GRIB1
c  Nov 13, 1996 -- reverse the order of lines 1349,1350
c  Sep 11, 1996 -- fix gds_GRIB1 subroutine for special grids
c  Feb 12, 1996 -- initialize rmax,rmin to be -999999.0,999999.0
c  Jan  5, 1996 -- fix iskp in BMS.
c  Nov 30, 1995 -- fix BMS section when using a bitmap.
c  Aug 31, 1995 -- major change, to unpack THINNED grids.
c  Jul 10, 1995 -- minor change, to fix bit map section.
c  Feb 17, 1995 -- minor change, to correct GDS LOV computation.
c
c Systems Tested:
c  SUN,CRAY,PC/MS-DOS,DEC/ALPHA,IBM/RS6000
c==================================================================
c The subroutines below are to deocode GRIB format
c
      subroutine bds_GRIB1(iprnt,gribedi,nword,inoff,dscalf,ierr,
     &  nmax,npoints,rc00,rmax,rmin,numgd,mapuse,mapb,pds,ingrib,
     &  ix,datan)
c
c iprnt:   print out flag, 0=NO printing
c gribedi: GRIB edition number, 1, 0, -1
c nword:   length of ingrib
c inoff:   offset number of bits 
c dscalf:  decimal scale factor, from PDS
c ierr: error code, 
c       0=no error, 
c      -1=number of bits per data value exceeds word length or maximum allowed
c      -2=input array holds fewer coded values than expected,
c      -3=output array too small to hold decoded data values
c      -4=currently undefined GRIB Edition 1 option selected,
c      -8=constant field, npoints cannot be computed, use number of grids from 
c         gds to fill datan, the reference value is in datan(1) 
c      -9=blank record
c     -11=unknown predefined grid
c     -12=complex packing of spher harmonic coeff, to be developed
c       1=no '7777' at the end
c nmax:    maximum dimension of datan
c npoints: actual number of array unpacked
c rc00:
c rmax:    maximum value of array datan
c rmin:    minimum value of array datan
c numgd:   number of actual grids used, counted from BMP bitmap
c mapuse:  if 0, do not use mapb; otherwise, mapb(mapuse)
c mapb:    bit map from BMS or generated
c pds:     PDS array
c ingrib:  integer array containing GRIB record
c ix:      integer array of unpacked array, of dimension (nnx*nny)
c datan:   returned array of length (nmax=nnx*nny)
c
c bdslen: length of BDS
c bdsflg: BDS flag (byte 4 bits 1-4 and byte 14)
c bdsub:  number of unused bits in BDS
c bdse:   BDS scale factor
c bdsref: BDS reference value
c bdsn:   number of bits used for packing
c
      dimension ingrib(nword),datan(nmax)
      logical*1 mapb(nmax)
      real*4    refval, bscalf
      integer*4 bdslen,bdsflg(8),bdsub,bdse,bdsn,bdsn1
      integer*4 pds(22),ix(nmax),refa,refb,gribedi
      integer*4 N,P,JL,KL,ML
      integer*4 ifov(361),kbdsn2(361)  ! 361 is the maximum nny
      character*72 bdsnam(6)
      data ifirst/0/
c
c     save
c
      if(ifirst.eq.0) then
        bdsnam(1)=' BDS length'                       ! byte 1-3
        bdsnam(2)=' BDS flag'                         ! byte 4, bits 1-4 & byte 14
        bdsnam(3)=' BDS number of unused bits'        ! byte 4, bits 5-8
        bdsnam(4)=' BDS scale factor'                 ! byte 5-6
        bdsnam(5)=' BDS reference value'              ! byte 7-10
        bdsnam(6)=' BDS number of bits for packing'   ! byte 11
        ifirst=9
      endif
      scalten = 10**dscalf
c
c unpack BDS section
c
      iskp=inoff
      call gbyte(ingrib,bdslen,iskp,24)
      if(iprnt.ne.0) write(*,*) '     bdslen: ',bdslen

      do i=1,8
        bdsflg(i)=0
      enddo
      iskp=iskp+24
      if(gribedi.eq.-1) then
        call gbyte(ingrib,bdsflg(1),iskp+3,1)
        call gbyte(ingrib,bdsflg(2),iskp+2,1)
        call gbyte(ingrib,bdsflg(3),iskp+1,1)
        call gbyte(ingrib,bdsflg(4),iskp,1)
        call gbyte(ingrib,bdsub,iskp+4,4)
        if(bdsflg(4).eq.1) then
          call gbyte(ingrib,bdsflg(5),iskp+10*8,1)       ! should be zero
          call gbyte(ingrib,bdsflg(6),iskp+10*8+1,1)
          call gbyte(ingrib,bdsflg(7),iskp+10*8+2,1)
          call gbyte(ingrib,bdsflg(8),iskp+10*8+3,1)
        endif
      else
        call gbyte(ingrib,bdsflg(1),iskp,1)
        call gbyte(ingrib,bdsflg(2),iskp+1,1)
        call gbyte(ingrib,bdsflg(3),iskp+2,1)
        call gbyte(ingrib,bdsflg(4),iskp+3,1)
        call gbyte(ingrib,bdsub,iskp+4,4)
        if(bdsflg(4).eq.1) then
          call gbyte(ingrib,bdsflg(5),iskp+10*8,1)       ! should be zero
          call gbyte(ingrib,bdsflg(6),iskp+10*8+1,1)
          call gbyte(ingrib,bdsflg(7),iskp+10*8+2,1)
          call gbyte(ingrib,bdsflg(8),iskp+10*8+3,1)
        endif
      endif
      if(iprnt.ne.0) 
     &  write(*,*) '     bdsflg: ',(bdsflg(i),i=1,8)
      if(iprnt.ne.0) write(*,*) '     bdsub:  ',bdsub
c
c scale factor      ! byte 5-6
c
      iskp=iskp+8
      call gbyte(ingrib,bdse,iskp,16)
      if(bdse.gt.32768) bdse=32768-bdse
      bscalf=float(bdse)
      scaltwo=2**(bscalf)
      if(iprnt.ne.0) write(*,*) '     bdse:   ',bdse
c
c reference value   ! byte 7-10
c
      iskp=iskp+16
      call gbyte(ingrib,isign,iskp,1)
      call gbyte(ingrib,refa,iskp+1,7)
      call gbyte(ingrib,refb,iskp+8,24)
      rdum=refb
      refval=(-1)**isign * rdum * 16**(refa-64) / 2**24
c
c if bdsflg(3) == 1, refval needs to be round into an integer
c
      if(iprnt.ne.0) 
     &  write(*,*) '     bdsref: ',isign,refa,refb,refval
c
c number of bits used for packing   ! byte 11
c if bdsn == 0, constatn field
c
      iskp=iskp+32
      call gbyte(ingrib,bdsn,iskp,8)
      if(iprnt.ne.0) then
        write(*,*) '     bdsn:   ',bdsn
        write(*,*) ' refval,bscalf,dscalf: ',refval,bscalf,dscalf
      endif

c
c compute or find out how many points used
c
      if(bdsflg(4).eq.0) then        ! no flag in byte 14
c
c if (bdslen-11) is odd, the last byte may be padded
c
        idum=(bdslen-11)*8-bdsub
        if(bdsn.eq.0) then           ! constant field when bdsn=0
c
c Ierr: -8=constant field, 
c          npoints cannot be computed, use number of grids from 
c          gds to fill datan, the reference value is in datan(1) 
c
          npoints=1
          datan(1)=refval
          rmax=refval
          rmin=refval
          ijrmax=-9999
          ijrmin=-9999
          ierr=-8
          return
        else
          if(bdsflg(7).eq.0 .and. mapuse.gt.0) then                   !951130
            npoints=numgd                                             !951130
          else                                                        !951130
            npoints=idum/bdsn                                         !951130
          endif                                                       !951130
          if(mod(idum,bdsn).ne.0) then
c                                                                     !950710
c fix NMC GRIB encoder error for 1x1 degree (360x181=65160) grids     !950710
c                                                                     !950710
            if(abs(npoints-65160).le.2) then                          !950710
              npoints=65160                                           !950710
            else                                                      !950710
c              write(*,*) 'WARNING! length of packed bits can not ',   !950710
c     &         'be divided by bdsn ',idum,bdsn,npoints                !950710
c              write(*,815)idum,bdsn,npoints,pds(7),pds(9)              !950710
c 815          format('WARNING! length of packed bits can not ',        !950710
c     &         'be divided by bdsn ',5i8)                              !950710
            endif                                                     !950710
          endif
        endif
      else                          ! extended flags in byte 14
c
c   Extended BDS flags:
c                        NAME        BYTE LOCATION
c                        bdsn1       11
c                        n1          12-13
c                                    14
c                        n2          15-16
c                        np1         17-18
c                        np2         19-20
c                                    21
c                        bdsn2(np1)  22-(xx-1)  one byte each  xx=np1+22
c                        bmap2(np2)  xx-(n1-1)
c                        mp1(np1)    n1-(n2-1)
c                        mp2(np2)    n2-...
c
        bdsn1=bdsn
        iskp=iskp+8
        call gbyte(ingrib,n1,iskp,16)
        iskp=iskp+16+8
        call gbyte(ingrib,n2,iskp,16)
        iskp=iskp+16
        call gbyte(ingrib,np1,iskp,16)
        iskp=iskp+16
        call gbyte(ingrib,np2,iskp,16)
        iskp=iskp+16+8
        if(iprnt.ne.0) then
          write(*,*) '    bdsn1:   ',bdsn1
          write(*,*) '       n1:   ',n1
          write(*,*) '       n2:   ',n2
          write(*,*) '      np1:   ',np1
          write(*,*) '      np2:   ',np2
        endif
        if(bdsflg(7).eq.0 .and. mapuse.gt.0) then
          npoints=numgd
        else
          npoints=-9999  ! to be determined counting secondary bit map
        endif
      endif
c
c All flags decoded, unpack the data
c
c bdsflg(1) = 0, grid point data
c             1, spherical harmonic coefficients
c bdsflg(2) = 0, simple packing
c             1, second order ("complex") packing
c bdsflg(3) = 0, original data were floating point values
c             1, original data were integer values
c bdsflg(4) = 0, no additional flags at oct 14
c             1, octet 14 contains flag bits 5-12
c bdsflg(5) = 0, reserved, should be 0
c bdsflg(6) = 0, single datum at each grid point
c             1, matrix of values at each grid point
c bdsflg(7) = 0, no secondary bit maps
c             1, secondary bit maps present
c bdsflg(8) = 0, second order values have constant width
c             1, second order values have different width
c
      if(bdsflg(2).eq.0) then        ! simple packing
        if(iprnt.ne.0) 
     &    write(*,*) ' total number of grids: ',npoints, nmax
        if(npoints.gt.nmax) then
          ierr=-3
          write(*,*) 'ERROR in bds_GRIB1, ierr = ',ierr
          return
        endif
        if(bdsflg(1).eq.0) then      ! simple packing, grid point data
          rc00=0.
          iskp=11*8+inoff
          call gbytes(ingrib,ix,iskp,bdsn,0,npoints)
          do i=1,npoints
            datan(i)=(refval+ix(i)*scaltwo)/scalten
          enddo
        elseif(bdsflg(1).eq.1) then  ! simple packing, spher harmonic coeff
          iskp=11*8+inoff
          call gbyte(ingrib,isign,iskp,1)
          call gbyte(ingrib,refa,iskp+1,7)
          call gbyte(ingrib,refb,iskp+8,24)
          rdum=refb
          rc00=(-1)**isign * rdum * 16**(refa-64) / 2**24
          if(iprnt.ne.0)
     &      write(*,*) ' real part of coeff (0,0): ',rc00
          iskp=iskp+32
          call gbytes(ingrib,ix,iskp,bdsn,0,npoints-1)
          datan(1)=rc00
          do i=2,npoints
            datan(i)=(refval+ix(i-1)*scaltwo)/scalten
          enddo
        else
        endif
      elseif(bdsflg(2).eq.1) then    ! complex packing
        if(bdsflg(1).eq.0) then      ! complex packing, grid point data
c
c octet
c 22 -- (xx-1): widths in bits of the 2nd order packed values
c xx -- (n1-1): secondary bit maps, padded to an even number octet
c n1 -- (n2-1): np1 local minima, each is bdsn1 wide, padded
c n2 -- bdslen: np2 second order packed values, padded at the end
c
c if bdsflg(7)==0, [22--(n1-1)]: widths in bits of 2nd order packed values
c                  this is "row by row (or column by column) packing"
c
c if bdsflg(8)==0, [22--22]: width of 2nd order packed values
c if bdsflg(8)==1, but width==0, this sub-section is constant
c
          if(bdsflg(7).ne.0) then
            write(*,*) '** Secondary Bit Map in use, no unpacking yet!'
            npoints=-9999
            return
          endif
          k1off=inoff+(n1-1)*8
          do k1=1,np1
            call gbyte(ingrib,ifov(k1),k1off,bdsn1)
c           if(k1.ge.(np1-10))write(*,*) ' ifov: ',k1,k1off,ifov(k1)
            k1off=k1off+bdsn1
          enddo
c
c unpack second order value
c
          k2off=inoff+21*8         ! width starts from byte 22
          do k2=1,np1
            call gbyte(ingrib,kbdsn2(k2),k2off,8)
            k2off=k2off+8
c           if(k2.ge.(np1-10))
c    &        write(*,*) ' kbdsn2: ',k2,k2off,kbdsn2(k2)
          enddo
c
c         write(*,*) ' unpacking 2nd order value: ',numgd,mapuse
          k3off=inoff+(n2-1)*8
          nt=0
          kone=mapuse/np1
          do k2=1,mapuse
            kk=((k2-1)/kone)+1
            if(mapb(k2)) then
              if(kbdsn2(kk).eq.0) then
                isov=0
              else
                call gbyte(ingrib,isov,k3off,kbdsn2(kk))
                k3off=k3off+kbdsn2(kk)
              endif
              nt=nt+1
              datan(nt)=(refval+float(ifov(kk)+isov)*scaltwo)/scalten
c             if(nt.ge.130 .and. nt.le.140) 
c    &          write(*,*) ' isov: ',k2,nt,datan(nt),kk,ifov(kk),isov
            endif
          enddo
c         write(*,*) ' unpacking 2nd done ',numgd,nt,k2off,k3off
          if(numgd.ne.nt) write(*,*) '**WARNING, number of grids',
     &      'mismatch in 2nd order unpacking! ',numgd,nt
          npoints=nt
        elseif(bdsflg(1).eq.1) then  ! complex packing, spher harmonic coeff
          ierr=-12
          if((pds(3).ne.98) .or. (gribedi.eq.1)) then
            write(*,*) '**Complex packing of non-ECMWF spectral'
            write(*,*) '**or GRIB Edition 1, NO unpacking yet!'
            npoints=-9999
            return
          else
c
c                  Second Order Packing Formats:
c
c               GRIB 1              ECMWF(version 0, x)
c              NAME  LOCATION      LOCATION   NAME
c                N1   12-13         12-13     N
c                     14            14-15     P
c                N2   15-16         16        JL
c                P1   17-18         17        KL
c                P2   19-20         18        ML
c                     21            19-(N-1)  binary data
c         BIT WIDTH   22-(P1-1)     N-        packed binary data
c Secondary BIT MAP   xx-(N1-1)
c          MP1(NP1)   N1-(N2-1)
c          MP2(NP2)   N2-
c
c ECMWF spherical harmonic complex packing
c
            iskp=11*8+inoff
            call gbyte(ingrib,N,iskp,16)
            iskp=iskp+16
            call gbyte(ingrib,P,iskp,16)
            iskp=iskp+16
            call gbyte(ingrib,JL,iskp,8)
            iskp=iskp+8
            call gbyte(ingrib,KL,iskp,8)
            iskp=iskp+8
            call gbyte(ingrib,ML,iskp,8)
            iskp=iskp+8
c
c compute how many values to unpack
c
            idum=bdslen-19
            num=idum/4
            if(mod(idum,4).ne.0) then
              write(*,*) 'UNEVEN ECMWF spherical harmonic unpacking!'
              write(*,*) '    ',bdslen,idum,num
              return
            endif
            if(iprnt.ne.0) write(*,*) 'ECMWF spherical harmonic: ',num
c
c unpack every 4 bytes into real
c
            do i=1,num
              iskp=iskp+(i-1)*32
              call gbyte(ingrib,isign,iskp,1)
              call gbyte(ingrib,refa,iskp+1,7)
              call gbyte(ingrib,refb,iskp+8,24)
              rdum=refb
              datan(i)=(-1)**isign * rdum * 16**(refa-64) / 2**24
            enddo
            write(*,*) '**PARTIALLY DONE of '
            write(*,*) '**ECMWF spherical harmonic complex unpacking!'
            write(*,*) '**DO NOT use yet!'
            npoints=-9999
            return
          endif
        else
        endif
      else
        write(*,*) '****ERROR, unknown packing, grid/spectral!'
        stop'UNKNOWN PACKING ERROR'
      endif
c
c after unpacking, locate the maximum and minimum among datan
c
      rmax=-999999.
      rmin=999999.
      do i=1,npoints
        if(datan(i).gt.rmax) then
          ijrmax=i
          rmax=datan(i)
        endif
        if(datan(i).lt.rmin) then
          rmin=datan(i)
          ijrmin=i
        endif
      enddo
      if(iprnt.ne.0) then
        write(*,*) ' rmax, at ij = ',rmax,ijrmax
        write(*,*) ' rmin, at ij = ',rmin,ijrmin
      endif
      ierr=0
 9998 return
      end
c
c
      subroutine bms_GRIB1(iprnt,nword,nmax,inoff,
     &  kbms,nx,ny,la1,lo1,ierr,numb,mapuse,pds,
     &  ingrib,mapb)
c
c This is a generic GRIB Edition 1 BMS decoder
c
c iprnt:  print out flag, 0=NO printing
c lenpds: input, length of PDS in bytes
c ipds:   input, integer array containing GRIB PDS
c nword:  input, number of words used in ipds
c kbms:   input, GDS exists if kbms=1
c numb:   actual number of grids counted from bitmap
c mapuse: number of grids expected from grid specification
c ierr:   returned error code, 
c         0=no error, -11=unknown predefined grid
c
      dimension ingrib(nword),igrd37(73)
      logical*1 mapb(nmax)
      character*72 bmsnam(3)
      integer*4 kbms,iskp,idum,pds(22)
      integer*4 bmslen,bmsub,bmstbl
      integer*4 nx,ny,la1,lo1,mapuse
c
      data ifirst/0/
c
c igrd37 is good for grid_id 37-44
      data igrd37/73, 73, 73, 
     &    73, 73, 73, 73, 73, 72, 72, 72, 71, 71,
     &    71, 70, 70, 69, 69, 68, 67, 67, 66, 65, 
     &    65, 64, 63, 62, 61, 60, 60, 59, 58, 57,
     &    56, 55, 54, 52, 51, 50, 49, 48, 47, 45, 
     &    44, 43, 42, 40, 39, 38, 36, 35, 33, 32, 
     &    30, 29, 28, 26, 25, 23, 22, 20, 19, 17,
     &    16, 14, 12, 11,  9,  8,  6,  5,  3,  2/                 
c
      ierr=0
      if(ifirst.eq.0) then
        bmsnam(1)=' 1 = BMS length'                               !byte 1-3
        bmsnam(2)=' 2 = number of unused bits in BMS'             !byte 4
        bmsnam(3)=' 3 = BMS Table Reference'                      !byte 5-6
        ifirst=9
      endif
c
c process BMS
c
      mapuse=0
      numb=0
      if(kbms.ne.0) then
        iskp=inoff
        call gbyte(ingrib,bmslen,iskp,24)
        iskp=iskp+24
        call gbyte(ingrib,bmsub,iskp,8)
        iskp=iskp+8
        call gbyte(ingrib,bmstbl,iskp,16)                         !950710
        iskp=iskp+16                                           !960105
        if(iprnt.ne.0) then
          write(*,*) ' BMS length: ',bmslen
          write(*,*) ' BMS number of unused bits: ',bmsub
          write(*,*) ' BMS Table Reference: ',bmstbl
          write(*,*) ' BMS LA1,LO1: ',la1,lo1
        endif
c                                                                 !950710
c if bmstbl =0, a bit map follows, otherwise see Center Specs     !950710
c                                                                 !950710
        if(bmstbl.eq.0) then                                      !950710
          mapuse=bmslen*8-6*8-bmsub                               !950710
c         call gbytes(ingrib,mapb,iskp,1,0,numb)                  !950710
          do i=1,mapuse                                           !951130
            mapb(i)=.false.
            call gbyte(ingrib,idum,iskp,1)
            if(idum.eq.1) then                                    !951130
              mapb(i)=.true.                                      !951130
              numb=numb+1                                         !951130
            endif                                                 !951130
            iskp=iskp+1                                           !951130
          enddo
c         write(*,*) ' BMS Numb bitmap:     ',numb,mapuse
          return
        endif
      endif
c
c pds(3) == 07, US NMC
c           08, US NWS Telecomms Gateway
c           09, US NWS Field Stations
c           34, Japan Meteorological Agency
c           52, US NHC, Miami
c           54, Canadian
c           57, US Air Forece, GWC
c           58, US FNOC
c           59, US NOAA FSL
c           74, UKMET
c           85, French Weather Service - Toulouse
c           97, European Space Agency (ESA)
c           98, ECMWF
c           99, DeBilt, Netherlands
c
c     if(pds(3).ne.7) return
c
c NMC's predefined grids: make an rectangular mapb
c
      if(pds(5).eq.21 .or. pds(5).eq.22) then
        mapuse=37*37
        numb=1333
        m=0
        k=0
        do j=1,ny+1
          do i=1,nx
            m=m+1
            mapb(m)=.true.
            if(j.eq.(ny+1) .and. i.ge.2) mapb(m)=.false.
            if(mapb(m)) k=k+1
          enddo
        enddo
        if(k.ne.numb) write(*,*) '**WARNING! in bms_GRIB1-1 ',k,numb
        if(m.ne.mapuse) write(*,*) '**WARNING! in bms_GRIB1-2 ',m,mapuse
      elseif(pds(5).eq.23 .or. pds(5).eq.24) then
        mapuse=37*37
        numb=1333
        m=0
        k=0
        do j=1,ny+1
          do i=1,nx
            m=m+1
            mapb(m)=.true.
            if(j.eq.1 .and. i.ge.2) mapb(m)=.false.
            if(mapb(m)) k=k+1
          enddo
        enddo
        if(k.ne.numb) write(*,*) '**WARNING! in bms_GRIB1-3 ',k,numb
        if(m.ne.mapuse) write(*,*) '**WARNING! in bms_GRIB1-4 ',m,mapuse
      elseif(pds(5).eq.25) then
        numb=1297
        mapuse=72*19
        m=0
        k=0
        do j=1,ny+1
          do i=1,nx
            m=m+1
            mapb(m)=.true.
            if(j.eq.(ny+1) .and. i.ge.2) mapb(m)=.false.
            if(mapb(m)) k=k+1
          enddo
        enddo
        if(k.ne.numb) write(*,*) '****WARNING! in bms_GRIB1-5 ',k,numb
        if(m.ne.mapuse) write(*,*) '**WARNING! in bms_GRIB1-6 ',m,mapuse
      elseif(pds(5).eq.26) then
        numb=1297
        m=0
        k=0
        do j=1,ny+1
          do i=1,nx
            m=m+1
            mapb(m)=.true.
            if(j.eq.1 .and. i.ge.2) mapb(m)=.false.
            if(mapb(m)) k=k+1
          enddo
        enddo
        if(k.ne.numb) write(*,*) '**WARNING! in bms_GRIB1-7 ',k,numb
        if(m.ne.mapuse) write(*,*) '**WARNING! in bms_GRIB1-8 ',m,mapuse
      elseif(pds(5).ge.37 .and. pds(5).le.44) then
        numb=3447
        mapuse=73*73
        m=0
        k=0
        do j=1,73
          jj=j
          if(la1.eq.-90000) jj=74-j
          do i=1,73
            m=m+1
            mapb(m)=.false.
            if(i.le.igrd37(jj)) mapb(m)=.true.
            if(mapb(m)) k=k+1
          enddo
        enddo
        if(k.ne.numb) write(*,*) '**WARNING! in bms_GRIB1-37 ',k,numb
        if(m.ne.mapuse) 
     &    write(*,*) '**WARNING! in bms_GRIB1-38 ',m,mapuse
      elseif(pds(5).eq.50) then
        numb=964
        mapuse=36*33
        m=0
        k=0
        do j=1,4
          do i=1,36
            m=m+1
            mapb(m)=.false.
            if(i.ge.8 .and. i.le.29) mapb(m)=.true.
            if(mapb(m)) k=k+1
          enddo
        enddo
        do j=5,8
          do i=1,36
            m=m+1
            mapb(m)=.false.
            if(i.ge.7 .and. i.le.30) mapb(m)=.true.
            if(mapb(m)) k=k+1
          enddo
        enddo
        do j=9,12
          do i=1,36
            m=m+1
            mapb(m)=.false.
            if(i.ge.6 .and. i.le.31) mapb(m)=.true.
            if(mapb(m)) k=k+1
          enddo
        enddo
        do j=13,16
          do i=1,36
            m=m+1
            mapb(m)=.false.
            if(i.ge.5 .and. i.le.32) mapb(m)=.true.
            if(mapb(m)) k=k+1
          enddo
        enddo
        do j=17,20
          do i=1,36
            m=m+1
            mapb(m)=.false.
            if(i.ge.4 .and. i.le.33) mapb(m)=.true.
            if(mapb(m)) k=k+1
          enddo
        enddo
        do j=21,24
          do i=1,36
            m=m+1
            mapb(m)=.false.
            if(i.ge.3 .and. i.le.34) mapb(m)=.true.
            if(mapb(m)) k=k+1
          enddo
        enddo
        do j=25,28
          do i=1,36
            m=m+1
            mapb(m)=.false.
            if(i.ge.2 .and. i.le.35) mapb(m)=.true.
            if(mapb(m)) k=k+1
          enddo
        enddo
        do j=29,33
          do i=1,36
            m=m+1
            mapb(m)=.true.
            k=k+1
          enddo
        enddo
        if(k.ne.numb) write(*,*) '**WARNING! in bms_GRIB1-50 ',k,numb
        if(m.ne.mapuse) write(*,*)'**WARNING! in bms_GRIB1-51 ',m,mapuse
      elseif(pds(5).eq.61 .or. pds(5).eq.62) then
        numb=4096
        mapuse=91*46
        m=0
        k=0
        do j=1,ny+1
          do i=1,nx
            m=m+1
            mapb(m)=.true.
            if(j.eq.(ny+1) .and. i.ge.2) mapb(m)=.false.
            if(mapb(m)) k=k+1
          enddo
        enddo
        if(k.ne.numb) write(*,*) '**WARNING! in bms_GRIB1-9 ',k,numb
        if(m.ne.mapuse) write(*,*)'**WARNING! in bms_GRIB1-10 ',m,mapuse
      elseif(pds(5).eq.63 .or. pds(5).eq.64) then
        numb=4096
        mapuse=91*46
        m=0
        k=0
        do j=1,ny+1
          do i=1,nx
            m=m+1
            mapb(m)=.true.
            if(j.eq.1 .and. i.ge.2) mapb(m)=.false.
            if(mapb(m)) k=k+1
          enddo
        enddo
        if(k.ne.numb) write(*,*) '**WARNING! in bms_GRIB1-11 ',k,numb
        if(m.ne.mapuse) write(*,*)'**WARNING! in bms_GRIB1-12 ',m,mapuse
      else
        numb=0
        ierr=-11
      endif
      return
      end
c
c --------------------------------------------------------------------
      subroutine gds_GRIB1(iprnt,gribedi,nword,inoff,
     &  kgds,ierr,nx,ny,la1,lo1,pds,igds,lov,dx,dy,igds6)
c
c This is a generic GRIB Edition 1 GDS decoder
c
c iprnt:  print out flag, 0=NO printing
c lenpds: input, length of PDS in bytes
c igds:   input, integer array containing GRIB GDS section
c nword:  input, number of words used in igds
c kgds:   input, GDS exists if kgds=1
c ierr:   returned error code, 0=no error,
c
      include 'calruc.par'
      include 'calruc.cm1'

      dimension igds(nword)
      character*72 gdsnam(4), gds6name
      integer*4 kgds,iskp,pds(22),gribedi
      integer*4 gdslen,gdsnv,gdspv,gds6,shcrp,csmode
      integer*4 nx,ny,la1,lo1,gds17(8)
      integer*4 ksign,refa,refb
c
      integer*4 la2,lo2,di,dj,ngauss,latin,xssp,yssp,orient,nr
      integer*4 gds27,gds28(3),lov,dx,dy,latin2,latin1,latsp,lonsp
      integer*4 latpl,lonpl

      data ifirst/0/

c --------------------------
c
      if(ifirst.eq.0) then
        gdsnam(1)=' 1 = length'                                       !byte 1-3
        gdsnam(2)=' 2 = NV, number of vertical coordinate parameter'  !byte 4
        gdsnam(3)=' 3 = PV, or PL or 255'                             !byte 5
        gdsnam(4)=' 4 = data representation type (table 6)'           !byte 6
        ifirst=9
      endif
c
c process GDS
c
      if(kgds.eq.0) then
        gdslen=0
        return
      endif
      iskp=inoff
      call gbyte(igds,gdslen,iskp,24)
      iskp=iskp+24
      call gbyte(igds,gdsnv,iskp,8)
      iskp=iskp+8
      call gbyte(igds,gdspv,iskp,8)
      iskp=iskp+8
      call gbyte(igds,gds6,iskp,8)
      iskp=iskp+8
      call GDS_TAB6(pds(3),gribedi,gds6,gds6name)
      if(iprnt.ne.0) then
        write(*,*) ' GDS length: ',gdslen
        write(*,*) ' GDS NV: ',gdsnv
        write(*,*) ' GDS PV: ',gdspv
        write(*,*) ' GDS Data Repres Type:(table 6) ',gds6
        write(*,*) '   ',gds6name
      endif
      igds6=gds6

c
c bytes 7-17: either "spherical harmonic" or "the rest"
c                      gds6 50-80
c
      if(gds6.ge.50 .and. gds6.le.80) then
        call gbyte(igds,nj,iskp,16)
        iskp=iskp+16
        call gbyte(igds,nk,iskp,16)
        iskp=iskp+16
        call gbyte(igds,nm,iskp,16)
        iskp=iskp+16
        call gbyte(igds,shcrp,iskp,8)
        iskp=iskp+8
        call gbyte(igds,csmode,iskp,8)
        iskp=iskp+8
        if(iprnt.ne.0) then
          write(*,*) ' GDS NJ: ',nj
          write(*,*) ' GDS NK: ',nk
          write(*,*) ' GDS NM: ',nm
          write(*,*) ' GDS shcrp: (table 9)',shcrp
          write(*,*) ' GDS Coeff Storage Mode: (table 10) ',csmode
        endif
      else
        call gbyte(igds,nx,iskp,16)
        if(nx.ge.65535) nx=-99999
        iskp=iskp+16
        call gbyte(igds,ny,iskp,16)
        iskp=iskp+16
        call gbyte(igds,ksign,iskp,1)
        call gbyte(igds,la1,iskp+1,23)
        if(ksign.eq.1) la1=-la1
        iskp=iskp+24
        call gbyte(igds,ksign,iskp,1)
        call gbyte(igds,lo1,iskp+1,23)
        if(ksign.eq.1) lo1=-lo1
        iskp=iskp+24
        call gbytes(igds,gds17,iskp,1,0,8)
        if(iprnt.ne.0) then
          write(*,*) ' GDS Ni: ',nx
          write(*,*) ' GDS Nj: ',ny
          write(*,*) ' GDS La1: ',la1
          write(*,*) ' GDS Lo1: ',lo1
          write(*,*) ' GDS 17: (table 7) ',(gds17(i),i=1,8)
        endif

      endif

c
c pds(3)  gds6    bytes 18-  : 
c         0,4    (1) lat/long grids (including Gaussian lat/long)          7-32
c        201     (2) Arakawa semi-staggered grid on rotated lat/long grids 7-32
c        202     (3) Arakawa filled E-grid on rotated lat/long grids       7-32
c          5     (4) Polar Stereographic grids                             7-32
c ?        2     (5) Gnomonic Projection or Stereographic grids            7-32
c          3     (6) Lambert Conformal secant or tangent cone grids        7-42
c          1     (7) Mercator grids                                        7-42
c         90     (8) Space View Perspective or Orthographic grids          7-42
c         50     (9) Spherical Harmonic Coefficients                       7-32
c  98     10    (10) ECMWF Rotated lat/long grids                          7-42
c  98     14    (11) ECMWF Rotated Gaussian grids                          7-42
c  98     60    (12) ECMWF Rotated Spherical Harmonics                     7-42
c  98     20    (13) ECMWF Stretched lat/long grids                        7-42
c  98     24    (14) ECMWF Stretched Gaussian grids                        7-42
c  98     70    (15) ECMWF Stretched Spherical Harmonics                   7-42
c  98     30    (16) ECMWF Stretched and Rotated lat/long grids            7-52
c  98     34    (17) ECMWF Stretched and Rotated Gaussian grids            7-52
c  98     80    (18) ECMWF Stretched and Rotated Spherical Harmonics       7-52
c
c bytes 18-
c
      iskp=iskp+8
      if(gds6.eq.0 .or. gds6.eq.4 .or.
     &   gds6.eq.10 .or. gds6.eq.14) then  !lat/long grids (including Gaussian)
        call gbyte(igds,ksign,iskp,1)
        call gbyte(igds,la2,iskp+1,23)
        if(ksign.eq.1) la2=-la2
        iskp=iskp+24
        call gbyte(igds,ksign,iskp,1)
        call gbyte(igds,lo2,iskp+1,23)
        if(ksign.eq.1) lo2=-lo2
        iskp=iskp+24
        call gbyte(igds,ksign,iskp,1)
        call gbyte(igds,di,iskp+1,15)
        if(di.eq.32767) then
          di=-99999
        else
          if(ksign.eq.1) di=-di
        endif
        if(iprnt.ne.0) then
          write(*,*) ' GDS La2: ',la2
          write(*,*) ' GDS Lo2: ',lo2
          write(*,*) ' GDS Di: ',di
        endif
        iskp=iskp+16
        if(gds6.eq.4 .or. gds6.eq.14) then
          call gbyte(igds,ngauss,iskp,16)
          if(iprnt.ne.0) write(*,*) ' GDS N gaussian: ',ngauss
        else
          call gbyte(igds,ksign,iskp,1)
          call gbyte(igds,dj,iskp+1,15)
          if(dj.eq.32767) then
            dj=-99999
          else
            if(ksign.eq.1) dj=-dj
          endif
          if(iprnt.ne.0) write(*,*) ' GDS Dj: ',dj
        endif
        iskp=iskp+16
        call gbytes(igds,gds28,iskp,1,0,3)
        if(iprnt.ne.0) 
     &    write(*,*) ' GDS 28: (table 8) ',(gds28(i),i=1,3)
      elseif(gds6.eq.201) then    !Arakawa semi-staggered rotated lat/long
        call gbyte(igds,la2,iskp,24)
        iskp=iskp+24
        call gbyte(igds,lo2,iskp,24)
        if(iprnt.ne.0) then
          write(*,*) ' GDS La2: ',la2
          write(*,*) ' GDS Lo2: ',lo2
        endif
        iskp=iskp+24
        call gbyte(igds,ksign,iskp,1)
        call gbyte(igds,di,iskp+1,15)
        if(di.eq.32767) then
          di=-99999
        else
          if(ksign.eq.1) di=-di
        endif
        if(iprnt.ne.0) write(*,*) ' GDS Di: ',di
        iskp=iskp+16
        call gbyte(igds,ksign,iskp,1)
        call gbyte(igds,dj,iskp,15)
        if(dj.eq.32767) then
          dj=-99999
        else
          if(ksign.eq.1) dj=-dj
        endif
        if(iprnt.ne.0) write(*,*) ' GDS Dj: ',dj
        iskp=iskp+16
        call gbytes(igds,gds28,iskp,1,0,3)
        if(iprnt.ne.0) 
     &    write(*,*) ' GDS 28: (table 8) ',(gds28(i),i=1,3)
      elseif(gds6.eq.202) then    !Arakawa filled E-grid on rotated lat/long
        call gbyte(igds,la2,iskp,24)
        iskp=iskp+24
        call gbyte(igds,lo2,iskp,24)
        if(iprnt.ne.0) then
          write(*,*) ' GDS La2: ',la2
          write(*,*) ' GDS Lo2: ',lo2
        endif
        iskp=iskp+24
        call gbyte(igds,ksign,iskp,1)
        call gbyte(igds,di,iskp+1,15)
        if(di.eq.32767) then
          di=-99999
        else
          if(ksign.eq.1) di=-di
        endif
        if(iprnt.ne.0) write(*,*) ' GDS Di: ',di
        iskp=iskp+16
        call gbyte(igds,ksign,iskp,1)
        call gbyte(igds,dj,iskp+1,15)
        if(dj.eq.32767) then
          dj=-99999
        else
          if(ksign.eq.1) dj=-dj
        endif
        if(iprnt.ne.0) write(*,*) ' GDS Dj: ',dj
        iskp=iskp+16
        call gbytes(igds,gds28,iskp,1,0,3)
        if(iprnt.ne.0) 
     &    write(*,*) ' GDS 28: (table 8) ',(gds28(i),i=1,3)
      elseif(gds6.eq.5) then      !Polar Stereographic grids
c
        call gbyte(igds,ksign,iskp,1)      ! added 02/17/95
        call gbyte(igds,lov,iskp+1,23)     ! added 02/17/95
        if(ksign.eq.1) lov=-lov            ! added 02/17/95
c       call gbyte(igds,lov,iskp,24)
c
        iskp=iskp+24
        call gbyte(igds,dx,iskp,24)
        iskp=iskp+24
        call gbyte(igds,dy,iskp,24)
        iskp=iskp+24
        call gbyte(igds,gds27,iskp,1)
        iskp=iskp+8
        call gbytes(igds,gds28,iskp,1,0,3)
c       call gbyte(igds,gds28(1),iskp,1)
c       call gbyte(igds,gds28(2),iskp+1,1)
c       call gbyte(igds,gds28(3),iskp+2,1)
        if(iprnt.ne.0) then
          write(*,*) ' GDS Lov: ',lov
          write(*,*) ' GDS Dx: ',dx
          write(*,*) ' GDS Dy: ',dy
          write(*,*) ' GDS 27: (note 5) ',gds27
          write(*,*) ' GDS 28: (table 8) ',(gds28(i),i=1,3)
        endif

      elseif(gds6.eq.2) then    !Gnomonic Projection or Stereographic 
        write(*,*) '****Gnomonic Projection,  not ready yet.'
      elseif(gds6.eq.3) then      !Lambert Conformal secant or tangent cone
c
        call gbyte(igds,ksign,iskp,1)      ! added 02/17/95
        call gbyte(igds,lov,iskp+1,23)     ! added 02/17/95
        if(ksign.eq.1) lov=-lov            ! added 02/17/95
c       call gbyte(igds,lov,iskp,24)
c
        iskp=iskp+24
        call gbyte(igds,dx,iskp,24)
        iskp=iskp+24
        call gbyte(igds,dy,iskp,24)
        iskp=iskp+24
        call gbyte(igds,gds27,iskp,1)
        iskp=iskp+8
        call gbytes(igds,gds28,iskp,1,0,3)
        iskp=iskp+8
        call gbyte(igds,ksign,iskp,1)
        call gbyte(igds,latin1,iskp+1,23)
        if(ksign.eq.1) latin1=-latin1
        iskp=iskp+24
        call gbyte(igds,ksign,iskp,1)
        call gbyte(igds,latin2,iskp+1,23)
        if(ksign.eq.1) latin2=-latin2
        iskp=iskp+24
        call gbyte(igds,ksign,iskp,1)
        call gbyte(igds,latsp,iskp+1,23)
        if(ksign.eq.1) latsp=-latsp
        iskp=iskp+24
        call gbyte(igds,ksign,iskp,1)
        call gbyte(igds,lonsp,iskp+1,23)
        if(ksign.eq.1) lonsp=-lonsp
        iskp=iskp+24
        if(iprnt.ne.0) then
          write(*,*) ' GDS Lov: ',lov
          write(*,*) ' GDS Dx: ',dx
          write(*,*) ' GDS Dy: ',dy
          write(*,*) ' GDS 27: (note 5) ',gds27
          write(*,*) ' GDS 28: (table 8) ',(gds28(i),i=1,3)
          write(*,*) ' GDS latin 1: ',latin1
          write(*,*) ' GDS latin 2: ',latin2
          write(*,*) ' GDS lat SP:  ',latsp
          write(*,*) ' GDS lon SP:  ',lonsp
        endif
      elseif(gds6.eq.1) then      !Mercator grids
        call gbyte(igds,ksign,iskp,1)
        call gbyte(igds,la2,iskp+1,23)
        if(ksign.eq.1) la2=-la2
        iskp=iskp+24
        call gbyte(igds,ksign,iskp,1)
        call gbyte(igds,lo2,iskp+1,23)
        if(ksign.eq.1) lo2=-lo2
        iskp=iskp+24
        call gbyte(igds,ksign,iskp,1)
        call gbyte(igds,latin,iskp+1,23)
        if(ksign.eq.1) latin=-latin
        iskp=iskp+24
        call gbyte(igds,gds27,iskp,1)
        iskp=iskp+8
        call gbytes(igds,gds28,iskp,1,0,3)
        iskp=iskp+8
        call gbyte(igds,di,iskp,24)
        iskp=iskp+24
        call gbyte(igds,dj,iskp,24)
        if(iprnt.ne.0) then
          write(*,*) ' GDS La2:   ',la2
          write(*,*) ' GDS Lo2:   ',lo2
          write(*,*) ' GDS Latin: ',latin
          write(*,*) ' GDS 27: (note 5) ',gds27
          write(*,*) ' GDS 28: (table 8) ',(gds28(i),i=1,3)
          write(*,*) ' GDS Di: ',di
          write(*,*) ' GDS Dj: ',dj
        endif
      elseif(gds6.eq.90) then     !Space View Perspective or Orthographic grids
        call gbyte(igds,dx,iskp,24)
        iskp=iskp+24
        call gbyte(igds,dy,iskp,24)
        iskp=iskp+24
        call gbyte(igds,xssp,iskp,24)
        iskp=iskp+24
        call gbyte(igds,yssp,iskp,24)
        iskp=iskp+24
        call gbytes(igds,gds28,iskp,1,0,3)
        iskp=iskp+8
        call gbyte(igds,orient,iskp,24)
        iskp=iskp+24
        call gbyte(igds,nr,iskp,24)
        iskp=iskp+24
        if(iprnt.ne.0) then
          write(*,*) ' GDS Dx:     ',dx
          write(*,*) ' GDS Dy:     ',dy
          write(*,*) ' GDS Xssp:   ',xssp
          write(*,*) ' GDS Yssp:   ',yssp
          write(*,*) ' GDS 28: (table 8) ',(gds28(i),i=1,3)
          write(*,*) ' GDS Orient: ',orient
          write(*,*) ' GDS Nr:     ',nr
        endif
      elseif(gds6.eq.50) then     !Spherical Harmonic Coefficients

      else
      endif
c
      if(gds6.eq.10 .or. gds6.eq.14 .or. gds6.eq.60) then   !ECMWF Rotated only
        if(pds(3).eq.98) then
          call gbyte(igds,ksign,iskp,1)
          call gbyte(igds,latsp,iskp+1,23)
          if(ksign.eq.1) latsp=-latsp
          iskp=iskp+24
          call gbyte(igds,ksign,iskp,1)
          call gbyte(igds,lonsp,iskp+1,23)
          if(ksign.eq.1) lonsp=-lonsp
          iskp=iskp+24
          call gbyte(igds,isign,iskp,1)
          call gbyte(igds,refa,iskp+1,7)
          call gbyte(igds,refb,iskp+8,24)
          rdum=refb
          angrot=(-1)**isign * rdum * 16**(refa-64) / 2**24
          if(iprnt.ne.0) then
            write(*,*) ' GDS Lat SP:            ',latsp
            write(*,*) ' GDS Lon SP:            ',lonsp
            write(*,*) ' GDS Angle of Rotation: ',angrot
          endif
        endif
      elseif(gds6.eq.20 .or. gds6.eq.24 .or. gds6.eq.70) then
c                                                        !ECMWF Stretched only
        if(pds(3).eq.98) then
          call gbyte(igds,ksign,iskp,1)
          call gbyte(igds,latpl,iskp+1,23)
          if(ksign.eq.1) latpl=-latpl
          iskp=iskp+24
          call gbyte(igds,ksign,iskp,1)
          call gbyte(igds,lonpl,iskp+1,23)
          if(ksign.eq.1) lonpl=-lonpl
          iskp=iskp+24
          call gbyte(igds,isign,iskp,1)
          call gbyte(igds,refa,iskp+1,7)
          call gbyte(igds,refb,iskp+8,24)
          rdum=refb
          strfac=(-1)**isign * rdum * 16**(refa-64) / 2**24
          if(iprnt.ne.0) then
            write(*,*) ' GDS Lat Pole:          ',latpl
            write(*,*) ' GDS Lon Pole:          ',lonpl
            write(*,*) ' GDS Stretching Factor: ',strfac
          endif
        endif
      elseif(gds6.eq.30 .or. gds6.eq.34 .or. gds6.eq.80) then
c                                         !ECMWF Stretched & Rotated
        if(pds(3).eq.98) then
          call gbyte(igds,ksign,iskp,1)
          call gbyte(igds,latsp,iskp+1,23)
          if(ksign.eq.1) latsp=-latsp
          iskp=iskp+24
          call gbyte(igds,ksign,iskp,1)
          call gbyte(igds,lonsp,iskp+1,23)
          if(ksign.eq.1) lonsp=-lonsp
          iskp=iskp+24
          call gbyte(igds,isign,iskp,1)
          call gbyte(igds,refa,iskp+1,7)
          call gbyte(igds,refb,iskp+8,24)
          rdum=refb
          angrot=(-1)**isign * rdum * 16**(refa-64) / 2**24
          if(iprnt.ne.0) then
            write(*,*) ' GDS Lat SP:            ',latsp
            write(*,*) ' GDS Lon SP:            ',lonsp
            write(*,*) ' GDS Angle of Rotation: ',angrot
          endif
          call gbyte(igds,ksign,iskp,1)
          call gbyte(igds,latpl,iskp+1,23)
          if(ksign.eq.1) latpl=-latpl
          iskp=iskp+24
          call gbyte(igds,ksign,iskp,1)
          call gbyte(igds,lonpl,iskp+1,23)
          if(ksign.eq.1) lonpl=-lonpl
          iskp=iskp+24
          call gbyte(igds,isign,iskp,1)
          call gbyte(igds,refa,iskp+1,7)
          call gbyte(igds,refb,iskp+8,24)
          rdum=refb
          strfac=(-1)**isign * rdum * 16**(refa-64) / 2**24
          if(iprnt.ne.0) then
            write(*,*) ' GDS Lat Pole:          ',latpl
            write(*,*) ' GDS Lon Pole:          ',lonpl
            write(*,*) ' GDS Stretching Factor: ',strfac
          endif
        endif
      else
      endif

      if(gds28(3).eq.0) then
         iscan=0
      else
         iscan=1
      endif

      ierr=0
      return
      end
c
c ---------------------------------------------------------------

      subroutine GDS_TAB6(idcenter,gribedi,indx,name)
      parameter    (maxx=255)
      character*72 code(maxx),name
      integer*4    idcenter,gribedi
      data ifirst/0/
c
      if(ifirst.ne.0) go to 10
      do i=1,maxx
        code(i)=' '
      enddo
c
c               123456789012345678901234567890123456789012345678901234567890
      code(1)='latitude/longitude grid'
      code(2)='Mercator projection'
      code(3)='Gnomonic Porjection grid'
      code(4)='Lambert Conformal projection'
      code(5)='Gaussian latitude/longitude grid'
      code(6)='Polar stereographic'
      code(14)='Oblique Lambert conformal projection'
      code(51)='spherical harmonic coefficients'
      code(91)='space view perspective or orthographic grid'
      code(202)='Arakawa semi-stagg E-grid on rotated lat/long grids'
      code(203)='Arakawa filled E-grid on rotated lat/long grids'
c
   10 continue
      if(idcenter.eq.98 .and. (gribedi.eq.0 .or. gribedi.eq.-1)) then
        code(3)='Stereographic projection'
        code(11)='rotated latitude/longitude grids'
        code(14)=' '
        code(15)='rotated Gaussian latitude/longitude grids'
        code(21)='stretched latitude/longitude grids'
        code(25)='stretched Gaussian latitude/longitude grids'
        code(31)='stretched and rotated latitude/longitude grids'
        code(35)='stretched and rotated Gaussian lat/long grids'
        code(61)='rotated spherical harmonic coefficients'
        code(71)='streched spherical harmonics'
        code(81)='stretched and rotated spher harmonic coeff'
        code(91)=' '
        code(202)=' '
        code(203)=' '
      endif
      idum=indx+1
      name=code(idum)
      return
      end
c
c
C ----------------------------------------------------------------------
      subroutine pds_GRIB1(iprnt,gribedi,iskp,nword,
     &  kgds,kbms,dscalf,ierr,lvl2,pds,ipds)
c
c This is a generic GRIB Edition 1 PDS decoder 
c
c SUN FORTRAN version 
c
c lenpds: input, length of PDS in bytes
c iskp:   input, number of bits to skip from the beginning
c ipds:   input, integer array containing GRIB PDS
c nword:  input, number of words used in ipds
c pds:    returned, PDS array
c kgds:   returned, GDS exists if kgds=1
c kbms:   returned, BMS exists if kbms=1
c ierr:   returned error code, 0=no error, 
c
      dimension ipds(nword)
      character pdsnam(22)*72
      integer*4 pds(22),kgds,kbms,iskp,gribedi,ibloc(2)
      data ifirst/0/

c
      save
c
      if(ifirst.eq.0) then                                              ! pds bytes
        pdsnam(1)=' 1=length'                                           !  1-3
        pdsnam(2)=' 2=parameter table number'                           !  4
        pdsnam(3)=' 3=center ID'                                        !  5
        pdsnam(4)=' 4=process ID'                                       !  6
        pdsnam(5)=' 5=grid ID'                                          !  7
        pdsnam(6)=' 6=flag'                                             !  8
        pdsnam(7)=' 7=parameter ID (table 2)'                           !  9 
        pdsnam(8)=' 8=type of level or layer (table 3 & 3a)'            ! 10
        pdsnam(9)=' 9=pressure or height of level or layer'             ! 11-12
        pdsnam(10)='10=year of centry'                                  ! 13
        pdsnam(11)='11=month of year'                                   ! 14
        pdsnam(12)='12=day of month'                                    ! 15
        pdsnam(13)='13=hour of day'                                     ! 16
        pdsnam(14)='14=minute of hour'                                  ! 17
        pdsnam(15)='15=forecast time units (table 4)'                   ! 18
        pdsnam(16)='16=P1'                                              ! 19
        pdsnam(17)='17=P2'                                              ! 20
        pdsnam(18)='18=time range indicator (table 5)'                  ! 21
        pdsnam(19)='19=number included in averge when pds(18)<>0'       ! 22-23
        pdsnam(20)='20=number missing from averages or accumulations'   ! 24
        pdsnam(21)='21=century of initial (reference) time'             ! 25
c                                                                       ! 26
        pdsnam(22)='22=decimal scale factor D'                          ! 27-28
c                                                                       ! 29-40
c                                                                       ! 41-
        ifirst=9
      endif
c
      if(gribedi.eq.-1) then
        pds(1)=20
        pds(2)=128
      else
        call gbyte(ipds,pds(1),iskp,24)
        iskp=iskp+24
        call gbyte(ipds,pds(2),iskp,8)
        iskp=iskp+8
      endif
      call gbyte(ipds,pds(3),iskp,8)
      iskp=iskp+8
      call gbyte(ipds,pds(4),iskp,8)
      iskp=iskp+8
      call gbyte(ipds,pds(5),iskp,8)
      iskp=iskp+8
      if(gribedi.eq.-1) then
        call gbyte(ipds,kgds,7*8+7,1)                   ! ECMWF Edition X
        call gbyte(ipds,kbms,7*8+6,1)                   ! ECMWF Edition X
      else
        call gbyte(ipds,kgds,iskp,1)
        call gbyte(ipds,kbms,iskp+1,1)
        pds(6)=(kgds*10+kbms)*1000000                   !950711
      endif
      iskp=iskp+8
      call gbyte(ipds,pds(7),iskp,8)
      iskp=iskp+8
      call gbyte(ipds,pds(8),iskp,8)
      iskp=iskp+8
      if(pds(8).eq.100 .or. pds(8).eq.103 .or.
     &   pds(8).eq.105 .or. pds(8).eq.107 .or.
     &   pds(8).eq.109 .or. pds(8).eq.111 .or.
     &   pds(8).eq.113 .or. pds(8).eq.125 .or. 
     &   pds(8).eq.115 .or. pds(8).eq.117 .or.      !1998MAY14
     &   pds(8).eq.119 .or.                         !1998MAY14
     &   pds(8).eq.160 .or. pds(8).eq.200 .or. 
     &   pds(8).eq.201 ) then
        if(gribedi.eq.-1) then                   ! ECMWF Edition X
          call gbytes(ipds,ibloc,10*8,8,0,2)
          pds(9)=ibloc(1)*32+ibloc(2)
        else
          call gbyte(ipds,pds(9),iskp,16)
        endif
        lvl2=-9999                               ! lvl2 is not used
        iskp=iskp+16
      else
        call gbyte(ipds,pds(9),iskp,8)
        iskp=iskp+8
        call gbyte(ipds,lvl2,iskp,8)
        iskp=iskp+8
      endif
      call gbyte(ipds,pds(10),iskp,8)
      iskp=iskp+8
      call gbyte(ipds,pds(11),iskp,8)
      iskp=iskp+8
      call gbyte(ipds,pds(12),iskp,8)
      iskp=iskp+8
      call gbyte(ipds,pds(13),iskp,8)
      iskp=iskp+8
      call gbyte(ipds,pds(14),iskp,8)
      iskp=iskp+8
      call gbyte(ipds,pds(15),iskp,8)
      iskp=iskp+8
      idum=iskp                                 !970103
      call gbyte(ipds,pds(16),iskp,8)
      iskp=iskp+8
      call gbyte(ipds,pds(17),iskp,8)
      iskp=iskp+8
      call gbyte(ipds,pds(18),iskp,8)
      iskp=iskp+8
      if(pds(18).eq.10) then                    !970103
        call gbyte(ipds,pds(16),idum,16)        !970103
        pds(17)=-9999                           !970103
      endif                                     !970103
      call gbyte(ipds,pds(19),iskp,16)
      iskp=iskp+16
      call gbyte(ipds,pds(20),iskp,8)
      iskp=iskp+8
c
c fix if pdslen < 28
c
      dscalf=0.
      pds(22)=-9999
      if(pds(1).le.24) pds(21)=-9999
      if(pds(1).ge.28) then
        call gbyte(ipds,pds(21),iskp,8)
        iskp=iskp+8
c
c skip byte 26
c
        iskp=iskp+8
        call gbyte(ipds,isign,iskp,1)
        call gbyte(ipds,pds(22),iskp+1,15)
        if(isign.eq.1) pds(22)=-pds(22)       !961113
        dscalf=float(pds(22))                 !961113
      endif
c
c print out PDS
c
      if(iprnt.ne.0) then
         do i=1,22
            if(i.eq.6) then
               write(*,101) pds(i),kgds,kbms,pdsnam(i)
 101           format(i10,i2,i2,a70)
            elseif(i.eq.9) then
               write(*,115) pds(i),lvl2,pdsnam(i)
 115           format(i7,i6,1x,a70)
            else
               write(*,102) pds(i),pdsnam(i)
 102           format(i10,4x,a70)
            endif
         enddo
c         write(18,181)pds(7),pds(9),lvl2
c 181     format('Find Variable: ',3i7)

c        write(20,2010)pds(7),pds(9),lvl2
c 2010   format('Var-Pres-Hgt: ',3i8)

      endif

c fill ib1par
c
c     ib1par(1)=pds(3)
c     ib1par(2)=pds(4)
c     ib1par(3)=pds(5)
c     ib1par(4)=pds(6)
c     ib1par(5)=pds(7)
c     ib1par(6)=pds(8)
c     ib1par(7)=pds(9)
c     ib1par(8)=lvl2
c     ib1par(9)=pds(10)
c     ib1par(10)=pds(11)
c     ib1par(11)=pds(12)
c     ib1par(12)=pds(13)
c     ib1par(13)=pds(14)
c     ib1par(14)=pds(15)
c     ib1par(15)=pds(16)
c     ib1par(16)=pds(17)
c     ib1par(17)=pds(18)
c     ib1par(18)=pds(19)
c     ib1par(19)=pds(20)
c     ib1par(20)=pds(21)
c
      ierr=0
      return
      end
c
c
c *************************************************************

      subroutine fill_ingrib(mxone,mxlen,nword,lenrd,lentot,
     &                       ione,ingrib)
      dimension ingrib(mxlen),ione(mxone)
c
      mword=lenrd/nword
      if(mod(lenrd,nword).ne.0) mword=mword+1 
      do i=1,mword
        lentot=lentot+1
        ingrib(lentot)=ione(i)
      enddo
      return
      end
c
c

C ---------------------------------------------------------
      subroutine fl360(flon)

C     Purpose: Convert longitude to 0-360 convention
      
      if(flon.le.0) flon=360.+ flon
        
      return
      end

C *****************************************************************

      subroutine pass2d(datan,nmax,npoints,iz,ivar)

C Pass 1-D data to a 2-D array area (one slice of a 3d array)
      
      include 'calruc.par'
      include 'calruc.cm1'

      dimension datan(nmax)

      if(iscan.ne.0) then
         print *,'Error: Pass 2D only good for ISCAN=0'
         print *,'       ISCAN=',iscan
         stop
      endif

      nxlen=mxruc
      do ii=1,npoints
         j=(ii-1)/nxlen+1
         i=ii-(j-1)*nxlen
         x3d(i,j,iz,ivar)=datan(ii)
      enddo

      return
      end
C *****************************************************************

      subroutine pass2d_2(datan,nmax,npoints,ivar)

C Pass 1-D data to a 2-D array area
      
      include 'calruc.par'
      include 'calruc.cm1'

      dimension datan(nmax)

      if(iscan.ne.0) then
         print *,'Error: Pass 2D only good for ISCAN=0'
         print *,'       ISCAN=',iscan
         stop
      endif

      nxlen=mxruc
      do ii=1,npoints
         j=(ii-1)/nxlen+1
         i=ii-(j-1)*nxlen
         x2d(i,j,ivar)=datan(ii)
      enddo

      return
      end

C *****************************************************************

      subroutine pass2d_ter(datan,nmax,npoints)

C Pass 1-D data to a 2-D array area
      
      include 'calruc.par'
      include 'calruc.cm1'

      dimension datan(nmax)

      if(iscan.ne.0) then
         print *,'Error: Pass 2D only good for ISCAN=0'
         print *,'       ISCAN=',iscan
         stop
      endif

      nxlen=mxruc
!      print *,'check terrain ',npoints
      do ii=1,npoints
         j=(ii-1)/nxlen+1
         i=ii-(j-1)*nxlen
         ter(i,j)=datan(ii)
!         print *,i,j,ter(i,j),ii,nxlen,npoints,nmax
      enddo
!      print *,'End of check terrain'
 
      return
      end

C --------------------------------------------------------

      Subroutine rotate(ws,wd,flat,flon)

C     Rotate map wind to true north
      include 'calruc.par'
      include 'calruc.cm1'

      xlonc=flonv
      call fl360(xlonc)
      call fl360(flon)

      dlon=(flon-xlonc)*conf

c      print 9,xlonc,conf,flon,flat,dlon,wd,ws
c 9    format(7f10.2)

      if(dlon .lt.-180.) dlon=dlon+360.
      if(dlon .gt. 180.) dlon=dlon-360.

      if(ws.gt.1.e-9) then
         if (flat.lt.0.) then
            wd=wd-dlon          !southern hemisphere
         else   
            wd=wd+dlon          !northern hemisphere
         endif
      endif

      if (wd.lt.  0.) wd=wd+360.
      if (wd.gt.360.) wd=wd-360.

      return
      end

C ********************************************************************

      Subroutine uv2ws(uu,vv,ws,wd)

C     Purpose: Convert u,v (m/s) to wind speed (m/s) and wind direction(deg)
      ws=sqrt(uu**2+vv**2)
      if(ws.gt.1.e-9) then
         xwd=270.-57.295778*atan2(vv,uu)
         wd=amod(xwd,360.)
      else
         wd=0.
      endif

      return
      end

C --------------------------------------------------
      Subroutine getrange

C Purpose: Select I/J ranges based required lat/lon,
c          determine which gridpoints are within 
c          selected domain
c Zhong Wu
C 10/16/2001

      include 'calruc.par'
      include 'calruc.cm1'
      
      if(iselect.eq.1) then
        i1=9999
        i2=-9999
        j1=9999
        j2=-9999
      else
        rlatmin = 9999.
        rlatmax = -9999.
        rlonmin = 9999.
        rlonmax = -9999.
        j1=ilatmin
        j2=ilatmax
        i1=ilonmin
        i2=ilonmax
      endif

      do j=1,myruc
         do i=1,mxruc

            flat=xy212(4,i,j)
            flon=xy212(3,i,j)
	    
	    if(iselect.eq.1) then

            if ((flat.ge.rlatmin).and.(flat.le.rlatmax) .and. 
     &            (flon.ge.rlonmin) .and. (flon.le.rlonmax)) then

               if(i.lt.i1) i1=i
               if(i.gt.i2) i2=i
               if(j.lt.j1) j1=j
               if(j.gt.j2) j2=j
            endif
	    
	    else
	    
            if ((j.ge.ilatmin).and.(j.le.ilatmax) .and. 
     &            (i.ge.ilonmin) .and. (i.le.ilonmax)) then

               if(flat.lt.rlatmin) rlatmin=flat
               if(flat.gt.rlatmax) rlatmax=flat
               if(flon.lt.rlonmin) rlonmin=flon
               if(flon.gt.rlonmax) rlonmax=flon
            endif
	    	    
	    endif
            
         enddo
      enddo   

      if(i1.eq.9999 .or. i2.eq.-9999 .or.
     &   j1.eq.9999 .or. j2.eq.-9999) then
         print *,'Error in getrange:',i1,i2,j1,j2
         stop
      endif

      nx1=i1
      nx2=i2
      ny1=j1
      ny2=j2

c      nz1=1
c      nz2=mxnz

      nxsub=(nx2-nx1)+1
      nysub=(ny2-ny1)+1
      nzsub=(nz2-nz1)+1

c --- SW point LCC locations (km) for Selected SubDomain
      rlatswsub=xy212(4,nx1,ny1)
      rlonswsub=xy212(3,nx1,ny1)

      xorgsub=xy212(5,nx1,ny1)
      yorgsub=xy212(6,nx1,ny1)

      write(ilg,214)nx1,nx2,ny1,ny2,nxsub,nysub
      write(ilg,215)rlatswsub,rlonswsub,xorgsub,yorgsub
 214  format('Selected domain I: ',2i5,/,
     &       '                J: ',2i5,/
     &       '  Number of Grids: ',2i5)
 215  format('Selected domain SW lat/lon: ',2f12.3,/,
     &       'Selected domain SW     X/Y: ',2f12.3)

c --- SW point LCC locations (km) for AWIPS whole domain
      rlatsw=xy212(4,1,1)
      rlonsw=xy212(3,1,1)
      xorg = -3332.034  !xy212(5,1,1)
      yorg = -2214.661  !xy212(6,1,1)

      write(ilg,314)mxruc,myruc
      write(ilg,315)rlatsw,rlonsw,xorg,yorg
 314  format('SWIPS domain Grids (NX/NY): ',2i5)
 315  format('SWIPS domain SW lat/lon: ',2f12.3,/,
     &       'SWIPS domain SW     X/Y: ',2f12.3)

C     Domain Center: Assume center at I=185, J=129
      ic=nint(mxruc/2.0+0.5)
      jc=nint(myruc/2.0+0.5)

      rlatc=39.46  !xy212(4,ic,jc)
      rlonc=-95    !xy212(3,ic,jc)
      truelat1=25.0
      truelat2=25.0
      flonref=flonv
      if(flonref.gt.180) flonref=360.0-flonref  ! WH if > 180 degree

      dxym=(xy212(5,mxruc,1)-xy212(5,1,1))/(mxruc-1)

      write(ilg,316)ic,jc,rlatc,rlonc
      write(ilg,317)truelat1,truelat2,flonref
      write(ilg,318)dxym

 316  format('AWIPS Fake Center I/J & Latc/Lonc: ',2i5,2f12.4)
 317  format('AWIPS True Lat1/Lat2 & Lon_ref: ',3f12.4)
 318  format('AWIPS Grid size: ',f12.4)
 
      return
      end


c***********************************************************************

       subroutine getflname(string,nt)

c ---   Get file name from an input line 
c       The file name is at the begining of and before the first 
c       blank space in the line

c -------------------------------------------------------------------
       character*80 string

       nt=index(string,' ')-1
       string=string(1:nt)

       return
       end

c -------------------------------------------------------------------
      subroutine noblank(string,nt)

c ---   Get file name from an input line 
c       The file name is at the begining of and before the first 
c       blank space in the line

c -------------------------------------------------------------------
      character*80 string

      do i=80,1,-1
         if(string(i:i).ne.' ') then
            nt=i
            return
         endif
      enddo
        
      nt=10
      
      return
      end

C ----------------------------------------------------------
      subroutine wrthd

C     Output 3D.DAT and 2D.DAT headers
      include 'calruc.par'
      include 'calruc.cm1'

      call noblank(title,nt)

      if (iform.eq.1) then    ! write header in the new format

c ---   3D.DAT file in the new format
        write(im3d,1012)dataset,dataver,datamod
1012    format(2a16,a64)

        write(comment,1013)cver,clevel
1013    format('Produced by CALRUC Version: ',a8, ', Level: ',a8)

        ncomm = 1
        write(im3d,'(i4)')ncomm
        do i=1,ncomm
          write(im3d,1014)comment
        enddo
1014    format(a132)

        if(iosrf.eq.1) then
c ---     2D.DAT file in the new format
          write(im2d,1012)dataset2d,dataver2d,datamod     
          write(im2d,'(i4)')ncomm
          do i=1,ncomm
            write(im2d,1014)comment
          enddo
        endif

       else   ! write header in the old format

         write(im3d,101)title(1:nt)
         write(im3d,1011)cset3d,cver,clevel
         if(iosrf.eq.1) then
c ---      2D.DAT file in the old format
           write(im2d,101)title(1:nt)     
           write(im2d,1011)cset3d,cver,clevel
         endif

       endif

c 101  format(a80)
 101   format(a)
 1011  format(3a12)
      
C     Record #3
      write (im3d,102)ioutw,ioutq,ioutc,iouti,ioutg,iosrf
 102  format (6(i3))

      if(iosrf.eq.1) 
     &  write (im2d,102)ioutw,ioutq,ioutc,iouti,ioutg,iosrf

      write(ilg,103) ioutw, ioutq, ioutc, iouti, ioutg, iosrf
 103  format(3x,'ioutw:',i2/,
     :     3x,'ioutq:',i2/,
     :     3x,'ioutc:',i2/,
     :     3x,'iouti:',i2/,
     :     3x,'ioutg:',i2/,
     :     3x,'iosrf:',i2)

C     Record #4 (Need Vertical level- NZ)
      if(itype.eq.0) then
      write(im3d,105)rlatc,rlonc,truelat1,truelat2,
     &      xorg,yorg,dxym,mxruc,myruc,mxnz
      else
      write(im3d,105)rlatc,rlonc,truelat1,truelat2,
     &      xorg,yorg,dxym,mxruc,myruc,mxnzp      
      endif 
 105  format('LCC ',f9.4,f10.4,2f7.2,2f10.3,f8.3,2i4,i3)
 
      if (iosrf.eq.1)
     1     write(im2d,105)rlatc,rlonc,truelat1,truelat2,
     2     xorg,yorg,dxym,mxruc,myruc,mxnz 

C     Record #5
      write(im3d,106)inhyd,imphys,icupa,ibltyp,ifrad,isoil,
     :     ifddaan,ifddaob,igrdt,ipbl,ishf,ilhf,iustr,iswdn,
     :     ilwdn,ist1,ist2,ist3,ist4,ist5,ist6
c     :     ilwdn,ist1,ist2,ist3,ist4,ist5,ist6,iswou,ilwou
      if (iosrf.eq.1) then
         write(im2d,105)inhyd,imphys,icupa,ibltyp,ifrad,isoil,
     &        ifddaan,ifddaob,igrdt,ipbl,ishf,ilhf,iustr,iswdn,
     &        ilwdn,ist1,ist2,ist3,ist4,ist5,ist6
c     &        ilwdn,ist1,ist2,ist3,ist4,ist5,ist6,iswou,ilwou
      endif
 106  format (30(i3))
      
C     Record #6
c      nhours=nfile/ngap
      write(im3d,96)idate,nhours,nxsub,nysub,nzsub
      if(iosrf.eq.1)
     &  write(im2d,96)idate,nhours,nxsub,nysub,nzsub

c header record #7:
      write(im3d,97)nx1,ny1,nx2,ny2,nz1,nz2
     &      ,rlonmin,rlonmax,rlatmin,rlatmax
      if(iosrf.eq.1)
     & write(im2d,97)nx1,ny1,nx2,ny2,nz1,nz2
     &      ,rlonmin,rlonmax,rlatmin,rlatmax

c next nz records:sigma levels (normalized by 1013.0 hPa) 
c      dps=pres(1)-pres(mxnz)
      dps=1013.0
      numpts = (ny2-ny1+1)*(nx2-nx1+1)

      do k=nz1,nz2
         if(itype.eq.1) then
         sgm=pres(k)/dps
	  else
         avgpres = 0
         do j=ny1,ny2
            do i=nx1,nx2
	      avgpres = avgpres + x3d(i,j,k,1)/100./numpts
            enddo
          enddo
         sgm=avgpres/dps
	  endif
         write(im3d,98)sgm
         if(iosrf.eq.1) write(im2d,98)sgm
      enddo

c geophysical records. 
c note: lat/lon and elev at dot point, land use at cross point
      idum=-9

      do j=ny1,ny2
         do i=nx1,nx2
            flat=xy212(4,i,j)
            flon=xy212(3,i,j)
            write(im3d,99)i,j,flat,flon,nint(ter(i,j))
     &            ,iveg(i,j),fmiss,fmiss,imiss
            if(iosrf.eq.1) 
     &          write(im2d,99)i,j,flat,flon,nint(ter(i,j))
     &            ,iveg(i,j),fmiss,fmiss,imiss
         enddo
      enddo
 96   format(i10,i5,4i4)
 97   format(6i4,2f10.4,2f9.4)
 98   format(f6.3)
 99   format(2i4,f9.4,f10.4,i5,i3,1x,f9.4,f10.4,i5)

      return
      end

C ------------------------------------------------
      
C -------------------------------------------------
      Subroutine getz(ip9,iz,ifound)
c-------------------------------------------------------------
c
c --- CALRUC  Version: 2.21     Level: 031126            getw
c ---          Zhong Wu, Earth Tech
c
c --- purpose:  Get vertical level for a 3-D varaible
c
c --- inputs:
c           ip9 - integer   - pressure
c --- output:
c           iz - integer   - vertical level corresponding to pressure ip9            
c
c --- getw called by:  main
c --- getw calls:      getz
c----------------------------------------------------------------------

C     Note: GRIB variable may not be in order vertically
      include 'calruc.par'
      include 'calruc.cm1'

      ifound=0
      do k=1,mxnzp
         if(ip9.eq.nint(pres(k))) then
            iz=k
            ifound=1
            return
         endif
      enddo

c      print *,'Warning: This Vertical level does not exist '
c      print *,'         for all hours  - ',ip9

      return

      end

c-------------------------------------------------------------
      subroutine timestamp(iyr,imon,iday,ihour,idate) 
c-------------------------------------------------------------
c
c --- CALRUC  Version: 2.21     Level: 031126            timestamp
c ---          Zhong Wu, Earth Tech
c
c --- purpose:  Get timestamp
c
c --- inputs:
c           iyr   - integer    - year
c           imon  - integer    - month
c           iday  - integer    - day
c           ihour - integer    - hour
c
c --- output:
c           idate - integer      - timestamp              
c
c --- timestamp called by:  main
c --- timestamp calls:      none
c----------------------------------------------------------------------

      iyr2k=iyr
      if(iyr.lt.100) then
         if(iyr.lt.50) then
            iyr2k=iyr+2000
         else
            iyr2k=iyr+1900
         endif
      endif
      idate=ihour+iday*100+imon*10000+iyr2k*1000000

      return
      end


C -------------------------------------------------
      Subroutine getw(wp,pp,atk,rh,ww,aq)
c-------------------------------------------------------------
c
c --- CALRUC  Version: 2.21     Level: 031126            getw
c ---          Zhong Wu, Earth Tech
c
c --- purpose:  Convert omega velocity to w velocity
c
c --- inputs:
c           wp  - real     - omega
c           pp  - real     - pressure (mb)
c           atk - real     - temperature (K)
c           rh  - real     - relative humidity (%)
c           aq  - real     - humidity
c
c --- output:
c           ww - real      - vertical velocity (m/s)
c           aq - real      - humidity              
c
c --- getw called by:  main
c --- getw calls:      gettv
c----------------------------------------------------------------------

      parameter(g=9.80616,Rd=287.0,alpha=Rd/g)

      call gettv(pp,atk,rh,tv,aq)
      
      ro=pp*100./Rd/tv
      ww=-wp/ro/g

c     print *,'ro:',ro,pp,atk,ww

      return
      end

c----------------------------------------------------------------------
      function esat(tdegc)
c----------------------------------------------------------------------
c
c --- CALRUC   Version: 2.21     Level: 901130                    ESAT
c ---          J. Scire, SRC
c
c --- PURPOSE:  Compute the saturation water vapor pressure (mb) using
c               the method of Lowe (1977) (JAM, 16, pp 100-103).
c
c --- INPUT:
c                TDEGC - real          - Air temperature (deg. C)
c
c --- OUTPUT:
c                 ESAT - real          - Saturation water vapor
c                                        pressure (mb)
c
c --- ESAT called by: WATER
c --- ESAT calls:     none
c----------------------------------------------------------------------
      data a0/6.107799961/,a1/4.436518521e-1/,a2/1.428945805e-2/
      data a3/2.650648471e-4/,a4/3.031240396e-6/,a5/2.034080948e-8/
      data a6/6.136820929e-11/
c
c --- compute saturation water vapor pressure (mb)
c --- NOTE: temperature is in deg. C

c     This equation gives negative esat at low temperature
      esat=a0+tdegc*(a1+tdegc*(a2+tdegc*(a3+tdegc*(a4+tdegc*
     1 (a5+tdegc*a6)))))

c     Magnus equation
c      aa=(17.67*tdegc)/(243.5+tdegc)
c      esat=6.112*exp(aa)

      if(esat.lt.0) then
cmjp         print *,'esat < 0 : ',esat
cmjp         print *,'set esat=0'
         esat=0
      endif
c
      return
      end


c ------------------------------------------------------------
      subroutine gettv(p,tk,rh,tv,aq)
c-------------------------------------------------------------
c
c --- CALRUC  Version: 2.21     Level: 010619            gettv
c ---          Zhong Wu, Earth Tech
c
c --- purpose:  Get virtual temperature
c
c --- inputs:
c           p - real      - pressure (mb)
c           tk - real     - temperature (K)
c           rh - real     - relative humidity (%)
c
c --- output:
c           tv - real      - virtual temperature(K)
c           aq - real      - humidity              
c
c --- gettv called by:  main
c --- gettv calls:      none
c----------------------------------------------------------------------

      if(rh.gt.100) then
cmjp         print *,'RH > 100 in gettv. Set it to 100'
         rh=100.
      endif

      tc=tk-273.15

      es0=esat(tc)
      ee=es0*rh/100.

      qq=0.622*ee/(p-0.378*ee)
      if(qq.lt.0) qq=0

      tv=tk*(1+0.608*qq)
      aq=qq*1000.

c      print *,'Sub gettv (tk/tv/dt):', tk,tv,tv-tk

      return
      end

c----------------------------------------------------------------------
      subroutine getdate(ndate,iyr,imon,iday,ihour)
c----------------------------------------------------------------------
c
c --- CALRUC  Version: 2.21     Level: 031126                  getdate
c ---          Zhong Wu, Earth Tech
c
c --- purpose:  compute gregorian date
c
c --- inputs:
c           iyr - integer      - year
c           imo - integer      - month
c          iday - integer      - day
c          ihour - integer     - hour
c
c --- output:
c          ndate - integer      - date
c
c --- getdate called by:  main
c --- getdate calls:      none
c----------------------------------------------------------------------

      include 'calruc.par'

      iyr=ndate/isix
      imon=ndate/ifour-iyr*100
      iday=ndate/100-iyr*ifour-imon*100
      ihour=ndate-iyr*isix-imon*ifour-iday*100
      
       return
       end

c -------------------------------------------------------------------
      subroutine wrtcmp(ipre,izh,atk,iwd,aws,awz,irh,aq,xcmp,ncmp)

c Zhong-xiang Wu

c     Compressed output of MM5.dat data record for ioutform >= 93

      parameter (xzero=0.0049)

      include 'calruc.par'
      include 'calruc.cm1'

      if(ncmp.lt.1) then
         print *,'Compression Error in wrtcmp:  NCMP < 1'
         print *,'NCMP = ',ncmp
         stop
      endif

      idcmp=0
      do i=1,ncmp
         if(xcmp(i).ge.xzero) then
            idcmp=1
            goto 1000
         endif
      enddo

 1000 continue

      if(idcmp.eq.0) then
         write(im3d,95)ipre,izh,atk,iwd,aws,awz,irh,aq,float(-ncmp)
      else
         write(im3d,95)ipre,izh,atk,iwd,aws,awz,irh,aq,
     &       (xcmp(i),i=1,ncmp)
      endif
 95   format( i4,i6,f6.1,i4,f5.1,f6.2,i3,f5.2,5f6.3)

      return
      end

C -----------------------------------------------------------------
      Subroutine chkprof(xtmp,nz,iorder,ivar,ilg,iok
     &          ,idex,jdex,h0)

c --- CALRUC  Version: 2.21     Level: 100525                  chkprof
c ---          Zhong Wu, Earth Tech

C Purpose:
C     Quality-Check of vertical profile
C Zhong Wu
C 3/16/2006
     
      dimension xtmp(nz)
      character*20 cnames(7)

      data cnames/'Press','GH','VPT','Q',' U',' V','WP'/

      ifl=0
      if(iorder.eq.0) goto 1000

C --- Check vertical order
      aa=xtmp(1)*iorder
      do k=2,nz
         bb=xtmp(k)*iorder
         if(bb.lt.aa) then
            write(ilg,101)cnames(ivar),idex,jdex,k,xtmp(k-1),xtmp(k)
            print 101,cnames(ivar),idex,jdex,k,xtmp(k-1),xtmp(k)
 101        format(' Error-Prof Order: ',a8,3i5,2f12.4)
            ifl=ifl+1
         endif

         aa=bb
      enddo

 1000 continue

      if(ivar.eq.2) then   ! GH
         h1=xtmp(1)
         dh=h1-h0
         if(dh.gt.1.0) then
            write(ilg,112)cnames(ivar),idex,jdex,h1,h0,dh
            print 112,cnames(ivar),idex,jdex,h1,h0,dh
 112        format(' Error-Terrain Changed: ',a8,2i5,3f12.4)
            ifl=ifl+1            
         endif
      endif

      if(ifl.gt.0) then
         write(ilg,103)cnames(ivar),ifl
         print 103,cnames(ivar),ifl
 103     format(' Error - QA fail:  ',a8,i4)
         iok=0
      endif

      return
      end
C ---------------------------------------------------------------
      Subroutine getfilename(infile,inbase,ires,itype,iapply
     &                 ,isite,IYY,IMM,IDD,IHH,IHHINI,ihour)

c --- CALRUC  Version: 2.21     Level: 060626                getfilename

C Purpose:
C     Determine GRIB file name
C
C Zhong-Xiang Wu
C 6/26/2006

C Input: 
c     ires:     RUC grid resolution of input GRIB data (0 - 20 km, 1 - 40 km)
c     itype:    RUC vertical level Type (0 - hybrid, 1 - pressure)
c     iapply:   RUC application type (0 - analysis, 1 - forecast)
c     isite:     Web site for FTP downloading (1 - ncep; 2 - NWS, 3 - ARM)

      character*80 infile,inbase

      if(isite.eq.1) then
         call ncepname(infile,inbase,ires,itype,iapply
     &        ,IYY,IMM,IDD,IHH,IHHINI,ihour) 
      elseif(isite.eq.2) then
         call nwsname(infile,inbase,ires,itype,iapply
     &        ,IYY,IMM,IDD,IHH,IHHINI,ihour) 
      elseif(isite.eq.3) then
         call armname(infile,inbase,ires,itype,iapply
     &        ,IYY,IMM,IDD,IHH,IHHINI,ihour) 
      else
         print *,'Error: Illegal web site id: ',isite
         stop
      endif

      return
      end

C ---------------------------------------------------------------
      Subroutine ncepname(infile,inbase,ires,itype,iapply
     &                 ,IYY,IMM,IDD,IHH,IHHINI,ihour)

c --- CALRUC  Version: 2.21     Level: 060626                ncepname

C Purpose:
C     Determine RUC GRIB file name for the NCEP web site
C
C Zhong-Xiang Wu
C 6/26/2006

C Input: 
c     ires:     RUC grid resolution of input GRIB data (0 - 20 km, 1 - 40 km)
c     itype:    RUC vertical level Type (0 - hybrid, 1 - pressure)
c     iapply:   RUC application type (0 - analysis, 1 - forecast)
C Note:

      character*80 infile, inbase

      nti=len_trim(inbase)

C     20-km Hybrid
      if(ires.eq.0 .and. itype.eq.0 .and.iapply.eq.0) then
          write(infile,101)inbase(1:nti),IHH
 101      format(a,'ruc2.t',i2.2,'z.bgrb20anl')
      elseif(ires.eq.0 .and. itype.eq.0 .and.iapply.eq.1) then
          if(ihour.eq.1) then   ! Use analysis file for first grib file
             write(infile,101)inbase(1:nti),IHH
          else
             write(infile,102)inbase(1:nti),IHHINI,IHH
 102         format(a,'ruc2.t',i2.2,'z.bgrb20f',i2.2)
          endif
C     20-km Pressure 
      elseif(ires.eq.0 .and. itype.eq.1 .and.iapply.eq.0) then
          write(infile,201)inbase(1:nti),IHH
 201      format(a,'ruc2.t',i2.2,'z.pgrb20anl')
      elseif(ires.eq.0 .and. itype.eq.1 .and.iapply.eq.1) then
          if(ihour.eq.1) then   ! Use analysis file for first grib file
             write(infile,201)inbase(1:nti),IHH
          else
             write(infile,202)inbase(1:nti),IHHINI,IHH
 202         format(a,'ruc2.t',i2.2,'z.pgrb20f',i2.2)
          endif
C     40-km Hybrid
      elseif(ires.eq.1 .and. itype.eq.0 .and.iapply.eq.0) then
          write(infile,301)inbase(1:nti),IHH
 301      format(a,'ruc2.t',i2.2,'z.bgrbanl')
      elseif(ires.eq.1 .and. itype.eq.0 .and.iapply.eq.1) then
          if(ihour.eq.1) then   ! Use analysis file for first grib file
             write(infile,301)inbase(1:nti),IHH
          else
             write(infile,302)inbase(1:nti),IHHINI,IHH
 302         format(a,'ruc2.t',i2.2,'z.bgrbf',i2.2)
          endif
C     40-km Pressure 
      elseif(ires.eq.1 .and. itype.eq.1 .and.iapply.eq.0) then
          write(infile,401)inbase(1:nti),IHH
 401      format(a,'ruc2.t',i2.2,'z.pgrbanl')
      elseif(ires.eq.1 .and. itype.eq.1 .and.iapply.eq.1) then
          if(ihour.eq.1) then   ! Use analysis file for first grib file
             write(infile,401)inbase(1:nti),IHH
          else
             write(infile,402)inbase(1:nti),IHHINI,IHH
 402         format(a,'ruc2.t',i2.2,'z.pgrbf',i2.2)
          endif
      else
          print *,'Error in ncepname: Illegal ids:',ires,itype,iapply
          stop
      endif

      print *,'NCEP File:', trim(infile)

      return
      end

C ---------------------------------------------------------------
      Subroutine nwsname(infile,inbase,ires,itype,iapply
     &                 ,IYY,IMM,IDD,IHH,IHHINI,ihour)

c --- CALRUC  Version: 2.21     Level: 060626                nwsname

C Purpose:
C     Determine RUC GRIB file name for the NWS web site
C
C Zhong-Xiang Wu
C 6/26/2006

C Input: 
c     ires:     RUC grid resolution of input GRIB data (0 - 20 km, 1 - 40 km)
c     itype:    RUC vertical level Type (0 - hybrid, 1 - pressure)
c     iapply:   RUC application type (0 - analysis, 1 - forecast)

      character*80 infile, inbase

      nti=len_trim(inbase)

C     20-km Hybrid
      if(ires.eq.0 .and. itype.eq.0 .and.iapply.eq.0) then
          write(infile,101)inbase(1:nti)
 101      format(a,'fh.anal_tl.press_gr.bgrib20')
      elseif(ires.eq.0 .and. itype.eq.0 .and.iapply.eq.1) then
          if(ihour.eq.1) then   ! Use analysis file for first grib file
             write(infile,101)inbase(1:nti)
          else
             write(infile,102)inbase(1:nti),IHH
 102         format(a,'fh.',i4.4,'_tl.press_gr.bgrib20')
          endif
C     20-km Pressure 
      elseif(ires.eq.0 .and. itype.eq.1 .and.iapply.eq.0) then
          write(infile,201)inbase(1:nti)
 201      format(a,'fh.anal_tl.press_gr.us20')
      elseif(ires.eq.0 .and. itype.eq.1 .and.iapply.eq.1) then
          if(ihour.eq.1) then   ! Use analysis file for first grib file
             write(infile,201)inbase(1:nti)
          else
             write(infile,202)inbase(1:nti),IHH
 202         format(a,'fh.',i4.4,'_tl.press_gr.us20')
          endif
C     40-km Hybrid
      elseif(ires.eq.1 .and. itype.eq.0 .and.iapply.eq.0) then
          write(infile,301)inbase(1:nti)
 301      format(a,'fh.anal_tl.press_gr.bgrib')
      elseif(ires.eq.1 .and. itype.eq.0 .and.iapply.eq.1) then
          if(ihour.eq.1) then   ! Use analysis file for first grib file
             write(infile,301)inbase(1:nti)
          else
             write(infile,302)inbase(1:nti),IHH
 302         format(a,'fh.',i4.4,'_tl.press_gr.bgrib')
          endif
C     40-km Pressure 
      elseif(ires.eq.1 .and. itype.eq.1 .and.iapply.eq.0) then
          write(infile,401)inbase(1:nti)
 401      format(a,'fh.anal_tl.press_gr.us40') 
      elseif(ires.eq.1 .and. itype.eq.1 .and.iapply.eq.1) then
          if(ihour.eq.1) then   ! Use analysis file for first grib file
             write(infile,401)inbase(1:nti)
          else
             write(infile,402)inbase(1:nti),IHH
 402         format(a,'fh.',i4.4,'_tl.press_gr.us40')
          endif
      else
          print *,'Error in ncepname: Illegal ids:',ires,itype,iapply
          stop
      endif

      print *,'NWS File:', trim(infile)

      return
      end

C ---------------------------------------------------------------
      Subroutine armname(infile,inbase,ires,itype,iapply
     &                 ,IYY,IMM,IDD,IHH,IHHINI,ihour)

c --- CALRUC  Version: 2.21     Level: 060626                armname

C Purpose:
C     Determine RUC GRIB file name for the ARM web site
C
C Zhong-Xiang Wu
C 6/26/2006

C Input: 
c     ires:     RUC grid resolution of input GRIB data (0 - 20 km, 1 - 40 km)
c     itype:    RUC vertical level Type (0 - hybrid, 1 - pressure)
c     iapply:   RUC application type (0 - analysis, 1 - forecast)

      character*80 infile, inbase

      if(iapply.eq.1) then
         print *,'Error in armname - Illegal iapply: ',iapply
         stop
      endif

      nti=len_trim(inbase)
      
C     20-km Hybrid
      if(ires.eq.0 .and. itype.eq.0 .and.iapply.eq.0) then
          write(infile,101)inbase(1:nti),IYY,IMM,IDD,IHH
 101      format(a,'sgpallruc20hybrX1.00.',
     &           i4.4,2i2.2,'.',i2.2,'0000.raw.grb')
C     20-km Pressure 
      elseif(ires.eq.0 .and. itype.eq.1 .and.iapply.eq.0) then
          write(infile,201)inbase(1:nti),IYY,IMM,IDD,IHH  
 201      format(a,'sgpallruc20isobX1.00.',                  ! ARM Files
     &           i4.4,2i2.2,'.',i2.2,'0000.raw.grb')
C     40-km Hybrid
      elseif(ires.eq.1 .and. itype.eq.0 .and.iapply.eq.0) then
          write(infile,301)inbase(1:nti),IYY,IMM,IDD,IHH
 301      format(a,'sgpallruc40hybrX1.00.',
     &           i4.4,2i2.2,'.',i2.2,'0000.raw.grb')
C     40-km Pressure 
      elseif(ires.eq.1 .and. itype.eq.1 .and.iapply.eq.0) then
          write(infile,401)inbase(1:nti),IYY,IMM,IDD,IHH
 401      format(a,'sgpallruc40isobX1.00.',
     &           i4.4,2i2.2,'.',i2.2,'0000.raw.grb')
      else
          print *,'Error in ncepname: Illegal ids:',ires,itype,iapply
          stop
      endif

      print *,'ARM File:', trim(infile)

      return
      end
c --------------------------------------------------------------
	subroutine gethours(ndate1,ndate2,nhrs)

C --- CALRUC   Version: 2.21Level: 061130               

c Purpose:
C     Get hours between to dates
C
C --- Changes from Version 1.1 Level 060910 to Version 1.2 Level 061130
C     1. Change gethours subroutine to account for leap years

	if(ndate2.lt.ndate1) then
	   print *,'Error in gethours: ndate2 < ndate1'
	   print *,ndate1,ndate2
	   stop
	endif

        io=6
 	call getdate(ndate1,iyr1,imon1,iday1,ihour1)
	call julday(io,iyr1,imon1,iday1,jday1)

	call getdate(ndate2,iyr2,imon2,iday2,ihour2)
	call julday(io,iyr2,imon2,iday2,jday2)

        call deltt(iyr1,jday1,ihour1,iyr2,jday2,ihour2,nhrs)

        nhrs=nhrs+1   ! deltt is one hour less total hours

	return
	end

C -------------------------------------------------------------------
      Subroutine begend(idate,ndate1,ndate2,infile,ilg)

c --- CALRUC    Version: 2.21      Level: 091014               

c --- Zhong Wu
C
C --- Get beginning and end time for tiles
C --- Data coverage in RUC tiles:
C     2003: 2003010100 - 2004010111
C     2004: 2004010100 - 2005010123
C     2005: 2005010100 - 2006010123
C     2006: 2006010100 - 2007010123
C     2007: 2007010100 - 2008010123
C     2008: 2008010100 - 200901012
C     2009: 2009010100 - 2010010123
C     2010: 2010010100 - 2011010123

C --- Get terrain ID for terrain elevation changes
C     2005: Three terrain elevation for 2005 RUC grib files
C     a. For periods before 2005062812: ruc20.dat
C     b. 2005062812 - 2005101813:       ruc20_2.dat
C     c. 2005101814 - 2005112219:       ruc20_3.dat
C     d. For periods after 2005101820:  ruc20_2.dat

C     2006: Two terrain elevation for 2006 RUC grib files
C     a: 2006010100 - 2006071113: Same as b/d in 2005 (ruc20_2.dat)
C     b: 2006071114 - 2007010123: Same as c in 2005   (ruc20_3.dat)

C     2007: One terrain elevation
C     Up to now: Same as b in 2006 (ruc20_3.dat)

C     2008: Two terrain elevation
C     a: 2008010100 - 2008061117: Same as b in 2006 (ruc20_3.dat)
C     b: 2008061118 - 2009010123: ruc20_4.dat

C     2009: One terrain elevation
C     a: 2009010100 - 2009010123: ruc20_4.dat (Same as 2008 b)

      character*120 infile
      call getdate(idate,iyr,imn,idy,ihr)
      infile='ruc20.dat'
      if(iyr.eq.2003) then
         ndate1=2003010100
         ndate2=2004010111
      elseif(iyr.eq.2004) then
         ndate1=2004010100
         ndate2=2005010123
      elseif(iyr.eq.2005) then
         if(idate.lt.2005062812) then  ! separate to four periods
            ndate1=2005010100
            ndate2=2005062811
         elseif(idate.lt.2005101814) then
            ndate1=2005062812
            ndate2=2005101813
            infile='ruc20_2.dat'
         elseif(idate.lt.2005112220) then
            ndate1=2005101814
            ndate2=2005112219
            infile='ruc20_3.dat'
         else
            ndate1=2005112220
            ndate2=2006010123
            infile='ruc20_2.dat'
         endif
      elseif(iyr.eq.2006) then
         if(idate.lt.2006071114) then  ! separate to two periods
            ndate1=2006010100
            ndate2=2006071113
            infile='ruc20_2.dat'
         elseif(idate.ge.2006071114) then
            ndate1=2006071114
            ndate2=2007010123
            infile='ruc20_3.dat'
         endif
      elseif(iyr.eq.2007) then
         ndate1=2007010100
         ndate2=2008010123
         infile='ruc20_3.dat'
      elseif(iyr.eq.2008) then
         if(idate.lt.2008061118) then  ! separate to two periods
            ndate1=2008010100
            ndate2=2008061117
            infile='ruc20_3.dat'
         elseif(idate.ge.2008061118) then
            ndate1=2008061118
            ndate2=2009010123
            infile='ruc20_4.dat'
         endif
      elseif(iyr.eq.2009) then
         ndate1=2009010100
         ndate2=2010010123
         infile='ruc20_4.dat'
      else
         write(ilg,*)' Error in getterid'
         write(ilg,*)' Not coded for this year: ',iyr 
         print *,' Error in getterid'
         print *,' Not coded for this year: ',iyr 
         stop
      endif

      write(ilg,101)ndate1,ndate2,trim(infile)
      print 101,ndate1,ndate2,trim(infile)
101   format(' Time period limit (beg/end): ',2i12,/,
     &     ' Terrain file: ',a)
 
      return
      end
